#ifndef CDK_H
#define CDK_H

/*
 * $Author: glover $
 * $Date: 1995/12/23 22:55:50 $
 * $Revision: 1.132 $
 */

#ifdef NCURSES
#include <ncurses/curses.h>
#else
#include <curses.h>
#endif
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <dirent.h>
#include <time.h>
#include <errno.h>
#include <pwd.h>
#include <grp.h>
#include <sys/stat.h>
#include <sys/types.h>
#include "curdefs.h"

/*
 * =========================================================
 * 	Delcare Global Generic Defines and Typedefs
 * =========================================================
 */
typedef enum	{
			vENTRY,
			vMENTRY,
			vLABEL,
			vSCROLL,
			vDIALOG,
			vSCALE,
			vMARQUEE,
			vMENU,
			vMATRIX,
			vHISTOGRAM,
			vSELECTION,
			vVIEWER,
			vGRAPH,
			vRADIO,
			vTEMPLATE,
			vSWINDOW,
			vITEMLIST,
			vFSELECT
		} EObjectType;
typedef enum	{
			vCHAR,
			vHCHAR,
			vINT,
			vHINT,
			vMIXED,
			vHMIXED,
			vUCHAR,
			vLCHAR,
			vUHCHAR,
			vLHCHAR,
			vUMIXED,
			vLMIXED,
			vUHMIXED,
			vLHMIXED,
			vVIEWONLY
		} EDisplayType;
typedef enum	{
			vNONE,
			vPERCENT,
			vFRACTION,
			vREAL
		} EHistogramDisplayType;
typedef enum	{
			vPLOT,
			vLINE
		} EGraphDisplayType;
typedef int boolean;

#define	LEFT		9000
#define	RIGHT		9001
#define	CENTER		9002
#define	TOP		9003
#define	BOTTOM		9004
#define	HORIZONTAL	9005
#define	VERTICAL	9006
#define	FULL		9007

#define NONE		0
#define ROW		1
#define COL		2

#define	BOX		1
#define	NOBOX		0

#define	DIFF(a,b)	(abs(a-b))
#define	MAX(a,b)	(a > b ? a : b)
#define	MIN(a,b)	(a < b ? a : b)
#define	HALF(a)		(a>>1)

/* Define the GLOBAL DEBUG FILEHANBDLE	*/
extern FILE	*CDKDEBUG;

/*
 * This is for the Cdk Screen object.
 */
#define MAXOBJECTS	30
struct _screen_st {
   WINDOW	*window;
   void		*object[MAXOBJECTS];
   EObjectType	cdktype[MAXOBJECTS];
   int		objectCount;
};
typedef struct _screen_st CDKSCREEN;

/*
 * This is for the key bindings.
 */
#define MAXBINDINGS	300
#define	MAXARGS		100
typedef void (*BINDFN) (EObjectType cdktype, void *object, void *clientData);

/*
 * =========================================================
 * 	Delcare Global Generic Prototypes
 * =========================================================
 */
void Beep();
void alignxy   (WINDOW *window,
		int *xpos, int *ypos,
		int boxWidth, int boxHeight);
void alignlabel (char *label, int lplace, int fieldwidth, int *boxWidth, int *boxHeight, int *labelxoffset, int *labelyoffset, int *fieldxoffset, int *fieldyoffset);
void drawScrollItem  (WINDOW *win, char *item, int woffset, int voffset, int hoffset, int width, chtype attr);
char *substring (char *string, int start, int width);
int justifyMessage (int fieldWidth, char *mesg, int justify);
int justifyString (int fieldWidth, int mesglen, int justify);
int readFile (char *filename, char **info, int maxlines);
void popUpMessage (CDKSCREEN *win, char **mesg, int count);
void quickSort (char *list[], int left, int right);
void swapIndex (char *list[], int i, int j);

/*
 * =========================================================
 * 	Delcare Debugging Routines.
 * =========================================================
 */
FILE *startCDKDebug(char *filename);
void writeCDKDebugMessage (FILE *fd, char *filename, char *function, int line, char *message);
void stopCDKDebug (FILE *fd);

/*
 * =========================================================
 * 	Delcare String Handling Routines.
 * =========================================================
 */
void freeChar (char *string);
void freeChtype (chtype *string);
void cleanChar (char *string, int length, char character);
void cleanChtype (chtype *string, int length, chtype character);
char *chtype2Char (chtype *string);
chtype *char2Chtype (char *string, int *length, int *align);
chtype *copyChtype (chtype *string);
char *copyChar (char *string);
int chlen (chtype *string);

/*
 * =========================================================
 * 	These functions are 'private' functions which
 * 	take care of screen objects and refreshing.
 * =========================================================
 */
CDKSCREEN *initCDKScreen (WINDOW *window);
void initCDKColor();
void registerCDKObject (CDKSCREEN *screen, EObjectType cdktype, void *object);
void unregisterCDKObject (EObjectType cdktype, void *object);
void raiseCDKObject (EObjectType cdktype, void *object);
void lowerCDKObject (EObjectType cdktype, void *object);
void refreshCDKScreen (CDKSCREEN *screen);
void eraseCDKScreen (CDKSCREEN *screen);
void destroyCDKScreen (CDKSCREEN *screen);
void debugCDKScreen (CDKSCREEN *screen, char *mesg);
void endCDK();

/*
 * =========================================================
 * 	These functions are 'private' functions which
 * 	take care of object key bindings.
 * =========================================================
 */
void bindCDKObject (EObjectType cdktype,
			void *object, chtype key, BINDFN function, void *data);
void unbindCDKObject (EObjectType cdktype, void *object, chtype key);
int checkCDKObjectBind (EObjectType cdktype, void *object, chtype key);
void cleanCDKObjectBindings (EObjectType cdktype, void *object);
int mapChtype ( chtype key );

/*
 * =========================================================
 * 	Declare Drawing routines.
 * =========================================================
 */
void boxWindow (WINDOW *window, chtype attr);
void drawLine (WINDOW *window,
		int startx, int starty, int endx, int endy, chtype line);
void writeChar (WINDOW *window,
		int xpos, int ypos, char *string,
		int align, int start, int len);
void writeCharAttrib (WINDOW *window,
		int xpos, int ypos, char *string,
		chtype attr, int align, int start, int len);
void writeChtype (WINDOW *window,
			int xpos, int ypos, chtype *string,
			int align, int start, int len);
void writeChtypeAttrib (WINDOW *window,
			int xpos, int ypos, chtype *string,
			chtype attr, int align, int start, int len);
void attrbox (WINDOW *window,
		chtype tlc, chtype trc,
		chtype blc, chtype brc,
		chtype hor, chtype vert,
		chtype type);
void drawShadow (WINDOW *window);

/* REMOVE ME LATER	*/
void printattr         (WINDOW *window, int xpos, int ypos, int align, chtype attr, char *mesg);

/*
 * =========================================================
 * 	Declare Label Field Prototypes and Defines
 * =========================================================
 */
#define	MAXLABELROWS	300
struct _label_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*info[MAXLABELROWS];
   int		infolen[MAXLABELROWS];
   int		infopos[MAXLABELROWS];
   int		width;
   int		xpos;
   int		ypos;
   int		rows;
   boolean	box;
   boolean	shadow;
   CDKSCREEN	*screen;
   int		screenIndex;
};
typedef struct _label_st CDKLABEL;
CDKLABEL *newCDKLabel (CDKSCREEN *screen, int xPos, int yPos,
			char **mesg, int rows,
			boolean box, boolean shadow);
void drawCDKLabel (CDKLABEL *label, boolean box);
void eraseCDKLabel (CDKLABEL *label);
void destroyCDKLabel (CDKLABEL *label);
char waitCDKLabel (CDKLABEL *label, char key);

/*
 * ===============================================================
 *	Declare Single-Line Entry Field Prototypes and Defines
 * ===============================================================
 */
struct _entry_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*labelwin;
   chtype	*label;
   int		labelpos;
   int		labellen;
   WINDOW	*field;
   chtype	fieldattr;
   int		fieldwidth;
   char		*info;
   int		totalwidth;
   int		screencol;
   int		leftchar;
   int		min;
   int		max;
   EDisplayType	disptype;
   boolean	box;
   boolean	shadow;
   chtype	filler;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
   void	*	callbackfn;
};
typedef struct _entry_st CDKENTRY;
typedef void (*ENTRYCB) (CDKENTRY *entry, chtype character);
CDKENTRY *newCDKEntry (CDKSCREEN *cdkscreen,
			int xpos, int ypos, int lpos,
			char *label, 
			chtype fieldAttrib, chtype filler,
			EDisplayType disptype,
			int fieldWidth, int min, int max,
			ENTRYCB X, boolean box, boolean shadow);
char *activateCDKEntry (CDKENTRY *entry);
void setCDKEntry (CDKENTRY *entry, char *value, int min, int max, boolean box);
void drawCDKEntry (CDKENTRY *entry, boolean box);
void eraseCDKEntry (CDKENTRY *entry);
void cleanCDKEntry (CDKENTRY *entry);
void destroyCDKEntry (CDKENTRY *entry);
void CDKEntryCallBack (CDKENTRY *entry, chtype character);
void drawCDKEntryField (CDKENTRY *entry);

/*
 * =============================================================
 *	Declare Muli-Line Entry Field Prototypes and Defines
 * =============================================================
 */
struct _mentry_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*labelwin;
   WINDOW	*field;
   chtype	*label;
   int		labellen;
   int		labelpos;
   chtype	fieldattr;
   int		fieldwidth;
   char		*info;
   int		totalwidth;
   int		rows;
   int		currentrow;
   int		currentcol;
   int		toprow;
   EDisplayType	disptype;
   int 		min;
   int 		logical;
   boolean	box;
   boolean	shadow;
   chtype	filler;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
   void	*	callbackfn;
};
typedef struct _mentry_st CDKMENTRY;
typedef	void (*MENTRYCB) (CDKMENTRY *mentry, chtype character);
CDKMENTRY *newCDKMentry (CDKSCREEN *cdkscreen,
				int xpos, int ypos, int lpos,
				char *label,
				chtype fieldattr, chtype filler,
				EDisplayType disptype,
				int fieldwidth, int fieldrows,
				int logicalrows, int min,
				MENTRYCB X, boolean box, boolean shadow);
char *activateCDKMentry (CDKMENTRY *entry);
void setCDKMentry (CDKMENTRY *mentry, char *value, int min, boolean box);
void drawCDKMentry (CDKMENTRY *mentry, boolean box);
void eraseCDKMentry (CDKMENTRY *mentry);
void cleanCDKMentry (CDKMENTRY *mentry);
void destroyCDKMentry (CDKMENTRY *mentry);
void redrawCDKMentry (CDKMENTRY *mentry);
void CDKMentryCallBack (CDKMENTRY *entry, chtype character);

/*
 * =========================================================
 * 	Declare Scrolling List Prototypes and Defines
 * =========================================================
 */
#define MAXITEMS	2000
#define MAXVIEWSIZE	25
struct _scroll_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*title;
   int		titlepos;
   int		titlelen;
   int		titleadj;
   chtype	*item[MAXITEMS];
   int		itempos[MAXITEMS];
   int		itemlen[MAXITEMS];
   int		maxTopItem;
   int		maxLeftChar;
   int		leftChar;
   int		lastItem;
   int		currentTop;
   int		currentItem;
   int		currentHigh;
   int		listSize;
   int		boxWidth;
   int		boxHeight;
   int		viewSize;
   boolean	box;
   boolean	shadow;
   chtype	titlehighlight;
   chtype	highlight;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
#define	NUMBERS		TRUE
#define	NONUMBERS	FALSE
typedef struct _scroll_st CDKSCROLL;
CDKSCROLL *newCDKScroll(CDKSCREEN *cdkscreen,
				int xpos, int ypos, int height, int width,
				char *title,
				char **scrollItems, int items, boolean numbers,
				chtype highlight, boolean box, boolean shadow);
int  activateCDKScroll (CDKSCROLL *scroll);
void setCDKScroll (CDKSCROLL *scroll, char **scrollItems, int listSize,
			boolean numbers, chtype highlight, boolean box);
void drawCDKScroll (CDKSCROLL *scroll, boolean box);
void eraseCDKScroll (CDKSCROLL *scroll);
void destroyCDKScroll (CDKSCROLL *scroll);
void createCDKScrollItemList (CDKSCROLL *scroll, boolean numbers,
				char **list, int listSize);

/*
 * =========================================================
 * 	Declare Dialog Box Prototypes and Defines
 * =========================================================
 */
#define	MAXDIALOGROWS		50
#define	MAXDIALOGBUTTONS	25
#define	MINDIALOGWIDTH		10
struct _dialogBox_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*info[MAXDIALOGROWS];
   int		infolen[MAXDIALOGROWS];
   int		infopos[MAXDIALOGROWS];
   chtype	*buttonlabel[MAXDIALOGBUTTONS];
   int		buttonlen[MAXDIALOGROWS];
   int		buttonpos[MAXDIALOGROWS];
   int		messageRows;
   int		buttonCount;
   int		currentButton;
   int		boxWidth;
   int		boxHeight;
   boolean	separator;
   boolean	box;
   boolean	shadow;
   chtype	highlight;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _dialogBox_st CDKDIALOG;
CDKDIALOG *newCDKDialog (CDKSCREEN *cdkscreen,
				int xPos, int yPos,
				char **message, int Rows,
				char **buttons, int buttonCount,
				chtype highlight,
				boolean separator, boolean box, boolean shadow);
int activateCDKDialog (CDKDIALOG *dialog);
void setCDKDialog (CDKDIALOG *dialog, chtype highlight,
			boolean separator, boolean box);
void drawCDKDialogButton (CDKDIALOG *dialog, int button,
				boolean box, chtype active, chtype highlight);
void drawCDKDialog (CDKDIALOG *dialog, boolean box);
void eraseCDKDialog (CDKDIALOG *dialog);
void destroyCDKDialog (CDKDIALOG *dialog);
void redrawCDKDialogButtons (CDKDIALOG *dialog);

/*
 * =========================================================
 * 	Declare Scale Box Prototypes and Defines
 * =========================================================
 */
struct _scale_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*fieldwin;
   WINDOW	*labelwin;
   chtype	*label;
   int		labellen;
   int		labelpos;
   int		fieldwidth;
   chtype	fieldattr;
   int		low;
   int		high;
   int		current;
   int		inc;
   int		fastinc;
   boolean	box;
   boolean	shadow;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _scale_st CDKSCALE;
CDKSCALE *newCDKScale (CDKSCREEN *cdkscreen,
			int xpos, int ypos, int lpos,
			char *label, chtype fieldattr, int fieldwidth,
			int start, int low, int high, int inc, int fastinc,
			boolean box, boolean shadow);
int activateCDKScale (CDKSCALE *scale);
void setCDKScale (CDKSCALE *scale, int low, int high, int value, boolean box);
void drawCDKScale (CDKSCALE *scale, boolean box);
void eraseCDKScale (CDKSCALE *scale);
void destroyCDKScale (CDKSCALE *scale);

/*
 * =========================================================
 * 	Declare Marquee Prototypes and Defines
 * =========================================================
 */
struct _marquee_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   int		active;
   int		width;
   CDKSCREEN	*screen;
   int		screenIndex;
   boolean	box;
   boolean	shadow;
};
typedef struct _marquee_st CDKMARQUEE;
CDKMARQUEE *newCDKMarquee (CDKSCREEN *cdkscreen, int xpos, int ypos,
				int width, boolean shadow);
int activateCDKMarquee (CDKMARQUEE *marquee, char *message,
				int delay, int repeat, boolean box);
void drawCDKMarquee (CDKMARQUEE *marquee, boolean box);
void eraseCDKMarquee (CDKMARQUEE *marquee);
void destroyCDKMarquee (CDKMARQUEE *marquee);

/*
 * =========================================================
 * 	Declare Menu Prototypes and Defines
 * =========================================================
 */
#define MAXMENUITEMS	30
#define MAXSUBITEMS	98
struct _menu_st {
   WINDOW	*parent;
   WINDOW	*pullwin[MAXMENUITEMS];
   WINDOW	*titlewin[MAXMENUITEMS];
   chtype	*title[MAXMENUITEMS];
   int		titlelen[MAXMENUITEMS];
   chtype	*sublist[MAXMENUITEMS][MAXSUBITEMS];
   int		sublistlen[MAXMENUITEMS][MAXSUBITEMS];
   int		subsize[MAXMENUITEMS];
   int		menuitems;
   chtype	titleattr;
   chtype	subtitleattr;
   int		currentTitle;
   int		currentSubtitle;
   int		lastTitle;
   int		lastSubtitle;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _menu_st CDKMENU;
CDKMENU *newCDKMenu (CDKSCREEN *cdkscreen,
			char *menulist[MAXMENUITEMS][MAXSUBITEMS],
			int menuitems,
			int *subsize,
			int *menuloc,
			chtype titleattr,
			chtype subtitleattr);
int activateCDKMenu (CDKMENU *menu);
void setCDKMenu (CDKMENU *menu,
			int menuitem, int submenuitem,
			chtype titlehighlight, chtype subtitlehighlight);
void drawCDKMenu (CDKMENU *menu);
void drawCDKMenuTitles (CDKMENU *menu);
void drawCDKMenuSubwin (CDKMENU *menu);
void eraseCDKMenu (CDKMENU *menu);
void eraseCDKMenuSubwin (CDKMENU *menu);
void destroyCDKMenu (CDKMENU *menu);
void cleanUpMenu (CDKMENU *menu);

/*
 * =========================================================
 * 	Declare Matrix Prototypes and Defines
 * =========================================================
 */
#define	MAXMATRIXROWS	1000
#define	MAXMATRIXCOLS	1000
struct _matrix_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*cell[MAXMATRIXROWS][MAXMATRIXCOLS];
   char		*info[MAXMATRIXROWS][MAXMATRIXCOLS];
   int		rows;
   int		cols;
   int		vrows;
   int		vcols;
   int		colwidths[MAXMATRIXCOLS];
   int		colvalues[MAXMATRIXCOLS];
   chtype	*coltitle[MAXMATRIXCOLS];
   int		coltitlelen[MAXMATRIXROWS];
   int		coltitlepos[MAXMATRIXROWS];
   int		maxct;
   chtype	*rowtitle[MAXMATRIXROWS];
   int		rowtitlelen[MAXMATRIXROWS];
   int		rowtitlepos[MAXMATRIXROWS];
   int		maxrt;
   int		rowspace;
   int		colspace;
   int		row;
   int		col;
   int		crow;
   int		ccol;
   int		trow;
   int		lcol;
   int		oldcrow;
   int		oldccol;
   int		oldvrow;
   int		oldvcol;
   boolean	boxMatrix;
   boolean	boxCell;
   boolean	shadow;
   chtype	highlight;
   int		dominant;
   chtype	filler;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
   void	*	callbackfn;
};
typedef struct _matrix_st CDKMATRIX;
typedef void (*MATRIXCB) (CDKMATRIX *matrix, chtype input);
CDKMATRIX *newCDKMatrix (CDKSCREEN *cdkscreen,
				int xpos, int ypos,
				int rows, int cols,
				int vrows, int vcols,
				char **rowtitles,
				char **coltitles,
				int *colwidths, int *coltypes,
				int rowspace, int colspace, chtype filler,
				int dominantAttrib, MATRIXCB X,
				boolean boxMatrix, boolean boxCell,
				boolean shadow);
void activateCDKMatrix (CDKMATRIX *matrix);
void setCDKMatrix (CDKMATRIX *matrix,
			char *info[MAXMATRIXROWS][MAXMATRIXCOLS],
			int rows, int *subSize);
void drawCDKMatrix (CDKMATRIX *matrix, boolean box);
void eraseCDKMatrix (CDKMATRIX *matrix);
void cleanCDKMatrix (CDKMATRIX *matrix);
int moveToCDKMatrixCell (CDKMATRIX *matrix, int newrow, int newcol);
void destroyCDKMatrix (CDKMATRIX *matrix);
void drawCDKMatrixCell (CDKMATRIX *matrix,
			int srow, int scol, int vrow, int vcol,
			chtype attr, boolean box);
void CDKMatrixCallBack (CDKMATRIX *matrix, chtype input);
void highlightCDKMatrixCell (CDKMATRIX *matrix);
int jumpToCell (CDKMATRIX *matrix);

/*
 * =========================================================
 * 	Declare Histogram Prototypes and Defines
 * =========================================================
 */
struct _histogram_st {
   WINDOW			*parent;
   WINDOW			*win;
   WINDOW			*shadowwin;
   WINDOW 			*labelwin;
   chtype			*label;
   int				labellen;
   char				*statschar;
   char				*lowchar;
   char				*highchar;
   chtype			filler;
   chtype			labelattr;
   float			percent;
   int				height;
   int				width;
   int 				orient;
   int				statspos;
   int				statsattr;
   EHistogramDisplayType	viewtype;
   int				high;
   int				low;
   int				value;
   int				chars;
   int				lowx,	lowy;
   int				highx,	highy;
   int				labelx,	labely;
   int				statsx,	statsy;
   int				histx,	histy;
   boolean			box;
   boolean			shadow;
   CDKSCREEN			*screen;
   int				screenIndex;
};
typedef struct _histogram_st CDKHISTOGRAM;
CDKHISTOGRAM *newCDKHistogram (CDKSCREEN *cdkscreen,
				int xpos, int ypos, int lpos,
				int height, int width, int orient,
				char *label, boolean box, boolean shadow);
void setCDKHistogram (CDKHISTOGRAM *histogram,
			EHistogramDisplayType viewtype,
			int statspos, chtype statsattr,
			int low, int high, int value,
			chtype filler, boolean box);
void drawCDKHistogram (CDKHISTOGRAM *histogram, boolean box);
void eraseCDKHistogram (CDKHISTOGRAM *histogram);
void destroyCDKHistogram (CDKHISTOGRAM *histogram);

/*
 * =========================================================
 * 	Declare Selection List Prototypes and Defines
 * =========================================================
 */
#define	MAXCHOICES	100
struct _selection_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*title;
   int		titlepos;
   int		titlelen;
   int		titleadj;
   chtype	*item[MAXITEMS];
   int		itemlen[MAXITEMS];
   int		itempos[MAXITEMS];
   chtype	*choice[MAXCHOICES];
   int		choicelen[MAXCHOICES];
   int		choiceCount;
   int		maxchoicelen;
   int		selections[MAXITEMS];
   int		maxTopItem;
   int		maxLeftChar;
   int		leftChar;
   int		lastItem;
   int		currentTop;
   int		currentHigh;
   int		currentItem;
   int		listSize;
   int		boxWidth;
   int		boxHeight;
   int		viewSize;
   boolean	box;
   boolean	shadow;
   chtype	highlight;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _selection_st CDKSELECTION;
CDKSELECTION *newCDKSelection (CDKSCREEN *cdkscreen,
				int xpos, int ypos, int height, int width,
				char *title,
				char **list, int listSize,
				char **choices, int choiceSize,
				chtype highlight,
				boolean box, boolean shadow);
void activateCDKSelection (CDKSELECTION *selection);
void drawCDKSelection (CDKSELECTION *selection, boolean box);
void eraseCDKSelection (CDKSELECTION *selection);
void destroyCDKSelection (CDKSELECTION *selection);
void drawCDKSelectionList (CDKSELECTION *selection);
void setCDKSelection (CDKSELECTION *selection, chtype highlight,
			int *defChoices, boolean box);

/*
 * =========================================================
 * 	Declare Viewer List Prototypes and Defines
 * =========================================================
 */
#define	MAXLINES	10000
#define	MAXBUTTONS	50
struct _viewer_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*button[MAXBUTTONS];
   int		buttonlen[MAXBUTTONS];
   int		buttonpos[MAXBUTTONS];
   int		buttonCount;
   chtype	buttonHighlight;
   chtype	*info[MAXLINES];
   int		infolen[MAXLINES];
   int		infopos[MAXLINES];
   chtype	*title;
   int		titlelen;
   int		titlepos;
   int		infoSize;
   int		boxHeight;
   int		boxWidth;
   int		viewSize;
   int		currentTop;
   int		currentButton;
   int		leftChar;
   int		length;
   int		maxLeftChar;
   int		maxTopLine;
   int		characters;
   boolean	interpret;
   boolean	box;
   boolean	shadow;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _viewer_st CDKVIEWER;
CDKVIEWER *newCDKViewer (CDKSCREEN *cdkscreen,
				int xpos, int ypos,
				int height, int width,
				char **buttons, int buttonCount,
				boolean box, boolean shadow);
int activateCDKViewer (CDKVIEWER *viewer,
			char *title, 
			char **info, int infoSize,
			chtype buttonHighlight,
			boolean interpret, boolean box);
int manageCDKViewer (CDKVIEWER *viewer);
void setCDKViewer (CDKVIEWER *viewer, chtype buttonHighlight, boolean box);
void drawCDKViewer (CDKVIEWER *viewer, int boolean);
void eraseCDKViewer (CDKVIEWER *viewer);
void destroyCDKViewer (CDKVIEWER *viewer);

/*
 * =========================================================
 * 	Declare Graph Prototypes and Defines
 * =========================================================
 */
struct _graph_st {
   WINDOW		*parent;
   WINDOW		*win;
   WINDOW		*shadowwin;
   chtype		*title;
   chtype		*graphchar;
   int			titlepos;
   int			titlelen;
   boolean		box;
   boolean		shadow;
   int			boxHeight;
   int			boxWidth;
   chtype		*xtitle;
   int			xtitlepos;
   int			xtitlelen;
   int			values[MAXLINES];
   int			count;
   int			minx;
   int			maxx;
   int			xscale;
   int			txpos;
   chtype		*ytitle;
   int			ytitlepos;
   int			ytitlelen;
   int			yscale;
   int			typos;
   EGraphDisplayType	displayType;
   CDKSCREEN		*screen;
   int			screenIndex;
};
typedef struct _graph_st CDKGRAPH;
CDKGRAPH *newCDKGraph (CDKSCREEN *cdkscreen,
			int xpos, int ypos, int height, int width,
			char *title, char *xtitle, char *ytitle);
int setCDKGraph (CDKGRAPH *graph, 
			int *values, int count, char *graphchar,
			boolean startAtZero,
			EGraphDisplayType displayType);
void drawCDKGraph (CDKGRAPH *graph, boolean box);
void eraseCDKGraph (CDKGRAPH *graph);
void destroyCDKGraph (CDKGRAPH *graph);

/*
 * =========================================================
 * 	Declare Radio Button Prototypes and Defines
 * =========================================================
 */
struct _radio_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*title;
   int		titlelen;
   int		titlepos;
   chtype	*item[MAXITEMS];
   int		itemlen[MAXITEMS];
   int		itempos[MAXITEMS];
   int		titleadj;
   chtype	choiceChar;
   int		maxLeftChar;
   int		widestItem;
   int		leftChar;
   int		selectedItem;
   int		currentTop;
   int		currentItem;
   int		currentHigh;
   int		listSize;
   int		lastItem;
   int		maxTopItem;
   int		boxWidth;
   int		boxHeight;
   int		viewSize;
   int		defItem;
   boolean	box;
   boolean	shadow;
   chtype	highlight;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _radio_st CDKRADIO;
CDKRADIO *newCDKRadio (CDKSCREEN *cdkscreen,
			int xpos, int ypos, int height, int width,
			char *title, char **mesg, int items,
			chtype choiceChar, int defItem,
			chtype highlight, boolean box, boolean shadow);
int activateCDKRadio (CDKRADIO *radio);
void setCDKRadio (CDKRADIO *radio, chtype highlight,
			chtype choiceChar, boolean box);
void drawCDKRadio (CDKRADIO *radio, boolean box);
void eraseCDKRadio (CDKRADIO *radio);
void destroyCDKRadio (CDKRADIO *radio);

/*
 * =========================================================
 * 	       Declare Template Object....
 * =========================================================
 */
struct _template_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*labelwin;
   chtype	*label;
   chtype	*overlay;
   chtype	overlayattr;
   char 	*plate;
   char		*info;
   int		labellen;
   int		overlaylen;
   int		labelpos;
   int		fieldwidth;
   int		platePos;
   int		screenPos;
   int		infoPos;
   chtype	labelattr;
   chtype	fieldattr;
   WINDOW	*field;
   CDKSCREEN	*screen;
   int		screenIndex;
   boolean	box;
   boolean	shadow;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
   void	*	callbackfn;
};
typedef struct _template_st CDKTEMPLATE;
typedef void (*TEMPLATECB) (CDKTEMPLATE *template, chtype input);
CDKTEMPLATE *newCDKTemplate (CDKSCREEN *cdkscreen,
				int xpos, int ypos, int lpos,
				char *label, char *plate, char *overlay,
				TEMPLATECB X,
				boolean box, boolean shadow);
char *activateCDKTemplate (CDKTEMPLATE *template);
void setCDKTemplate (CDKTEMPLATE *template, char *value, boolean box);
void drawCDKTemplate (CDKTEMPLATE *template, boolean box);
void eraseCDKTemplate (CDKTEMPLATE *template);
void cleanCDKTemplate (CDKTEMPLATE *template);
char *mixCDKTemplate (CDKTEMPLATE *template);
char *unmixCDKTemplate (CDKTEMPLATE *template, char *string);
void destroyCDKTemplate (CDKTEMPLATE *template);
void CDKTemplateCallBack (CDKTEMPLATE *template, chtype input);

/*
 * =========================================================
 * 	       Declare Scrolling Window Object....
 * =========================================================
 */
struct _swindow_st {
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*info[MAXLINES];
   int		infopos[MAXLINES];
   int		infolen[MAXLINES];
   chtype	*title;
   int		titlepos;
   int		titlelen;
   int		titleadj;
   int		itemCount;
   int		boxHeight;
   int		boxWidth;
   int		viewSize;
   int		maxTopLine;
   int		currentTop;
   int		leftChar;
   int		maxLeftChar;
   int		widestLine;
   int		saveLines;
   boolean	box;
   boolean	shadow;
   CDKSCREEN	*screen;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _swindow_st CDKSWINDOW;
typedef void (*SWINDOWCB) (CDKSWINDOW *swindow, chtype input);
CDKSWINDOW *newCDKSwindow (CDKSCREEN *cdkscreen,
				int xpos, int ypos, int height, int width,
				char *title,
				int saveLines,
				boolean box, boolean shadow);
void activateCDKSwindow	(CDKSWINDOW *swindow);
void setCDKSwindow (CDKSWINDOW *swindow, char **info,
			int lines, boolean box);
void getCDKSwindow (CDKSWINDOW *swindow, char **info);
void drawCDKSwindow (CDKSWINDOW *swindow, boolean box);
void eraseCDKSwindow (CDKSWINDOW *swindow);
void cleanCDKSwindow (CDKSWINDOW *swindow);
void addCDKSwindow (CDKSWINDOW *swindow, char *info, int insertPos);
void trimCDKSwindow (CDKSWINDOW *swindow, int start, int finish);
void destroyCDKSwindow (CDKSWINDOW *swindow);

/*
 * =========================================================
 * 	       Declare Item List Widget
 * =========================================================
 */
struct _itemlist_st {
   CDKSCREEN	*screen;
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   WINDOW	*labelwin;
   chtype	*label;
   int		labelpos;
   int		labellen;
   WINDOW	*fieldwin;
   int		fieldWidth;
   chtype	*item[MAXITEMS];
   int		itempos[MAXITEMS];
   int		itemlen[MAXITEMS];
   int		itemCount;
   int		currentItem;
   int		defaultItem;
   boolean	box;
   boolean	shadow;
   int		screenIndex;
   BINDFN	bindFunction[MAXBINDINGS];
   void *	bindData[MAXBINDINGS];
};
typedef struct _itemlist_st CDKITEMLIST;
CDKITEMLIST *newCDKItemlist (CDKSCREEN *cdkscreen,
				int xpos, int ypos,
				int lpos, char *label,
				char **itemlist, int count, int defaultItem,
				boolean box, boolean shadow);
int activateCDKItemlist (CDKITEMLIST *itemlist);
void setCDKItemlist (CDKITEMLIST *itemlist, char **list, int count,
			int current, boolean box);
void drawCDKItemlist (CDKITEMLIST *itemlist, boolean box);
void drawCDKItemlistField (CDKITEMLIST *itemlist);
void eraseCDKItemlist (CDKITEMLIST *itemlist);
void destroyCDKItemlist	(CDKITEMLIST *itemlist);

/*
 * =========================================================
 *             Declare File Selector Widget
 * =========================================================
 */
struct _fileselector_st {
   CDKSCREEN	*screen;
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   CDKLABEL	*title;
   CDKENTRY	*filename;
   CDKSCROLL	*filelist;
   char		*dirContents[MAXITEMS];
   int		fileCounter;
   char 	*pwd;
   char		*pathname;
   int		xpos;
   int		ypos;
   int		height;
   int		width;
   chtype	fieldAttribute;
   chtype	fillerCharacter;
   chtype	highlight;
   char		*dirAttribute;
   char		*fileAttribute;
   char		*linkAttribute;
   char		*sockAttribute;
   boolean	box;
   boolean	shadow;
   int		screenIndex;
};
typedef struct _fileselector_st CDKFSELECT;
CDKFSELECT *newCDKFselect (CDKSCREEN *cdkscreen,
				int xpos, int ypos,
				int height, int width, 
				chtype fieldAttribute, chtype fillerChar,
				chtype highlight,
				char *dirAttributes, char *fileAttributes,
				char *linkAttribute, char *sockAttribute,
				boolean box, boolean shadow);
char *activateCDKFselect (CDKFSELECT *fselect);
void setCDKFselect (CDKFSELECT *fselect,
			char *directory,
			chtype fieldAttribute, chtype fillerChar,
			chtype highlight,
			char *dirAttribute, char *fileAttribute,
			char *linkAttribute, char *sockAttribute,
			boolean box);
void drawCDKFselect (CDKFSELECT *fselect, boolean box);
void eraseCDKFselect (CDKFSELECT *fselect);
void destroyCDKFselect (CDKFSELECT *fselect);

/*
 * =========================================================
 *             Declare Canvas Widget
 * =========================================================
 */
#define	MAXCANVASHEIGHT	300
struct _canvas_st {
   CDKSCREEN	*screen;
   WINDOW	*parent;
   WINDOW	*win;
   WINDOW	*shadowwin;
   chtype	*characters[MAXCANVASHEIGHT];
   boolean	box;
   boolean	shadow;
   int		screenIndex;
};
typedef struct _canvas_st CDKCANVAS;
CDKCANVAS *newCDKCanvas (CDKSCREEN *cdkscreen,
				int xpos, int ypos,
				int height, int width, 
				boolean box, boolean shadow);
void drawCDKCanvas (CDKCANVAS *canvas, boolean box);
void drawlineCDKCanvas (CDKCANVAS *canvas,
			int startXpos, int startYpos,
			int enxXpos, int endYpos);
void drawboxCDKCanvas (CDKCANVAS *canvas,
			int leftXpos, int leftYpos,
			int height, int width);
void destroyCDKCanvas (CDKCANVAS *canvas);
#endif
