#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 18:38:39 $
 * $Revision: 1.33 $
 */

/*
 * This creates a new CDK screen.
 */
CDKSCREEN *initCDKScreen(WINDOW *window)
{
   CDKSCREEN	*screen		= (CDKSCREEN *)malloc (sizeof (CDKSCREEN));
   screen->objectCount		= 0;
   screen->window		= window;

   /* Set up basic curses settings.		*/
   noecho();
   cbreak();

   /* OK, we are done.				*/
   return (screen);
}

/*
 * This registers a CDK object with a screen.
 */
void registerCDKObject (CDKSCREEN *screen, EObjectType cdktype, void *object)
{
   /* Set some basic vars.						*/
   int objectNumber				= screen->objectCount;
   screen->object[objectNumber]			= (void *)object;

   /* Since dereferencing a void pointer is a no-no, we have to cast	*/
   /* our pointer correctly.						*/
   if (cdktype == vLABEL)
   {
      ((CDKLABEL *)object)->screenIndex		= objectNumber;
      ((CDKLABEL *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vLABEL;
   }
   else if (cdktype == vENTRY)
   {
      ((CDKENTRY *)object)->screenIndex		= objectNumber;
      ((CDKENTRY *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vENTRY;
   }
   else if (cdktype == vMENTRY)
   {
      ((CDKMENTRY *)object)->screenIndex	= objectNumber;
      ((CDKMENTRY *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vMENTRY;
   }
   else if (cdktype == vSCROLL)
   {
      ((CDKSCROLL *)object)->screenIndex	= objectNumber;
      ((CDKSCROLL *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vSCROLL;
   }
   else if (cdktype == vDIALOG)
   {
      ((CDKDIALOG *)object)->screenIndex	= objectNumber;
      ((CDKDIALOG *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vDIALOG;
   }
   else if (cdktype == vSCALE)
   {
      ((CDKSCALE *)object)->screenIndex		= objectNumber;
      ((CDKSCALE *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vSCALE;
   }
   else if (cdktype == vMARQUEE)
   {
      ((CDKMARQUEE *)object)->screenIndex	= objectNumber;
      ((CDKMARQUEE *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vMARQUEE;
   }
   else if (cdktype == vMENU)
   {
      ((CDKMENU *)object)->screenIndex		= objectNumber;
      ((CDKMENU *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vMENU;
   }
   else if (cdktype == vMATRIX)
   {
      ((CDKMATRIX *)object)->screenIndex	= objectNumber;
      ((CDKMATRIX *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vMATRIX;
   }
   else if (cdktype == vHISTOGRAM)
   {
      ((CDKHISTOGRAM *)object)->screenIndex	= objectNumber;
      ((CDKHISTOGRAM *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vHISTOGRAM;
   }
   else if (cdktype == vSELECTION)
   {
      ((CDKSELECTION *)object)->screenIndex	= objectNumber;
      ((CDKSELECTION *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vSELECTION;
   }
   else if (cdktype == vVIEWER)
   {
      ((CDKVIEWER *)object)->screenIndex	= objectNumber;
      ((CDKVIEWER *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vVIEWER;
   }
   else if (cdktype == vGRAPH)
   {
      ((CDKGRAPH *)object)->screenIndex		= objectNumber;
      ((CDKGRAPH *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vGRAPH;
   }
   else if (cdktype == vRADIO)
   {
      ((CDKRADIO *)object)->screenIndex		= objectNumber;
      ((CDKRADIO *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vRADIO;
   }
   else if (cdktype == vTEMPLATE)
   {
      ((CDKTEMPLATE *)object)->screenIndex	= objectNumber;
      ((CDKTEMPLATE *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vTEMPLATE;
   }
   else if (cdktype == vSWINDOW)
   {
      ((CDKSWINDOW *)object)->screenIndex	= objectNumber;
      ((CDKSWINDOW *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vSWINDOW;
   }
   else if (cdktype == vITEMLIST)
   {
      ((CDKITEMLIST *)object)->screenIndex	= objectNumber;
      ((CDKITEMLIST *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vITEMLIST;
   }
   else if (cdktype == vFSELECT)
   {
      ((CDKFSELECT *)object)->screenIndex	= objectNumber;
      ((CDKFSELECT *)object)->screen		= screen;
      screen->cdktype[objectNumber]		= vFSELECT;
   }

   /* Don't forget to inc the count.					*/
   screen->objectCount++;
}

/*
 * This removes an object from the CDK screen.
 */
void unregisterCDKObject (EObjectType cdktype, void *object)
{
   /* Declare some vars.						*/
   CDKSCREEN *screen;
   char *type;
   int screenSize, index, x;

   /* Since dereferencing a void pointer is a no-no, we have to cast	*/
   /* our pointer correctly.						*/
   if (cdktype == vLABEL)
   {
      screen		= ((CDKLABEL *)object)->screen;
      index		= ((CDKLABEL *)object)->screenIndex;
      type		= "Label";
   }
   else if (cdktype == vENTRY)
   {
      screen		= ((CDKENTRY *)object)->screen;
      index		= ((CDKENTRY *)object)->screenIndex;
      type		= "Entry";
   }
   else if (cdktype == vMENTRY)
   {
      screen		= ((CDKMENTRY *)object)->screen;
      index		= ((CDKMENTRY *)object)->screenIndex;
      type		= "MEntry";
   }
   else if (cdktype == vSCROLL)
   {
      screen		= ((CDKSCROLL *)object)->screen;
      index		= ((CDKSCROLL *)object)->screenIndex;
      type		= "Scroll";
   }
   else if (cdktype == vDIALOG)
   {
      screen		= ((CDKDIALOG *)object)->screen;
      index		= ((CDKDIALOG *)object)->screenIndex;
      type		= "Dialog";
   }
   else if (cdktype == vSCALE)
   {
      screen		= ((CDKSCALE *)object)->screen;
      index		= ((CDKSCALE *)object)->screenIndex;
      type		= "Scale";
   }
   else if (cdktype == vMARQUEE)
   {
      screen		= ((CDKMARQUEE *)object)->screen;
      index		= ((CDKMARQUEE *)object)->screenIndex;
      type		= "Marquee";
   }
   else if (cdktype == vMENU)
   {
      screen		= ((CDKMENU *)object)->screen;
      index		= ((CDKMENU *)object)->screenIndex;
      type		= "Menu";
   }
   else if (cdktype == vMATRIX)
   {
      screen		= ((CDKMATRIX *)object)->screen;
      index		= ((CDKMATRIX *)object)->screenIndex;
      type		= "Matrix";
   }
   else if (cdktype == vHISTOGRAM)
   {
      screen		= ((CDKHISTOGRAM *)object)->screen;
      index		= ((CDKHISTOGRAM *)object)->screenIndex;
      type		= "Histogram";
   }
   else if (cdktype == vSELECTION)
   {
      screen		= ((CDKSELECTION *)object)->screen;
      index		= ((CDKSELECTION *)object)->screenIndex;
      type		= "Selection";
   }
   else if (cdktype == vVIEWER)
   {
      screen		= ((CDKVIEWER *)object)->screen;
      index		= ((CDKVIEWER *)object)->screenIndex;
      type		= "Viewer";
   }
   else if (cdktype == vGRAPH)
   {
      screen		= ((CDKGRAPH *)object)->screen;
      index		= ((CDKGRAPH *)object)->screenIndex;
      type		= "Graph";
   }
   else if (cdktype == vRADIO)
   {
      screen		= ((CDKRADIO *)object)->screen;
      index		= ((CDKRADIO *)object)->screenIndex;
      type		= "Radio";
   }
   else if (cdktype == vTEMPLATE)
   {
      screen		= ((CDKTEMPLATE *)object)->screen;
      index		= ((CDKTEMPLATE *)object)->screenIndex;
      type		= "Template";
   }
   else if (cdktype == vSWINDOW)
   {
      screen		= ((CDKSWINDOW *)object)->screen;
      index		= ((CDKSWINDOW *)object)->screenIndex;
      type		= "SWindow";
   }
   else if (cdktype == vITEMLIST)
   {
      screen		= ((CDKITEMLIST *)object)->screen;
      index		= ((CDKITEMLIST *)object)->screenIndex;
      type		= "Itemlist";
   }
   else if (cdktype == vFSELECT)
   {
      screen		= ((CDKFSELECT *)object)->screen;
      index		= ((CDKFSELECT *)object)->screenIndex;
      type		= "File Selector";
   }

   /* Set the screenSize.		*/
   screenSize	= screen->objectCount - 1;

   /* If this is the last object -1 then this is the last. If not	*/
   /* we have to shuffle all the other objects to the left.		*/
   x = index;
   for (x=index; x < screenSize; x++)
   {
      screen->object[x]		= screen->object[x+1];
      screen->cdktype[x]	= screen->cdktype[x+1];
   }
   screen->object[x]		= (void *)NULL;
   screen->cdktype[x]		= 0;
   screen->objectCount--;
}

/*
 * This 'brings' a CDK object to the top of the stack.
 */
void raiseCDKObject (EObjectType cdktype, void *object)
{
   /* Declare some vars.						*/
   void *swapobject	= (void *)NULL;
   int toppos		= -1;
   int swapindex	= -1;
   EObjectType swaptype;

   /* Should never dereference a void * pointer.			*/
   if (cdktype == vLABEL)
   {
      /* Save the old info...						*/
      toppos		= ((CDKLABEL *)object)->screen->objectCount - 1;
      swapobject	= (void *)((CDKLABEL *)object)->screen->object[toppos];
      swaptype		= ((CDKLABEL *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKLABEL *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKLABEL *)object)->screenIndex			= toppos;
      ((CDKLABEL *)object)->screen->object[toppos]	= (void *)object;
      ((CDKLABEL *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vENTRY)
   {
      /* Save the old info...						*/
      toppos		= ((CDKENTRY *)object)->screen->objectCount;
      swapobject	= ((CDKENTRY *)object)->screen->object[toppos];
      swaptype		= ((CDKENTRY *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKENTRY *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKENTRY *)object)->screenIndex			= toppos;
      ((CDKENTRY *)object)->screen->object[toppos]	= (void *)object;
      ((CDKENTRY *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMENTRY)
   {
      /* Save the old info...						*/
      toppos		= ((CDKMENTRY *)object)->screen->objectCount;
      swapobject	= ((CDKMENTRY *)object)->screen->object[toppos];
      swaptype		= ((CDKMENTRY *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMENTRY *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMENTRY *)object)->screenIndex		= toppos;
      ((CDKMENTRY *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMENTRY *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSCROLL)
   {
      /* Save the old info...						*/
      toppos		= ((CDKSCROLL *)object)->screen->objectCount;
      swapobject	= ((CDKSCROLL *)object)->screen->object[toppos];
      swaptype		= ((CDKSCROLL *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSCROLL *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSCROLL *)object)->screenIndex		= toppos;
      ((CDKSCROLL *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSCROLL *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vDIALOG)
   {
      /* Save the old info...						*/
      toppos		= ((CDKDIALOG *)object)->screen->objectCount;
      swapobject	= ((CDKDIALOG *)object)->screen->object[toppos];
      swaptype		= ((CDKDIALOG *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKDIALOG *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKDIALOG *)object)->screenIndex		= toppos;
      ((CDKDIALOG *)object)->screen->object[toppos]	= (void *)object;
      ((CDKDIALOG *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSCALE)
   {
      /* Save the old info...						*/
      toppos		= ((CDKSCALE *)object)->screen->objectCount;
      swapobject	= ((CDKSCALE *)object)->screen->object[toppos];
      swaptype		= ((CDKSCALE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSCALE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSCALE *)object)->screenIndex			= toppos;
      ((CDKSCALE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSCALE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMARQUEE)
   {
      /* Save the old info...						*/
      toppos		= ((CDKMARQUEE *)object)->screen->objectCount;
      swapobject	= ((CDKMARQUEE *)object)->screen->object[toppos];
      swaptype		= ((CDKMARQUEE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMARQUEE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMARQUEE *)object)->screenIndex		= toppos;
      ((CDKMARQUEE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMARQUEE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMENU)
   {
      /* Save the old info...						*/
      toppos		= ((CDKMENU *)object)->screen->objectCount;
      swapobject	= ((CDKMENU *)object)->screen->object[toppos];
      swaptype		= ((CDKMENU *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMENU *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMENU *)object)->screenIndex			= toppos;
      ((CDKMENU *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMENU *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMATRIX)
   {
      /* Save the old info...						*/
      toppos		= ((CDKMATRIX *)object)->screen->objectCount;
      swapobject	= ((CDKMATRIX *)object)->screen->object[toppos];
      swaptype		= ((CDKMATRIX *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMATRIX *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMATRIX *)object)->screenIndex		= toppos;
      ((CDKMATRIX *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMATRIX *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vHISTOGRAM)
   {
      /* Save the old info...						*/
      toppos		= ((CDKHISTOGRAM *)object)->screen->objectCount;
      swapobject	= ((CDKHISTOGRAM *)object)->screen->object[toppos];
      swaptype		= ((CDKHISTOGRAM *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKHISTOGRAM *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKHISTOGRAM *)object)->screenIndex		= toppos;
      ((CDKHISTOGRAM *)object)->screen->object[toppos]	= (void *)object;
      ((CDKHISTOGRAM *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSELECTION)
   {
      /* Save the old info...						*/
      toppos		= ((CDKSELECTION *)object)->screen->objectCount;
      swapobject	= ((CDKSELECTION *)object)->screen->object[toppos];
      swaptype		= ((CDKSELECTION *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSELECTION *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSELECTION *)object)->screenIndex		= toppos;
      ((CDKSELECTION *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSELECTION *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vVIEWER)
   {
      /* Save the old info...						*/
      toppos		= ((CDKVIEWER *)object)->screen->objectCount;
      swapobject	= ((CDKVIEWER *)object)->screen->object[toppos];
      swaptype		= ((CDKVIEWER *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKVIEWER *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKVIEWER *)object)->screenIndex		= toppos;
      ((CDKVIEWER *)object)->screen->object[toppos]	= (void *)object;
      ((CDKVIEWER *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vGRAPH)
   {
      /* Save the old info...						*/
      toppos		= ((CDKGRAPH *)object)->screen->objectCount;
      swapobject	= ((CDKGRAPH *)object)->screen->object[toppos];
      swaptype		= ((CDKGRAPH *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKGRAPH *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKGRAPH *)object)->screenIndex			= toppos;
      ((CDKGRAPH *)object)->screen->object[toppos]	= (void *)object;
      ((CDKGRAPH *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vRADIO)
   {
      /* Save the old info...						*/
      toppos		= ((CDKRADIO *)object)->screen->objectCount;
      swapobject	= ((CDKRADIO *)object)->screen->object[toppos];
      swaptype		= ((CDKRADIO *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKRADIO *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKRADIO *)object)->screenIndex			= toppos;
      ((CDKRADIO *)object)->screen->object[toppos]	= (void *)object;
      ((CDKRADIO *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vTEMPLATE)
   {
      /* Save the old info...						*/
      toppos		= ((CDKTEMPLATE *)object)->screen->objectCount;
      swapobject	= ((CDKTEMPLATE *)object)->screen->object[toppos];
      swaptype		= ((CDKTEMPLATE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKTEMPLATE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKTEMPLATE *)object)->screenIndex		= toppos;
      ((CDKTEMPLATE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKTEMPLATE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSWINDOW)
   {
      /* Save the old info...						*/
      toppos		= ((CDKSWINDOW *)object)->screen->objectCount;
      swapobject	= ((CDKSWINDOW *)object)->screen->object[toppos];
      swaptype		= ((CDKSWINDOW *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSWINDOW *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSWINDOW *)object)->screenIndex		= toppos;
      ((CDKSWINDOW *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSWINDOW *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vITEMLIST)
   {
      /* Save the old info...						*/
      toppos		= ((CDKITEMLIST *)object)->screen->objectCount;
      swapobject	= ((CDKITEMLIST *)object)->screen->object[toppos];
      swaptype		= ((CDKITEMLIST *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKITEMLIST *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKITEMLIST *)object)->screenIndex		= toppos;
      ((CDKITEMLIST *)object)->screen->object[toppos]	= (void *)object;
      ((CDKITEMLIST *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vITEMLIST)
   {
      /* Save the old info...						*/
      toppos		= ((CDKFSELECT *)object)->screen->objectCount;
      swapobject	= ((CDKFSELECT *)object)->screen->object[toppos];
      swaptype		= ((CDKFSELECT *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKFSELECT *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKFSELECT *)object)->screenIndex		= toppos;
      ((CDKFSELECT *)object)->screen->object[toppos]	= (void *)object;
      ((CDKFSELECT *)object)->screen->cdktype[toppos]	= cdktype;
   }
   
   /* OK, Lets swap 'em....						*/
   if (swaptype == vLABEL)
   {
      ((CDKLABEL *)object)->screenIndex			= swapindex;
      ((CDKLABEL *)object)->screen->object[swapindex]	= swapobject;
      ((CDKLABEL *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vENTRY)
   {
      ((CDKENTRY *)object)->screenIndex			= swapindex;
      ((CDKENTRY *)object)->screen->object[swapindex]	= swapobject;
      ((CDKENTRY *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMENTRY)
   {
      ((CDKMENTRY *)object)->screenIndex		= swapindex;
      ((CDKMENTRY *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMENTRY *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSCROLL)
   {
      ((CDKSCROLL *)object)->screenIndex		= swapindex;
      ((CDKSCROLL *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSCROLL *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vDIALOG)
   {
      ((CDKDIALOG *)object)->screenIndex		= swapindex;
      ((CDKDIALOG *)object)->screen->object[swapindex]	= swapobject;
      ((CDKDIALOG *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSCALE)
   {
      ((CDKSCALE *)object)->screenIndex			= swapindex;
      ((CDKSCALE *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSCALE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMARQUEE)
   {
      ((CDKMARQUEE *)object)->screenIndex			= swapindex;
      ((CDKMARQUEE *)object)->screen->object[swapindex]		= swapobject;
      ((CDKMARQUEE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMENU)
   {
      ((CDKMENU *)object)->screenIndex			= swapindex;
      ((CDKMENU *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMENU *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMATRIX)
   {
      ((CDKMATRIX *)object)->screenIndex		= swapindex;
      ((CDKMATRIX *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMATRIX *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vHISTOGRAM)
   {
      ((CDKHISTOGRAM *)object)->screenIndex			= swapindex;
      ((CDKHISTOGRAM *)object)->screen->object[swapindex]	= swapobject;
      ((CDKHISTOGRAM *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSELECTION)
   {
      ((CDKSELECTION *)object)->screenIndex			= swapindex;
      ((CDKSELECTION *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSELECTION *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vVIEWER)
   {
      ((CDKVIEWER *)object)->screenIndex		= swapindex;
      ((CDKVIEWER *)object)->screen->object[swapindex]	= swapobject;
      ((CDKVIEWER *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vGRAPH)
   {
      ((CDKGRAPH *)object)->screenIndex			= swapindex;
      ((CDKGRAPH *)object)->screen->object[swapindex]	= swapobject;
      ((CDKGRAPH *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vRADIO)
   {
      ((CDKRADIO *)object)->screenIndex			= swapindex;
      ((CDKRADIO *)object)->screen->object[swapindex]	= swapobject;
      ((CDKRADIO *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vTEMPLATE)
   {
      ((CDKTEMPLATE *)object)->screenIndex			= swapindex;
      ((CDKTEMPLATE *)object)->screen->object[swapindex]	= swapobject;
      ((CDKTEMPLATE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSWINDOW)
   {
      ((CDKSWINDOW *)object)->screenIndex			= swapindex;
      ((CDKSWINDOW *)object)->screen->object[swapindex]		= swapobject;
      ((CDKSWINDOW *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vITEMLIST)
   {
      ((CDKITEMLIST *)object)->screenIndex			= swapindex;
      ((CDKITEMLIST *)object)->screen->object[swapindex]	= swapobject;
      ((CDKITEMLIST *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vFSELECT)
   {
      ((CDKFSELECT *)object)->screenIndex			= swapindex;
      ((CDKFSELECT *)object)->screen->object[swapindex]	= swapobject;
      ((CDKFSELECT *)object)->screen->cdktype[swapindex]	= swaptype;
   }
}
   
/*
 * This 'lowers' an object.
*/
void lowerCDKObject (EObjectType cdktype, void *object)
{
   /* Declare some vars.						*/
   void *swapobject	= (void *)NULL;
   int toppos		= 0;
   int swapindex	= -1;
   EObjectType swaptype;

   /* Should never dereference a void * pointer.			*/
   if (cdktype == vLABEL)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKLABEL *)object)->screen->object[toppos];
      swaptype		= ((CDKLABEL *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKLABEL *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKLABEL *)object)->screenIndex			= toppos;
      ((CDKLABEL *)object)->screen->object[toppos]	= (void *)object;
      ((CDKLABEL *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vENTRY)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKENTRY *)object)->screen->object[toppos];
      swaptype		= ((CDKENTRY *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKENTRY *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKENTRY *)object)->screenIndex			= toppos;
      ((CDKENTRY *)object)->screen->object[toppos]	= (void *)object;
      ((CDKENTRY *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMENTRY)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKMENTRY *)object)->screen->object[toppos];
      swaptype		= ((CDKMENTRY *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMENTRY *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMENTRY *)object)->screenIndex		= toppos;
      ((CDKMENTRY *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMENTRY *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSCROLL)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKSCROLL *)object)->screen->object[toppos];
      swaptype		= ((CDKSCROLL *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSCROLL *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSCROLL *)object)->screenIndex		= toppos;
      ((CDKSCROLL *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSCROLL *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vDIALOG)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKDIALOG *)object)->screen->object[toppos];
      swaptype		= ((CDKDIALOG *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKDIALOG *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKDIALOG *)object)->screenIndex		= toppos;
      ((CDKDIALOG *)object)->screen->object[toppos]	= (void *)object;
      ((CDKDIALOG *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSCALE)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKSCALE *)object)->screen->object[toppos];
      swaptype		= ((CDKSCALE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSCALE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSCALE *)object)->screenIndex			= toppos;
      ((CDKSCALE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSCALE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMARQUEE)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKMARQUEE *)object)->screen->object[toppos];
      swaptype		= ((CDKMARQUEE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMARQUEE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMARQUEE *)object)->screenIndex		= toppos;
      ((CDKMARQUEE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMARQUEE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMENU)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKMENU *)object)->screen->object[toppos];
      swaptype		= ((CDKMENU *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMENU *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMENU *)object)->screenIndex			= toppos;
      ((CDKMENU *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMENU *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vMATRIX)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKMATRIX *)object)->screen->object[toppos];
      swaptype		= ((CDKMATRIX *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKMATRIX *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKMATRIX *)object)->screenIndex		= toppos;
      ((CDKMATRIX *)object)->screen->object[toppos]	= (void *)object;
      ((CDKMATRIX *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vHISTOGRAM)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKHISTOGRAM *)object)->screen->object[toppos];
      swaptype		= ((CDKHISTOGRAM *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKHISTOGRAM *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKHISTOGRAM *)object)->screenIndex		= toppos;
      ((CDKHISTOGRAM *)object)->screen->object[toppos]	= (void *)object;
      ((CDKHISTOGRAM *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSELECTION)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKSELECTION *)object)->screen->object[toppos];
      swaptype		= ((CDKSELECTION *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSELECTION *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSELECTION *)object)->screenIndex		= toppos;
      ((CDKSELECTION *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSELECTION *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vVIEWER)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKVIEWER *)object)->screen->object[toppos];
      swaptype		= ((CDKVIEWER *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKVIEWER *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKVIEWER *)object)->screenIndex		= toppos;
      ((CDKVIEWER *)object)->screen->object[toppos]	= (void *)object;
      ((CDKVIEWER *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vGRAPH)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKGRAPH *)object)->screen->object[toppos];
      swaptype		= ((CDKGRAPH *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKGRAPH *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKGRAPH *)object)->screenIndex			= toppos;
      ((CDKGRAPH *)object)->screen->object[toppos]	= (void *)object;
      ((CDKGRAPH *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vRADIO)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKRADIO *)object)->screen->object[toppos];
      swaptype		= ((CDKRADIO *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKRADIO *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKRADIO *)object)->screenIndex			= toppos;
      ((CDKRADIO *)object)->screen->object[toppos]	= (void *)object;
      ((CDKRADIO *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vTEMPLATE)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKTEMPLATE *)object)->screen->object[toppos];
      swaptype		= ((CDKTEMPLATE *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKTEMPLATE *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKTEMPLATE *)object)->screenIndex		= toppos;
      ((CDKTEMPLATE *)object)->screen->object[toppos]	= (void *)object;
      ((CDKTEMPLATE *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vSWINDOW)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKSWINDOW *)object)->screen->object[toppos];
      swaptype		= ((CDKSWINDOW *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKSWINDOW *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKSWINDOW *)object)->screenIndex		= toppos;
      ((CDKSWINDOW *)object)->screen->object[toppos]	= (void *)object;
      ((CDKSWINDOW *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vITEMLIST)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKITEMLIST *)object)->screen->object[toppos];
      swaptype		= ((CDKITEMLIST *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKITEMLIST *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKITEMLIST *)object)->screenIndex		= toppos;
      ((CDKITEMLIST *)object)->screen->object[toppos]	= (void *)object;
      ((CDKITEMLIST *)object)->screen->cdktype[toppos]	= cdktype;
   }
   else if (cdktype == vFSELECT)
   {
      /* Save the old info...						*/
      swapobject	= ((CDKFSELECT *)object)->screen->object[toppos];
      swaptype		= ((CDKFSELECT *)object)->screen->cdktype[toppos];
      swapindex		= ((CDKFSELECT *)object)->screenIndex;

      /* Set the new info...						*/
      ((CDKFSELECT *)object)->screenIndex		= toppos;
      ((CDKFSELECT *)object)->screen->object[toppos]	= (void *)object;
      ((CDKFSELECT *)object)->screen->cdktype[toppos]	= cdktype;
   }
   
   /* OK, Lets swap 'em....						*/
   if (swaptype == vLABEL)
   {
      ((CDKLABEL *)object)->screenIndex			= swapindex;
      ((CDKLABEL *)object)->screen->object[swapindex]	= swapobject;
      ((CDKLABEL *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vENTRY)
   {
      ((CDKENTRY *)object)->screenIndex			= swapindex;
      ((CDKENTRY *)object)->screen->object[swapindex]	= swapobject;
      ((CDKENTRY *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMENTRY)
   {
      ((CDKMENTRY *)object)->screenIndex		= swapindex;
      ((CDKMENTRY *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMENTRY *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSCROLL)
   {
      ((CDKSCROLL *)object)->screenIndex		= swapindex;
      ((CDKSCROLL *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSCROLL *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vDIALOG)
   {
      ((CDKDIALOG *)object)->screenIndex		= swapindex;
      ((CDKDIALOG *)object)->screen->object[swapindex]	= swapobject;
      ((CDKDIALOG *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSCALE)
   {
      ((CDKSCALE *)object)->screenIndex			= swapindex;
      ((CDKSCALE *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSCALE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMARQUEE)
   {
      ((CDKMARQUEE *)object)->screenIndex			= swapindex;
      ((CDKMARQUEE *)object)->screen->object[swapindex]		= swapobject;
      ((CDKMARQUEE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMENU)
   {
      ((CDKMENU *)object)->screenIndex			= swapindex;
      ((CDKMENU *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMENU *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vMATRIX)
   {
      ((CDKMATRIX *)object)->screenIndex		= swapindex;
      ((CDKMATRIX *)object)->screen->object[swapindex]	= swapobject;
      ((CDKMATRIX *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vHISTOGRAM)
   {
      ((CDKHISTOGRAM *)object)->screenIndex			= swapindex;
      ((CDKHISTOGRAM *)object)->screen->object[swapindex]	= swapobject;
      ((CDKHISTOGRAM *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSELECTION)
   {
      ((CDKSELECTION *)object)->screenIndex			= swapindex;
      ((CDKSELECTION *)object)->screen->object[swapindex]	= swapobject;
      ((CDKSELECTION *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vVIEWER)
   {
      ((CDKVIEWER *)object)->screenIndex		= swapindex;
      ((CDKVIEWER *)object)->screen->object[swapindex]	= swapobject;
      ((CDKVIEWER *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vGRAPH)
   {
      ((CDKGRAPH *)object)->screenIndex			= swapindex;
      ((CDKGRAPH *)object)->screen->object[swapindex]	= swapobject;
      ((CDKGRAPH *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vRADIO)
   {
      ((CDKRADIO *)object)->screenIndex			= swapindex;
      ((CDKRADIO *)object)->screen->object[swapindex]	= swapobject;
      ((CDKRADIO *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vTEMPLATE)
   {
      ((CDKTEMPLATE *)object)->screenIndex			= swapindex;
      ((CDKTEMPLATE *)object)->screen->object[swapindex]	= swapobject;
      ((CDKTEMPLATE *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vSWINDOW)
   {
      ((CDKSWINDOW *)object)->screenIndex			= swapindex;
      ((CDKSWINDOW *)object)->screen->object[swapindex]		= swapobject;
      ((CDKSWINDOW *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vITEMLIST)
   {
      ((CDKITEMLIST *)object)->screenIndex			= swapindex;
      ((CDKITEMLIST *)object)->screen->object[swapindex]	= swapobject;
      ((CDKITEMLIST *)object)->screen->cdktype[swapindex]	= swaptype;
   }
   else if (swaptype == vFSELECT)
   {
      ((CDKFSELECT *)object)->screenIndex			= swapindex;
      ((CDKFSELECT *)object)->screen->object[swapindex]		= swapobject;
      ((CDKFSELECT *)object)->screen->cdktype[swapindex]	= swaptype;
   }
}

/*
 * This refreshes all the objects in the screen.
 */
void refreshCDKScreen (CDKSCREEN *cdkscreen)
{
   int objectCount 	= cdkscreen->objectCount;
   int x;

   /* We just call the drawObject function...	*/
   for (x=0; x < objectCount; x++)
   {
      if (cdkscreen->cdktype[x] == vLABEL)
      {
         CDKLABEL *label	= (CDKLABEL *)(cdkscreen->object[x]);
         drawCDKLabel (label, label->box);
      }
      else if (cdkscreen->cdktype[x] == vENTRY)
      {
         CDKENTRY *entry	= (CDKENTRY *)(cdkscreen->object[x]);
         drawCDKEntry (entry, entry->box);
      }
      else if (cdkscreen->cdktype[x] == vMENTRY)
      {
         CDKMENTRY *mentry	= (CDKMENTRY *)(cdkscreen->object[x]);
         drawCDKMentry (mentry, mentry->box);
      }
      else if (cdkscreen->cdktype[x] == vSCROLL)
      {
         CDKSCROLL *scroll	= (CDKSCROLL *)(cdkscreen->object[x]);
         drawCDKScroll (scroll, scroll->box);
      }
      else if (cdkscreen->cdktype[x] == vDIALOG)
      {
         CDKDIALOG *dialog	= (CDKDIALOG *)(cdkscreen->object[x]);
         drawCDKDialog (dialog, dialog->box);
      }
      else if (cdkscreen->cdktype[x] == vSCALE)
      {
         CDKSCALE *scale	= (CDKSCALE *)(cdkscreen->object[x]);
         drawCDKScale (scale, scale->box);
      }
      else if (cdkscreen->cdktype[x] == vMARQUEE)
      {
         CDKMARQUEE *marquee	= (CDKMARQUEE *)(cdkscreen->object[x]);
         drawCDKMarquee (marquee, marquee->box);
      }
      else if (cdkscreen->cdktype[x] == vMENU)
      {
         CDKMENU *menu	= (CDKMENU *)(cdkscreen->object[x]);
         drawCDKMenu (menu);
      }
      else if (cdkscreen->cdktype[x] == vMATRIX)
      {
         CDKMATRIX *matrix	= (CDKMATRIX *)(cdkscreen->object[x]);
         drawCDKMatrix (matrix, matrix->boxMatrix);
      }
      else if (cdkscreen->cdktype[x] == vHISTOGRAM)
      {
         CDKHISTOGRAM *histogram	= (CDKHISTOGRAM *)(cdkscreen->object[x]);
         drawCDKHistogram (histogram, histogram->box);
      }
      else if (cdkscreen->cdktype[x] == vSELECTION)
      {
         CDKSELECTION *selection	= (CDKSELECTION *)(cdkscreen->object[x]);
         drawCDKSelection (selection, selection->box);
      }
      else if (cdkscreen->cdktype[x] == vVIEWER)
      {
         CDKVIEWER *viewer	= (CDKVIEWER *)(cdkscreen->object[x]);
         drawCDKViewer (viewer, viewer->box);
      }
      else if (cdkscreen->cdktype[x] == vGRAPH)
      {
         CDKGRAPH *graph	= (CDKGRAPH *)(cdkscreen->object[x]);
         drawCDKGraph (graph, graph->box);
      }
      else if (cdkscreen->cdktype[x] == vRADIO)
      {
         CDKRADIO *radio	= (CDKRADIO *)(cdkscreen->object[x]);
         drawCDKRadio (radio, radio->box);
      }
      else if (cdkscreen->cdktype[x] == vTEMPLATE)
      {
         CDKTEMPLATE *template	= (CDKTEMPLATE *)(cdkscreen->object[x]);
         drawCDKTemplate (template, template->box);
      }
      else if (cdkscreen->cdktype[x] == vSWINDOW)
      {
         CDKSWINDOW *swindow	= (CDKSWINDOW *)(cdkscreen->object[x]);
         drawCDKSwindow (swindow, swindow->box);
      }
      else if (cdkscreen->cdktype[x] == vITEMLIST)
      {
         CDKITEMLIST *itemlist	= (CDKITEMLIST *)(cdkscreen->object[x]);
         drawCDKItemlist (itemlist, itemlist->box);
      }
      else if (cdkscreen->cdktype[x] == vFSELECT)
      {
         CDKFSELECT *fselect	= (CDKFSELECT *)(cdkscreen->object[x]);
         drawCDKFselect (fselect, fselect->box);
      }
   }

   /* Refresh the screen.			*/
   wrefresh (cdkscreen->window);
}

/*
 * This clears all the objects in the screen.
 */
void eraseCDKScreen (CDKSCREEN *cdkscreen)
{
   int objectCount 	= cdkscreen->objectCount;
   int x;

   /* We just call the drawObject function...	*/
   for (x=0; x < objectCount; x++)
   {
      if (cdkscreen->cdktype[x] == vLABEL)
      {
         eraseCDKLabel ((CDKLABEL *)(cdkscreen->object[x]));
      }
      else if (cdkscreen->cdktype[x] == vENTRY)
      {
         eraseCDKEntry ((CDKENTRY *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vMENTRY)
      {
         eraseCDKMentry ((CDKMENTRY *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vSCROLL)
      {
         eraseCDKScroll ((CDKSCROLL *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vDIALOG)
      {
         eraseCDKDialog ((CDKDIALOG *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vSCALE)
      {
         eraseCDKScale ((CDKSCALE *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vMARQUEE)
      {
         eraseCDKMarquee ((CDKMARQUEE *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vMENU)
      {
         eraseCDKMenu ((CDKMENU *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vMATRIX)
      {
         eraseCDKMatrix ((CDKMATRIX *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vHISTOGRAM)
      {
         eraseCDKHistogram ((CDKHISTOGRAM *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vSELECTION)
      {
         eraseCDKSelection ((CDKSELECTION *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vVIEWER)
      {
         eraseCDKViewer ((CDKVIEWER *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vGRAPH)
      {
         eraseCDKGraph ((CDKGRAPH *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vRADIO)
      {
         eraseCDKRadio ((CDKRADIO *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vTEMPLATE)
      {
         eraseCDKTemplate ((CDKTEMPLATE *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vSWINDOW)
      {
         eraseCDKSwindow ((CDKSWINDOW *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vITEMLIST)
      {
         eraseCDKItemlist ((CDKITEMLIST *)cdkscreen->object[x]);
      }
      else if (cdkscreen->cdktype[x] == vFSELECT)
      {
         eraseCDKFselect ((CDKFSELECT *)cdkscreen->object[x]);
      }
   }

   /* Refresh the screen.			*/
   wrefresh (cdkscreen->window);
}

/*
 * This destroys a CDK screen.
 */
void destroyCDKScreen (CDKSCREEN *screen)
{
   free (screen);
}

/*
 * This is added to remain consistent.
 */
void endCDK()
{
   /* Turn echoing back on...		*/
   echo();

   /* Turn off cbreak...		*/
   nocbreak();

   /* End the curses windows...		*/
   endwin();
}

/*
 * This is for pure diagnostics. Really should be taken out
 * at some point in time.
 */
void debugCDKScreen (CDKSCREEN *screen, char *mesg)
{
   char *objects[]	= {"ENTRY", "MENTRY", "LABEL", "SCROLL", "DIALOG", "ARRAY", "SCALE", "MARQUEE", "MENU", "MATRIX", "HISTOGRAM", "SELECTION", "VIEWER", "GRAPH", "RADIO", "TEMPLATE", "SWINDOW"};
   int count		= screen->objectCount;
   int x;

   for (x=0; x < count ; x++)
   {
      printf ("CDK Object[%d] : Type: %s\n", x, objects[screen->cdktype[x]]);
   }
}
