#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/14 06:19:52 $
 * $Revision: 1.20 $
 */

/*
 * This function creates a dialog widget.
 */
CDKDIALOG *newCDKDialog( CDKSCREEN *cdkscreen, int xplace, int yplace, char **mesg, int rows, char **buttonlabel, int buttonCount, chtype highlight, boolean separator, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKDIALOG *dialog	= (CDKDIALOG *)malloc (sizeof (CDKDIALOG));
   int boxWidth		= MINDIALOGWIDTH;
   int boxHeight	= rows + 3 + separator;
   int maxmessagewidth	= -1;
   int buttonwidth	= 0;
   int xpos		= xplace;
   int ypos		= yplace;
   int temp		= 0;
   int buttonadj	= 0;
   int x		= 0;

   /* Translate the char * message to a chtype *			*/
   for (x=0; x < rows; x++)
   {
      dialog->info[x]	= char2Chtype (mesg[x], &dialog->infolen[x], &dialog->infopos[x]);
      maxmessagewidth	= MAX(maxmessagewidth, dialog->infolen[x]);
   }

   /* Translate the button label char * to a chtype *			*/
   for (x = 0; x < buttonCount; x++)
   {
      dialog->buttonlabel[x]	= char2Chtype (buttonlabel[x], &dialog->buttonlen[x], &temp);
      buttonwidth		+= dialog->buttonlen[x] + 1;
   }
   buttonwidth--;

   /* Determine the final dimensions of the box.			*/
   boxWidth	= MAX(boxWidth, maxmessagewidth);
   boxWidth	= MAX(boxWidth, buttonwidth);
   boxWidth	= boxWidth + 4;

   /* Now we have to readjust the x and y positions			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Set up the dialog box attributes.					*/
   dialog->parent		= cdkscreen->window;
   dialog->win			= newwin (boxHeight, boxWidth, ypos, xpos);
   dialog->shadowwin		= (WINDOW *)NULL;
   dialog->buttonCount		= buttonCount;
   dialog->currentButton	= 0;
   dialog->messageRows		= rows;
   dialog->boxHeight		= boxHeight;
   dialog->boxWidth		= boxWidth;
   dialog->highlight		= highlight;
   dialog->separator		= separator;
   dialog->box			= box;
   dialog->shadow		= shadow;

   /* If we couldn't create the window, we should return a NULL value.	*/
   if (dialog->win == (WINDOW *)NULL)
   {
      /* Couldn't create the window. Clean up used memory.		*/
      for (x=0; x < dialog->messageRows ; x++)
      {
         freeChtype (dialog->info[x]);
      }
      for (x=0; x < dialog->buttonCount; x++)
      {
         freeChtype (dialog->buttonlabel[x]);
      }

      /* Remove the memory used by the dialog pointer.			*/
      free (dialog);

      /* Return a NULL dialog box.					*/
      return ( (CDKDIALOG *)NULL );
   }

   /* Find the button positions.					*/
   buttonadj		= ((int)((boxWidth-buttonwidth)/2));
   for (x = 0; x < buttonCount; x++)
   {
      dialog->buttonpos[x]	= buttonadj;
      buttonadj			= buttonadj + dialog->buttonlen[x] + 1;
   }

   /* Create the string alignments.					*/
   for (x=0; x < rows; x++)
   {
      dialog->infopos[x]	= justifyString (boxWidth, dialog->infolen[x], dialog->infopos[x]);
   }

   /* Was there a shadow???						*/
   if (shadow)
   {
      dialog->shadowwin		= newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Let them use escape keys.						*/
   keypad (dialog->win, TRUE);

   /* Empty the key bindings.						*/
   cleanCDKObjectBindings (vDIALOG, dialog);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vDIALOG, dialog);

   /* Return the dialog box pointer.					*/
   return (dialog);
}

/*
 * This lets the user select the button.
 */
int activateCDKDialog (CDKDIALOG *dialog)
{
   /* Declare local variables.						*/
   int firstbutton	= 0;
   int lastbutton	= dialog->buttonCount - 1;
   chtype input		= (chtype)NULL;

   /* Draw the dialog box.						*/
   drawCDKDialog (dialog, dialog->box);

   /* Lets move to the first button					*/
   writeChtypeAttrib (dialog->win,
			dialog->buttonpos[dialog->currentButton],
			dialog->boxHeight-2,
			dialog->buttonlabel[dialog->currentButton],
			dialog->highlight,
			HORIZONTAL,
			0, dialog->buttonlen[dialog->currentButton]);
   wrefresh (dialog->win);

   /* Turn off the cursor.						*/
   curs_set (0);

   /* Lets start taking the input.					*/
   for (;;)
   {
      input = wgetch (dialog->win);

      /* Check for a key binding...					*/
      if (! checkCDKObjectBind (vDIALOG, dialog, input))
      {
         switch (input)
         {
            case KEY_LEFT :
                 dialog->currentButton	= (dialog->currentButton == firstbutton ? lastbutton : dialog->currentButton-1);
                 break;

            case KEY_RIGHT : case KEY_TAB : case ' ' :
                 dialog->currentButton	= (dialog->currentButton == lastbutton ? firstbutton : dialog->currentButton+1);
                 break;

            case KEY_UP : case KEY_DOWN :
                 Beep();
                 break;

            case CDK_REFRESH :
                 refreshCDKScreen (dialog->screen);
                 break;

            case KEY_RETURN : case CDK_NEXT :
                 curs_set(1);
                 return (dialog->currentButton);
                 break;

         default :
            break;
         }
      }

      /* Redraw the buttons.						*/
      redrawCDKDialogButtons (dialog);
      wrefresh (dialog->win);
   }
}

/*
 * This function draws the dialog widget.
 */
void drawCDKDialog (CDKDIALOG *dialog, boolean Box)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the dialog widget from the screen.				*/
   eraseCDKDialog (dialog);

   /* Is there a shadow???						*/
   if (dialog->shadow)
   {
      drawShadow (dialog->shadowwin);
   }

   /* Box the widget if they asked.					*/
   if (Box)
   {
      box (dialog->win, ACS_VLINE, ACS_HLINE);
   }

   /* Draw in the message.						*/
   for (x=0; x < dialog->messageRows; x++)
   {
      writeChtype (dialog->win,
			dialog->infopos[x],
			x+1,
			dialog->info[x],
			HORIZONTAL,
			0, dialog->infolen[x]);
   }

   /* Draw in the buttons.						*/
   redrawCDKDialogButtons (dialog);
   wrefresh (dialog->win);
}

/*
 * This function destroys the dialog widget.
 */
void destroyCDKDialog (CDKDIALOG *dialog)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the object.							*/
   eraseCDKDialog (dialog);

   /* Clean up the char pointers.					*/
   for (x=0; x < dialog->messageRows ; x++)
   {
      freeChtype (dialog->info[x]);
   }
   for (x=0; x < dialog->buttonCount; x++)
   {
      freeChtype (dialog->buttonlabel[x]);
   }

   /* Clean up the windows.						*/
   delwin (dialog->win);
   if (dialog->shadow)
   {
      delwin (dialog->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vDIALOG, dialog);

   /* Finish cleaning up.						*/
   free (dialog);
}

/*
 * This function erases the dialog widget from the screen.
 */
void eraseCDKDialog (CDKDIALOG *dialog)
{
   /* Erase the main window.						*/
   werase (dialog->win);
   wrefresh (dialog->win);

   /* Do we need to erase the shadow???					*/
   if (dialog->shadow)
   {
      werase (dialog->shadowwin);
      wrefresh (dialog->shadowwin);
   }
}

/*
 * This sets attributes of the dialog box.
 */
void setCDKDialog (CDKDIALOG *dialog, chtype highlight, boolean separator, boolean box)
{
   dialog->highlight	= highlight;
   dialog->separator	= separator;
   dialog->box		= box;
}

/*
 * This redraws the dialog buttons and trhe separation line.
 */
void redrawCDKDialogButtons (CDKDIALOG *dialog)
{
   /* Declare local variables.						*/
   int x;

   for (x=0; x < dialog->buttonCount; x++)
   {
      writeChtype (dialog->win,
			dialog->buttonpos[x],
			dialog->boxHeight-2,
			dialog->buttonlabel[x],
			HORIZONTAL,
			0, dialog->buttonlen[x]);
   }
   writeChtypeAttrib (dialog->win,
		dialog->buttonpos[dialog->currentButton],
		dialog->boxHeight-2,
		dialog->buttonlabel[dialog->currentButton],
		dialog->highlight,
		HORIZONTAL,
		0, dialog->buttonlen[dialog->currentButton]);

   /* Draw the separation line.						*/
   if (dialog->separator)
   {
      for (x=1; x < dialog->boxWidth-1; x++)
      {
         mvwaddch (dialog->win, dialog->boxHeight-3, x, ACS_HLINE);
      }
      mvwaddch (dialog->win, dialog->boxHeight-3, 0, ACS_LTEE);
      mvwaddch (dialog->win, dialog->boxHeight-3, dialog->win->_maxx, ACS_RTEE);
   }
}
