#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/16 17:23:24 $
 * $Revision: 1.13 $
 */

/*
 * This sets up a basic set of color pairs. These can be redefined
 * if wanted...
 */
void initCDKColor ()
{
#ifdef COLOR
   /* Declare local variables.					*/
   int color[]	= {
			COLOR_WHITE,	COLOR_RED,	COLOR_GREEN,
			COLOR_YELLOW,	COLOR_BLUE,	COLOR_MAGENTA,
			COLOR_CYAN,	COLOR_BLACK
		};
   int pair	= 1;
   int x, y;

   /* Start color first.					*/
   start_color();
  
   /* Create the color pairs.					*/
   for (x=0; x < 8; x++)
   {
      for (y=0; y < 8; y++)
      {
         init_pair (pair++, color[x], color[y]);
      }
   }
#endif
}

/*
 * This prints out a box around a window with attributes
 */
void boxWindow (WINDOW *window, chtype attr)
{
   /* Set some variables.					*/
   int tlx	= 0;
   int tly	= 0;
   int brx	= window->_maxx - 1;
   int bry	= window->_maxy;
   int x, y;

   /* Draw horizontal lines 					*/
   for (x=tlx; x<=brx;x++) 
   {
       mvwaddch(window, tly, x, ACS_HLINE | attr);
       mvwaddch(window, bry, x, ACS_HLINE | attr);
    }

   /* Draw vertical lines 					*/
   for (y=tly; y<=bry;y++) 
   {
       mvwaddch(window, y, tlx, ACS_VLINE | attr);
       mvwaddch(window, y, brx, ACS_VLINE | attr);
    }

   /* Draw in the corners */
   mvwaddch(window, tly, tlx, ACS_ULCORNER | attr);
   mvwaddch(window, tly, brx, ACS_URCORNER | attr);
   mvwaddch(window, bry, tlx, ACS_LLCORNER | attr);
   mvwaddch(window, bry, brx, ACS_LRCORNER | attr);
   wrefresh(window);
}

/*
 * This draws a box with attributes and lets the user define
 * each element of the box.
 */
void attrbox (WINDOW *win, chtype tlc, chtype trc, chtype blc, chtype brc, chtype horz, chtype vert, chtype attr)
{
   /* Set the coordinates.	*/
   int x1		= 0;
   int y1		= 0;
#ifdef COLOR
   int y2		= win->_maxy;
   int x2		= win->_maxx;
#else
   int y2		= win->_maxy - 1;
   int x2		= win->_maxx - 1;
#endif
   int x, y;

   /* Draw horizontal lines.	*/
   for (x=x1; x<=x2;x++) 
   {
       mvwaddch(win, y1, x, horz | attr);
       mvwaddch(win, y2, x, horz | attr);
    }

   /* Draw vertical lines.	*/
   for (y=y1; y<=y2;y++) 
   {
       mvwaddch(win, y, x1, vert | attr);
       mvwaddch(win, y, x2, vert | attr);
    }

   /* Draw in the corners.	*/
   mvwaddch(win, y1, x1, tlc | attr);
   mvwaddch(win, y1, x2, trc | attr);
   mvwaddch(win, y2, x1, blc | attr);
   mvwaddch(win, y2, x2, brc | attr);
   wrefresh(win);
}

/*
 * This draws a line on the given window. (odd angle lines not working yet)
 */
void drawLine  (WINDOW *window, int startx, int starty, int endx, int endy, chtype line)
{
   /* De=clare some local vars.	*/
   int xdiff	= endx - startx;
   int ydiff	= endy - starty;
   int x	= 0;
   int y	= 0;

   /* Determine if we are drawing a horizontal or vertical line.	*/
   if ( ydiff == 0)
   {
      /* Horizontal line.      <--------- X -------->			*/
      for (x=0; x < xdiff ; x++)
      {
         mvwaddch (window, starty, startx + x, line);
      }
   }
   else if (xdiff == 0)
   {
      /* Vertical line.							*/
      for (y=0; y < ydiff ; y++)
      {
         mvwaddch (window, starty + y, startx, line);
      }
   }
   else
   {
      /* We need to determine the angle of the line.	*/
      int height	= xdiff;
      int width		= ydiff;
      int xratio	= (height > width ? 1 : (width / height));
      int yratio	= (width > height ? (width / height) : 1);
      int xadj		= 0;
      int yadj		= 0;
  
      /* Set the vars.	*/
      x	= startx;
      y	= starty;
      while (x != endx && y != endy)
      {

         /* Add the char to the window.		*/
         mvwaddch (window, y, x, line);

         /* Make the x and y adjustments.	*/
         if (xadj != xratio)
         {
            x	= (xdiff < 0 ? x-1 : x+1);
            xadj++;
         }
         else
         {
            xadj	= 0;
         }
         if (yadj != yratio)
         {
            y	= (ydiff < 0 ? y-1 : y+1);
            yadj++;
         }
         else
         {
            yadj	= 0;
         }
      }
   }
}

/*
 * This draws a shadow around a window.
 */
void drawShadow (WINDOW *shadowwin)
{
   /* Declare local variables.					*/
   int x		= 0;

   /* Draw the line on the bottom.				*/
   for (x=1; x <= shadowwin->_maxx; x++)
   {
      mvwaddch (shadowwin, shadowwin->_maxy, x, ACS_HLINE | A_DIM);
   }

   /* Draw the line on the right.				*/
   for (x=0; x < shadowwin->_maxy; x++)
   {
      mvwaddch (shadowwin, x, shadowwin->_maxx, ACS_VLINE |A_DIM);
   }
   mvwaddch (shadowwin, 0, shadowwin->_maxx, ACS_URCORNER | A_DIM);
   mvwaddch (shadowwin, shadowwin->_maxy, 0, ACS_LLCORNER | A_DIM);
   mvwaddch (shadowwin, shadowwin->_maxy, shadowwin->_maxx, ACS_LRCORNER | A_DIM);
   wrefresh (shadowwin);
}

/*
 * This writes out a char * string with no attributes.
 */
void writeChar (WINDOW *window, int xpos, int ypos, char *string, int align, int start, int end)
{
   /* Declare local variables.					*/
   int display		= end - start;
   int x		= 0;

   /* Check the alignment of the message.			*/
   if (align == HORIZONTAL)
   { 
      display		= MIN(display,window->_maxx-1);
      for (x=0; x < display ; x++)
      {
         /* Draw the message on a horizontal axis.		*/
         mvwaddch (window, ypos, xpos+x, string[x+start] | A_NORMAL);
      }
   }
   else
   {
      /* Draw the message on a vertical axis.			*/
      display		= MIN(display,window->_maxy-1);
      for (x=0; x < display ; x++)
      {
         mvwaddch (window, ypos+x, xpos, string[x+start] | A_NORMAL);
      }
   }
}

/*
 * This writes out a char * string with attributes.
 */
void writeCharAttrib (WINDOW *window, int xpos, int ypos, char *string, chtype attr, int align, int start, int end)
{
   /* Declare local variables.					*/
   int display		= end - start;
   int x		= 0;

   /* Check the alignment of the message.			*/
   if (align == HORIZONTAL)
   { 
      /* Draw the message on a horizontal axis.			*/
      display		= MIN(display,window->_maxx-1);
      for (x=0; x < display ; x++)
      {
         mvwaddch (window, ypos, xpos+x, string[x+start] | attr);
      }
   }
   else
   {
      /* Draw the message on a vertical axis.			*/
      display		= MIN(display,window->_maxy-1);
      for (x=0; x < display ; x++)
      {
         mvwaddch (window, ypos+x, xpos, string[x+start] | attr);
      }
   }
}

/*
 * This writes out a chtype * string.
 */
void writeChtype (WINDOW *window, int xpos, int ypos, chtype *string, int align, int start, int end)
{
   /* Declare local variables.					*/
   int diff		= 0;
   int display		= 0;
   int x		= 0;

   /* Determine how much we need to display.			*/
   if ( end >= start)
   {
      diff	= end - start;
   }

   /* Check the alignment of the message.			*/
   if (align == HORIZONTAL)
   {
      /* Draw the message on a horizontal axis. 		*/
      display		= MIN(diff,window->_maxx-1);
      for (x=0; x < display; x++)
      {
         mvwaddch (window, ypos, xpos+x, string[x+start]);
      }
   }
   else
   {
      /* Draw the message on a vertical axis.			*/
      display		= MIN(diff,window->_maxy-1);
      for (x=0; x < display; x++)
      {
         mvwaddch (window, ypos+x, xpos, string[x+start]);
      }
   }
}

/*
 * This writes out a chtype * string forcing the chtype string
 * to be printed out with the given attributes instead.
 */
void writeChtypeAttrib (WINDOW *window, int xpos, int ypos, chtype *string, chtype attr, int align, int start, int end)
{
   /* Declare local variables.					*/
   int diff		= 0;
   int display		= 0;
   int x		= 0;
   chtype plain;

   /* Determine how much we need to display.			*/
   if ( end >= start)
   {
      diff	= end - start;
   }

   /* Check the alignment of the message.			*/
   if (align == HORIZONTAL)
   {
      /* Draw the message on a horizontal axis. 		*/
      display		= MIN(diff,window->_maxx-1);
      for (x=0; x < display; x++)
      {
         plain	= string[x+start] & A_CHARTEXT;
         mvwaddch (window, ypos, xpos+x, plain | attr);
      }
   }
   else
   {
      /* Draw the message on a vertical axis.			*/
      display		= MIN(diff,window->_maxy-1);
      for (x=0; x < display; x++)
      {
         plain	= string[x+start] & A_CHARTEXT;
         mvwaddch (window, ypos+x, xpos, plain | attr);
      }
   }
}

/*
 * This pops up a message.
 */
void popUpMessage (CDKSCREEN *screen, char **mesg, int count)
{
   /* Declare local variables.					*/
   CDKLABEL	*popup;

   /* Create the label.						*/
   popup	= newCDKLabel (screen, CENTER, CENTER,
		mesg, count, TRUE, FALSE);

   /* Draw it... wait ... kill it.				*/
   drawCDKLabel (popup, TRUE);
   wgetch (popup->win);
   destroyCDKLabel (popup);

   /* Clean the screen.						*/
   refreshCDKScreen (screen);
}
