#include "cdk.h"

/*
 * $Author: mikeg $
 * $Date: 1995/12/14 15:45:44 $
 * $Revision: 1.84 $
 */

/*
 * This creates a pointer to an entry widget.
 */
CDKENTRY *newCDKEntry (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, char *label, chtype fieldattr, chtype filler, EDisplayType disptype, int fieldWidth, int min, int max, ENTRYCB entryCallBack, boolean box, boolean shadow)
{
   /* Set up some variables.						*/
   CDKENTRY *entry	= (CDKENTRY *)malloc (sizeof (CDKENTRY));
   char *plainlabel	= (char *)NULL;
   int boxwidth		= 0;
   int boxheight	= 3;
   int xpos		= xplace;
   int ypos		= yplace;
   int labelxoffset	= 0;
   int labelyoffset	= 0;
   int fieldxoffset	= 0;
   int fieldyoffset	= 0;
   int temp		= 0;

   /* Translate the label char *pointer to a chtype pointer.		*/
   entry->label		= char2Chtype (label, &entry->labellen, &temp);
   plainlabel		= chtype2Char (entry->label);

   /* Adjust the offset values given the location of the label.		*/
   alignlabel (plainlabel, lplace, fieldWidth, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainlabel);
   
   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Make the label window.						*/
   entry->win = newwin (boxheight, boxwidth, ypos, xpos);

   /* Is the window NULL ???						*/
   if (entry->win == (WINDOW *)NULL)
   {
      /* Clean up the pointers.						*/
      freeChtype (entry->label);
      free (entry);
      
      /* Exit with NULL.						*/
      return ( (CDKENTRY *)NULL );
   }

   /* Make the field window.						*/
   entry->field		= subwin (cdkscreen->window, 1, fieldWidth, ypos+fieldyoffset, xpos+fieldxoffset);
   entry->labelwin	= subwin (cdkscreen->window, 1, entry->labellen+1, ypos+labelyoffset, xpos+labelxoffset);
   keypad (entry->field, TRUE);

   /* Make room for the info char * pointer.				*/
   entry->info		= (char *)malloc (sizeof(char) * (max+3));
   cleanChar (entry->info, max+3, '\0');

   /* Set up the rest of the structure.					*/
   entry->parent	= cdkscreen->window;
   entry->shadowwin	= (WINDOW *)NULL;
   entry->totalwidth	= 0;
   entry->fieldattr	= fieldattr;
   entry->fieldwidth	= fieldWidth;
   entry->filler	= filler;
   entry->box		= box;
   entry->shadow	= shadow;
   entry->screencol	= 0;
   entry->leftchar	= 0;
   entry->min		= min;
   entry->max		= max;
   entry->disptype	= disptype;
   entry->callbackfn	= (void *)entryCallBack;

   /* Do we want a shadow???						*/
   if (shadow)
   {
      entry->shadowwin = newwin (boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vENTRY, entry);

   /* Register this baby.               				*/
   registerCDKObject (cdkscreen, vENTRY, entry);

   /* Return the pointer to the structure				*/
   return (entry);
}

/* 
 * This means you want to use the given entry field. It takes input
 * from the keyboard, and when its done, it fills the entry info
 * element of the structure with what was typed.
 */
char *activateCDKEntry (CDKENTRY *entry)
{
   /* Declare local variables.						*/
   int temp		= 0;
   int x		= 0;
   chtype input;

   /* Add the filler character on the end of the info.			*/
   entry->info[(int)strlen(entry->info)] = entry->filler;

   /* Draw the widget.							*/
   drawCDKEntry (entry, entry->box);

   /* Lets get the typed info.						*/
   for (;;)
   {
      /* Get the input from the user.					*/
      input = wgetch (entry->field);

      /* Check a predefined binding...					*/
      if ( ! checkCDKObjectBind (vENTRY, entry, input) )
      {
         switch (input)
         {
            case KEY_UP : case KEY_DOWN :
                 Beep();
                 break;
   
            case KEY_LEFT :
                 if (entry->screencol == 0)
                 {
                    if (entry->leftchar == 0)
                    {
                       Beep();
                    }
                    else
                    {
                       /* Scroll left.	*/
                       entry->leftchar--;
                       temp = (int)strlen(entry->info);
                       for (x=0; x < temp; x++)
                       {
                          if ( (entry->leftchar + x) < temp)
                          {
                             mvwaddch (entry->field, 0, x, entry->info[entry->leftchar+x] | entry->fieldattr);
                          }
                          else
                          {
                             mvwaddch (entry->field, 0, x, entry->filler);
                          }
                       }
                       wmove (entry->field, 0, 0);
                       wrefresh (entry->field);
                    }
                 }
                 else
                 {
                    /* Move left.	*/
                    wmove (entry->field, 0, --entry->screencol);
                    wrefresh (entry->field);
                 }
                 break;
   
            case KEY_RIGHT :
                 if (entry->screencol == entry->fieldwidth-1)
                 {
                    temp = (int)strlen (entry->info);
                    if ( (entry->leftchar + entry->screencol+1) == temp)
                    {
                       Beep();
                    }
                    else
                    {
                       /* Scroll to the right				*/
                       entry->leftchar++;
                       temp = (int)strlen(entry->info);
                       for (x=0; x < temp; x++)
                       {
                          if ( (entry->leftchar + x) < temp)
                          {
                             mvwaddch (entry->field, 0, x, entry->info[entry->leftchar+x] | entry->fieldattr);
                          }
                          else
                          {
                             mvwaddch (entry->field, 0, entry->leftchar + x, entry->filler);
                          }
                       }
                       wmove (entry->field, 0, entry->screencol);
                       wrefresh (entry->field);
                    }
                 }
                 else
                 {
                    /* Move right.					*/
                    temp = (int)strlen (entry->info);
                    if ( (entry->leftchar + entry->screencol + 1) == temp)
                    {
                       Beep();
                    }
                    else
                    {
                       wmove (entry->field, 0, ++entry->screencol);
                    }
                    wrefresh (entry->field);
                 }
                 break;
   
            case DELETE :
                 if (entry->disptype == vVIEWONLY)
                 {
                    Beep();
                 }
                 else
                 {
                    /* Get the length of the widget information.	*/
                    int infoLength = (int)strlen (entry->info);

                    if ( (entry->leftchar + entry->screencol + 1) < infoLength)
                    {
                       /* We are deleteing from inside the string.	*/
                       int currPos = entry->screencol+entry->leftchar;
                       for (x=currPos; x < infoLength; x++)
                       {
                          entry->info[x] = entry->info[x+1];
                       }
                       entry->info[infoLength] = '\0';
   
                       /* Update the screen.				*/
                       if (entry->screencol > 0)
                       {
                          mvwdelch (entry->field, 0, --entry->screencol);
                       }
                       else
                       {
                          mvwdelch (entry->field, 0, entry->screencol);
                       }
                       if ( (int)strlen (entry->info)-1 - entry->leftchar > entry->fieldwidth-1)
                       {
                          mvwaddch (entry->field, 0, entry->fieldwidth-1, entry->info[entry->leftchar+entry->fieldwidth-1] | entry->fieldattr);
                       }
                       else
                       {
                          mvwaddch (entry->field, 0, entry->fieldwidth-1, entry->filler);
                       }
                       wmove (entry->field, 0, entry->screencol);
                       wrefresh (entry->field);
                    }
                    else
                    {
                       /* We are deleting from the end of the string.	*/
                       int infoLength = (int)strlen (entry->info);
                       if (infoLength > 1)
                       {
                          int sp = (entry->leftchar > 0 ? 1 : 0);

                          /* Update the character pointer.		*/
                          entry->info[infoLength-1] = '\0';
                          entry->info[infoLength-2] = entry->filler | entry->fieldattr;

                          /* Update the screen.				*/
                          if (entry->screencol == sp)
                          {
                             /* Make sure we can something to delete.	*/
                             infoLength = (int)strlen(entry->info);
                             if ( infoLength > 0 )
                             {
                                /* We need to adjust the field info	*/
                                if (infoLength > entry->fieldwidth)
                                {
                                   /* The field information is larger than the field.	*/
                                   entry->leftchar = (infoLength - entry->fieldwidth);
                                   entry->screencol = entry->fieldwidth-1;
                                }
                                else
                                {
                                   /* The field information is smaller than the field.	*/
                                   entry->leftchar = 0;
                                   entry->screencol = infoLength-1;
                                }

                                /* Redraw the field.					*/
                                drawCDKEntryField (entry);
                             }
                             else
                             {
                                Beep();
                             }
                          }
                          else
                          {
                             /* Update the screen.		*/
                             mvwdelch (entry->field, 0, --entry->screencol);
                             mvwaddch (entry->field, 0, entry->fieldwidth-1, entry->filler);
                             wmove (entry->field, 0, entry->screencol);
                             wrefresh (entry->field);
                          }
                       }
                       else
                       {
                          Beep();
                       }
                    }
                 }
                 break;
   
            case KEY_RETURN : case KEY_TAB : case CDK_NEXT :
                 if ((int)strlen (entry->info) >= entry->min)
                 {
                    entry->info[(int)strlen(entry->info)-1] = '\0';
                    return (entry->info);
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (entry->screen);
                 break;
   
            default :
                 ((ENTRYCB)entry->callbackfn)(entry, input);
                 break;
         }
      }
   }
}

/*
 * This is a generic character parser for the entry field. It is used as a 
 * callback function, so any personal modifications can be made by creating
 * a new function and calling the activation with its name.
 */
void CDKEntryCallBack (CDKENTRY *entry, chtype character)
{
   /* Declare local variables.						*/
   char	plainchar	= (character & A_CHARTEXT);
   int temp, x;

   /* Start checking the input.						*/
   if (entry->disptype == vINT && !isdigit(plainchar) )
   {
      Beep();
   }
   else if ( (	entry->disptype == vCHAR || entry->disptype == vUCHAR || \
		entry->disptype == vLCHAR || entry->disptype == vUHCHAR || \
		entry->disptype == vLHCHAR) && isdigit(plainchar))
   {
      Beep();
   }
   else if (entry->disptype == vVIEWONLY)
   {
      Beep();
   }
   else 
   {
      if ( (int)strlen (entry->info) == entry->max )
      {
         Beep();
      }
      else
      {
         /* We will make any adjustments to the case of the character.	*/
         if ( (	entry->disptype == vUCHAR || entry->disptype == vUHCHAR || \
		entry->disptype == vUMIXED || entry->disptype == vUHMIXED)  \
		&& !isdigit(plainchar) )
         {
            plainchar = toupper (plainchar);
         }
         else if ( (	entry->disptype == vLCHAR || entry->disptype == vLHCHAR || \
			entry->disptype == vLMIXED || entry->disptype == vLHMIXED) && \
			!isdigit(plainchar) )
         {
            plainchar = tolower (plainchar);
         }

         /* Update the screen and pointer.				*/
         if ( entry->screencol != entry->fieldwidth-1)
         {
            /* Update the character pointer.				*/
            temp = (int)strlen (entry->info);
            for (x=temp; x > entry->screencol+entry->leftchar; x--)
            {
               entry->info[x] = entry->info[x-1];
            }
            entry->info[ entry->screencol+entry->leftchar] = plainchar;

            /* Update the screen.					*/
            if (	entry->disptype == vHINT || entry->disptype == vHCHAR || \
			entry->disptype == vHMIXED || entry->disptype == vUHCHAR || \
			entry->disptype == vLHCHAR || entry->disptype == vUHMIXED || \
			entry->disptype == vLHMIXED )
            {
               mvwinsch (entry->field, 0, entry->screencol++, '.' | entry->fieldattr);
            }
            else
            {
               mvwinsch (entry->field, 0, entry->screencol++, plainchar | entry->fieldattr);
            }
            wmove (entry->field, 0, entry->screencol );
            wrefresh (entry->field);
         }
         else
         {
            /* Update the character pointer.				*/
            temp		= (int)strlen (entry->info);
            entry->info[temp-1]	= plainchar;
            entry->info[temp]	= entry->filler & A_CHARTEXT;
            entry->info[temp+1]	= '\0';

            /* Update the screen					*/
            mvwdelch (entry->field, 0, 0);
            if (	entry->disptype == vHINT || entry->disptype == vHCHAR || \
			entry->disptype == vHMIXED || entry->disptype == vUHCHAR || \
			entry->disptype == vLHCHAR || entry->disptype == vUHMIXED || \
			entry->disptype == vLHMIXED )
            {
               mvwaddch (entry->field, 0, entry->fieldwidth-2, '.' | entry->fieldattr);
            }
            else
            {
               mvwaddch (entry->field, 0, entry->fieldwidth-2, plainchar | entry->fieldattr);
            }
            mvwaddch (entry->field, 0, entry->fieldwidth-1, entry->filler);
            wmove (entry->field, 0, entry->fieldwidth-1);
            wrefresh (entry->field);
            entry->leftchar++;
         }
      }
   }
}

/*
 * This erases the information in the entry field
 * and redraws a clean and empty entry field.
 */
void cleanCDKEntry (CDKENTRY *entry)
{
   /* Declare local variables.						*/
   char plainchar	= entry->filler & A_CHARTEXT;
   int width = entry->fieldwidth;
   int x;

   /* Erase the information in the character pointer.			*/
   cleanChar (entry->info, entry->totalwidth, '\0');
   entry->info[0]	= plainchar;

   /* Clean the entry screen field.					*/
   for (x=0; x < width; x++)
   {
      mvwaddch (entry->field, 0, x, entry->filler);
   }

   /* Refresh the entry field.						*/
   wrefresh (entry->field);
}

/*
 * This draws the entry field.
 */
void drawCDKEntry (CDKENTRY *entry, boolean Box)
{
   /* Erase the entry widget from the screen.				*/
   eraseCDKEntry (entry);

   /* Did we ask for a shadow???					*/
   if (entry->shadow)
   {
      drawShadow (entry->shadowwin);
   }

   /* Box the widget if asked.						*/
   if (Box)
   {
      box (entry->win, ACS_VLINE, ACS_HLINE);
      wrefresh (entry->win);
   }

   /* Draw in the label to the widget.					*/
   writeChtype (entry->labelwin, 0, 0, entry->label, HORIZONTAL, 0, entry->labellen);
   wrefresh (entry->labelwin);

   /* Redraw the entry field.						*/
   drawCDKEntryField (entry);
}

/*
 * This redraws the entry field.
 */
void drawCDKEntryField (CDKENTRY *entry)
{
   /* Declare variables.						*/
   int infoLength = (int)strlen (entry->info);
   int x = 0;

   /* Draw in the filler characters.					*/
   for (x=0; x < entry->fieldwidth; x++)
   {
      mvwaddch (entry->field, 0, x, entry->filler);
   }
   
   /* If there is information in the field. Then draw it in.		*/
   if (entry->info != (char *) NULL)
   {
      /* We need to adjust the field info.				*/
      if (infoLength > entry->fieldwidth)
      {
         /* The field information is larger than the field.		*/
         entry->leftchar = (infoLength - entry->fieldwidth);
         entry->screencol = entry->fieldwidth-1;
      }
      else
      {
         /* The field information is smaller than the field.		*/
         entry->leftchar = 0;
         entry->screencol = infoLength-1;
      }
   }
 
   /* Redraw the field.							*/
   for (x=entry->leftchar; x < infoLength; x++)
   {
      mvwaddch (entry->field, 0, x-entry->leftchar, entry->info[x]);
   }
   wmove (entry->field, 0, entry->screencol);

   /* Refresh the field.						*/
   wrefresh (entry->field);
}

/*
 * This erases an entry widget from the screen.
 */
void eraseCDKEntry (CDKENTRY *entry)
{
   werase (entry->field);
   werase (entry->labelwin);
   werase (entry->win);
   wrefresh (entry->win);
  
   /* Do we want to erase the shadow.					*/
   if (entry->shadow)
   {
      werase (entry->shadowwin);
      wrefresh (entry->shadowwin);
   }
}

/*
 * This destroys an entry widget.
 */
void destroyCDKEntry (CDKENTRY *entry)
{
   /* Erase the object.							*/
   eraseCDKEntry (entry);

   /* Clear out the character pointers.					*/
   freeChtype (entry->label);
   freeChar (entry->info);

   /* Delete the windows.						*/
   delwin (entry->win);
   delwin (entry->field);
   delwin (entry->labelwin);
   if (entry->shadow)
   {
      delwin (entry->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vENTRY, entry);

   /* Finish cleaning up.						*/
   free (entry);
}

/*
 * This removes the old information in the entry field and keeps the
 * new information given.
 */
void setCDKEntry (CDKENTRY *entry, char *newValue, int min, int max, boolean box)
{
   /* Declare local variables.						*/
   int copychars	= 0;

   /* Keep the int values passed in.					*/
   entry->min		= min;
   entry->max		= max;
   entry->box		= box;

   /* If the pointer sent in is the same pointer as before, do nothing.	*/
   if (entry->info == newValue)
   {
      return;
   }

   /* Just to be sure, if lets make sure the new value isn't NULL.	*/
   if (newValue == (char *)NULL)
   { 
      /* Then we want to just erase the old value.			*/
      cleanChar (entry->info, entry->fieldwidth, '\0');
      return;
   }

   /* Determine how many characters we need to copy.			*/
   copychars	= MIN ((int)strlen(newValue), entry->max);

   /* OK, erase the old value, and copy in the new value.		*/
   cleanChar (entry->info, entry->max, '\0');
   strncpy (entry->info, newValue, copychars);
}
