#include "cdk.h"
 
/*
 * $Author: mikeg $
 * $Date: 1995/12/12 15:32:12 $
 * $Revision: 1.14 $
 */
 
/* Declare local function prototypes.					*/
void getDirContents (CDKFSELECT *fselect);
void displayFileinfo (EObjectType objectType, void *object, void *clientData);
void getDirName (EObjectType objectType, void *object, void *clientData);
void deleteFile (EObjectType objectType, void *object, void *clientData);

/*
 * This creates a file selection widget.
 */
CDKFSELECT *newCDKFselect (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, chtype fieldAttribute, chtype fillerChar, chtype highlight, char *dAttribute, char *fAttribute, char *lAttribute, char *sAttribute, boolean box, boolean shadow)
{
  /* Set up some variables.                                            */
   CDKFSELECT *fselect	= (CDKFSELECT *)malloc (sizeof (CDKFSELECT));
   int boxWidth		= width;
   int boxHeight	= height;
   int xpos		= xplace;
   int ypos		= yplace;
   int entryWidth, x;

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Set the height and width values.					*/
   boxWidth = (width < 15 ? 15 : width);
   boxHeight = (height < 6 ? 6 : height);

   /* Make the file selector window.					*/
   fselect->win		= newwin (boxHeight, boxWidth, ypos, xpos);

   /* Is the window NULL???						*/
   if (fselect->win == (WINDOW *)NULL)
   {
      return ( (CDKFSELECT *)NULL );
   }

   /* Set some variables.						*/
   fselect->screen		= cdkscreen;
   fselect->parent		= cdkscreen->window;
   fselect->dirAttribute	= copyChar (dAttribute);
   fselect->fileAttribute	= copyChar (fAttribute);
   fselect->linkAttribute	= copyChar (lAttribute);
   fselect->sockAttribute	= copyChar (sAttribute);
   fselect->highlight		= highlight;
   fselect->fillerCharacter	= fillerChar;
   fselect->fieldAttribute	= fieldAttribute;
   fselect->height		= boxHeight;
   fselect->width		= boxWidth;
   fselect->fileCounter		= 0;
   fselect->pwd			= (char *)NULL;
   fselect->box			= box;
   fselect->shadow		= shadow;

   /* Zero out the contents of the directory listing.			*/
   for (x=0; x < MAXITEMS; x++)
   {
      fselect->dirContents[x]	= (char *)NULL;
   }

   /* Get the present working directory.				*/
   fselect->pwd	= getcwd (NULL, 512);

   /* Get the contents of the current directory.			*/
   getDirContents (fselect);

   /* Create the entry field in the selector.				*/
   entryWidth = boxWidth - 8;
   fselect->filename	= newCDKEntry (cdkscreen,
			(fselect->win)->_begx, 
			(fselect->win)->_begy,
			LEFT,
			"File:", fieldAttribute, fillerChar, 
			vMIXED, entryWidth, 0, 512, &CDKEntryCallBack, 
			TRUE, FALSE);

   /* Put the current working directory in the entry field.		*/
   setCDKEntry (fselect->filename, fselect->pwd, 0, 512, (fselect->filename)->box);
 
   /* Create the scrolling list in the selector.			*/
   fselect->filelist	= newCDKScroll (cdkscreen,
			(fselect->win)->_begx,
			(fselect->win)->_begy + 2,
			boxHeight - 3, boxWidth,
			(char *)NULL,
			fselect->dirContents, fselect->fileCounter,
			NONUMBERS, fselect->highlight, TRUE, FALSE);

   /* Define the callbacks for the scrolling list.			*/
   bindCDKObject (vSCROLL, fselect->filelist, '?', displayFileinfo, fselect);
   bindCDKObject (vSCROLL, fselect->filelist, 'q', displayFileinfo, fselect);
   bindCDKObject (vSCROLL, fselect->filelist, KEY_TAB, getDirName, fselect);
   bindCDKObject (vSCROLL, fselect->filelist, DELETE, deleteFile, fselect);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vFSELECT, fselect);

   /* Return the file selector pointer.					*/
   return ( fselect );
}

/*
 * This erases the file selector from the screen.
 */
void eraseCDKFselect (CDKFSELECT *fselect)
{
   eraseCDKScroll (fselect->filelist);
   eraseCDKEntry (fselect->filename);
   werase (fselect->win);
}

/*
 * This draws the file selector widget.
 */
void drawCDKFselect (CDKFSELECT *fselect, boolean Box)
{
   /* Erase the old file selector.					*/
   eraseCDKFselect (fselect);

   /* Draw in the shadow if we need to.					*/
   if (fselect->shadow)
   {
      drawShadow (fselect->shadowwin);
   }

   /* Draw in the scrolling list.					*/
   drawCDKScroll (fselect->filelist, Box);

   /* Draw in the entry field.						*/
   drawCDKEntry (fselect->filename, Box);
}

/*
 * This creates a list of the files in the current directory.
 */
void getDirContents (CDKFSELECT *fselect)
{
   /* Declare local variables.						*/
   struct dirent *dirStruct;
   struct stat fileStat;
   char *dirList[MAXITEMS];
   char temp[200], mode;
   DIR *dp;
   int counter = 0;
   int x = 0;

   /* Clean out the old directory list.					*/
   for (counter = 0; counter < fselect->fileCounter; counter++)
   {
      freeChar (fselect->dirContents[counter]);
   }

   /* Open the directory.						*/
   dp = opendir (fselect->pwd);

   /* Read the directory.						*/
   counter = 0;
   while ( (dirStruct = readdir (dp)) != NULL)
   {
      dirList[counter] = copyChar (dirStruct->d_name);
      counter++;
   }
   fselect->fileCounter	= counter;

   /* Sort the info.							*/
   quickSort (dirList, 0, fselect->fileCounter-1);

   /* Set the properties of the files.					*/
   for (x=0; x < fselect->fileCounter; x++)
   {
      /* Stat the file.							*/
      lstat (dirList[x], &fileStat);

      /* Check the mode.						*/
      mode = ' ';
      if ( ((fileStat.st_mode & S_IXUSR) != 0) ||
		((fileStat.st_mode & S_IXGRP) != 0) ||
		((fileStat.st_mode & S_IXOTH) != 0) )
      {
          mode = '*';
      }

      /* Create the filename.						*/
      if ( S_ISLNK (fileStat.st_mode) )
      {
         sprintf (temp, "%s@", dirList[x]);
         fselect->dirContents[x] = copyChar (temp);
      }
      else if ( S_ISSOCK (fileStat.st_mode) )
      {
         sprintf (temp, "%s&", dirList[x]);
         fselect->dirContents[x] = copyChar (temp);
      }
      else if ( S_ISREG(fileStat.st_mode) )
      {
         sprintf (temp, "%s%s%c", fselect->fileAttribute, dirList[x], mode);
         fselect->dirContents[x] = copyChar (temp);
      }
      else if ( S_ISDIR(fileStat.st_mode) )
      {
         sprintf (temp, "%s%s/", fselect->dirAttribute, dirList[x]);
         fselect->dirContents[x] = copyChar (temp);
      }
      else
      {
         sprintf (temp, "%s%c", dirList[x], mode);
         fselect->dirContents[x] = copyChar (temp);
      }

      /* Free up this piece of memory.					*/
      freeChar (dirList[x]);
   }

   /* Close the directory.						*/
   closedir (dp);
}

/*
 * This activates the file selector.
 */
char *activateCDKFselect (CDKFSELECT *fselect)
{
   /* Declare local variables.						*/
   CDKSCROLL	*fscroll = fselect->filelist;
   struct stat	fileStat;
   char 	*filename;
   int 		choice;
   boolean	regularFile;
   char		temp[100];

   /* Do this forever....						*/
   for (;;)
   {
      /* Activate the scrolling list.					*/
      choice = activateCDKScroll ( fscroll );

      /* Check the value of the choice.					*/
      if (choice == -1)
      {
         return ((char *)NULL);
      }

      /* Create the filename.						*/
      filename = chtype2Char (fscroll->item[fscroll->currentItem]);
      filename[strlen(filename)-1] = '\0';

      /* Determine the file type.					*/
      lstat (filename, &fileStat);
      regularFile	= ( S_ISREG(fileStat.st_mode) ? 1 : 0);
      if (regularFile)
      {
         /* It's a regular file, create the full path.			*/
         sprintf (temp, "%s/%s", fselect->pwd, filename);
         fselect->pathname = copyChar (temp);
      
         /* Clean up the used memory.					*/
         free (filename);

         /* Return the complete pathname.				*/
         return ( fselect->pathname );
      }
      else
      {
         /* Create the directory name.					*/
         sprintf (temp, "%s/%s", fselect->pwd, filename);

         /* Clean up the used memory.					*/
         free (filename);

         /* Set the file selector information.				*/
         setCDKFselect (fselect, temp,
		fselect->fieldAttribute, fselect->fillerCharacter, fselect->highlight,
		fselect->dirAttribute, fselect->fileAttribute, 
		fselect->linkAttribute, fselect->sockAttribute, 
		fselect->box);
      }
   }
}

/* 
 * This is a callback from the scrolling list to activate the entry field.
 */
void getDirName (EObjectType objectType, void *object, void *clientData)
{
   /* Declare local variables.						*/
   CDKFSELECT	*fselect	= (CDKFSELECT *)clientData;
   CDKENTRY	*fentry		= (CDKENTRY *)fselect->filename;
   struct stat	fileStat;
   char		*directory;

   /* Activate the entry field.						*/
   directory = copyChar (activateCDKEntry (fentry));

   /* Check the directory/filename					*/
   lstat (directory, &fileStat);
   
   /* Set the file selector information.				*/
   setCDKFselect (fselect, directory,
			fselect->fieldAttribute, fselect->fillerCharacter, fselect->highlight,
			fselect->dirAttribute, fselect->fileAttribute, 
			fselect->linkAttribute, fselect->sockAttribute, 
			fselect->box);

   /* Clean up any memorty used.					*/
   free (directory);
}

/*
 * This function sets the information inside the file selector.
 */
void setCDKFselect (CDKFSELECT *fselect, char *directory, chtype fieldAttrib, chtype filler, chtype highlight, char *dirAttribute, char *fileAttribute, char *linkAttribute, char *sockAttribute, boolean box)
{
   /* Declare local variables.						*/
   CDKSCROLL *fscroll = fselect->filelist;
   CDKENTRY *fentry = fselect->filename;
   char *mesg[10], temp[100];

   /* Keep the info sent to us.						*/
   fselect->fieldAttribute = fieldAttrib;
   fselect->fillerCharacter = filler;
   fselect->highlight = highlight;

   /* Change directories.						*/
   if (chdir (directory))
   {
      /* Beep at them.							*/
      Beep();

      /* Couldn't get into the directory, pop up a little message.	*/
      sprintf (temp, "<C>Could not change into %s", directory);
      mesg[0] = copyChar (temp);

      sprintf (temp, "<C></U>%s", strerror(errno) );
      mesg[1] = copyChar (temp);

      mesg[2] = " ";
      mesg[3] = "<C>Press Any Key To Continue.";
      
      /* Pop Up a message.						*/
      popUpMessage (fselect->screen, mesg, 4);

      /* Clean up some memory.						*/
      freeChar (mesg[0]);
      freeChar (mesg[1]);

      /* Get out of here...						*/
      return;
   }

   /* If the information coming in is the same as the information	*/
   /* that is already there, there is no need to destroy it.		*/
   if (fselect->pwd != directory)
   {
      /* Remove the old pointer	and set the new value.			*/
      freeChar (fselect->pwd);
      fselect->pwd = getcwd (NULL, 512);
   }
   if (fselect->fileAttribute != fileAttribute)
   {
      /* Remove the old pointer	and set the new value.			*/
      freeChar (fselect->fileAttribute);
      fselect->fileAttribute = copyChar (fileAttribute);
      fselect->fileAttribute = copyChar (fileAttribute);
   }
   if (fselect->dirAttribute != dirAttribute)
   {
      /* Remove the old pointer	and set the new value.			*/
      freeChar (fselect->dirAttribute);
      fselect->dirAttribute = copyChar (dirAttribute);
   }
   if (fselect->linkAttribute != linkAttribute)
   {
      /* Remove the old pointer	and set the new value.			*/
      freeChar (fselect->linkAttribute);
      fselect->linkAttribute = copyChar (linkAttribute);
   }
   if (fselect->sockAttribute != sockAttribute)
   {
      /* Remove the old pointer	and set the new value.			*/
      freeChar (fselect->sockAttribute);
      fselect->sockAttribute = copyChar (sockAttribute);
   }
   
   /* Set the contents of the entry field.				*/
   setCDKEntry (fentry, fselect->pwd, 0, 512, fentry->box);
   drawCDKEntry (fentry, fentry->box);

   /* Get the directory contents.					*/
   getDirContents (fselect);

   /* Set the values in the scrolling list.				*/
   setCDKScroll (fscroll,
		fselect->dirContents, fselect->fileCounter,
		FALSE, fscroll->highlight, fscroll->box);
}

/*
 * This destroys the file selector.	
 */
void destroyCDKFselect (CDKFSELECT *fselect)
{
   int x;

   /* Erase the file selector.						*/
   eraseCDKFselect (fselect);

   /* Free up the character pointers.					*/
   freeChar (fselect->pwd);
   freeChar (fselect->pathname);
   freeChar (fselect->dirAttribute);
   freeChar (fselect->fileAttribute);
   freeChar (fselect->linkAttribute);
   freeChar (fselect->sockAttribute);
   for (x=0; x < fselect->fileCounter; x++)
   {
      freeChar (fselect->dirContents[x]);
   }

   /* Free up the window pointers. 					*/
   if (fselect->shadow)
   {
      delwin (fselect->shadowwin);
   }
   delwin (fselect->win);

   /* Destroy the other Cdk objects.					*/
   destroyCDKEntry (fselect->filename);
   destroyCDKScroll (fselect->filelist);
 
   /* Unregister the object.						*/
   unregisterCDKObject (vFSELECT, fselect);

   /* Free up the object pointer.					*/
   free (fselect);
}

/*
 ********************************
 * Callback functions.
 ********************************
 */

/* 
 * This is a callback to the scrolling list which displays information
 * about the current file. (and the whole directory as well)
 */
void displayFileinfo (EObjectType objectType, void *object, void *clientData)
{
   /* Declare local variables.						*/
   CDKSCROLL		*fscroll	= (CDKSCROLL *)object;
   CDKFSELECT		*fselect	= (CDKFSELECT *)clientData;
   CDKLABEL		*infoLabel;
   struct stat		fileStat;
   struct passwd	*pwEnt;
   struct group		*grEnt;
   char			*filename;
   char			*filetype;
   char 		*mesg[10];
   char			temp[100];
   int			len;

   /* Get the file name.						*/
   filename	= chtype2Char (fscroll->item[fscroll->currentItem]);
   filename[strlen(filename)-1] = '\0';

   /* Get specific information about the files.				*/
   lstat (filename, &fileStat);
   
   /* Determine the file type.						*/
   if ( S_ISLNK (fileStat.st_mode) )
   {
      filetype = "Link";
   }
   else if ( S_ISSOCK (fileStat.st_mode) )
   {
      filetype = "Socket";
   }
   else if ( S_ISREG(fileStat.st_mode) )
   {
      filetype = "Regular File";
   }
   else if ( S_ISDIR(fileStat.st_mode) )
   {
      filetype = "Directory";
   }
   else if ( S_ISCHR(fileStat.st_mode) )
   {
      filetype = "Character Device";
   }
   else if ( S_ISBLK(fileStat.st_mode) )
   {
      filetype = "Block Device";
   }
   else if ( S_ISFIFO(fileStat.st_mode) )
   {
      filetype = "FIFO Device";
   }
   else
   {
      filetype = "Unknown";
   }

   /* Get the user name and group name.					*/
   pwEnt = getpwuid (fileStat.st_uid);
   grEnt = getgrgid (fileStat.st_gid);

   /* Create the message.						*/
   sprintf (temp, "Directory  : </U>%s", fselect->pwd);
   mesg[0] = copyChar (temp);

   sprintf (temp, "Filename   : </U>%s", filename);
   mesg[1] = copyChar (temp);

   sprintf (temp, "Owner      : </U>%s<!U> (%d)", pwEnt->pw_name, fileStat.st_uid);
   mesg[2] = copyChar (temp);

   sprintf (temp, "Group      : </U>%s<!U> (%d)", grEnt->gr_name, fileStat.st_gid);
   mesg[3] = copyChar (temp);

   sprintf (temp, "Size       : </U>%ld<!U> bytes", fileStat.st_size);
   mesg[4] = copyChar (temp);

   sprintf (temp, "Last Access: </U>%s", ctime (&fileStat.st_atime) );
   len = strlen (temp);
   temp[len] = '\0'; temp[len-1] = '\0';
   mesg[5] = copyChar (temp);

   sprintf (temp, "Last Change: </U>%s", ctime (&fileStat.st_ctime) );
   len = strlen (temp);
   temp[len] = '\0'; temp[len-1] = '\0';
   mesg[6] = copyChar (temp);

   sprintf (temp, "File Type  : </U>%s", filetype);
   mesg[7] = copyChar (temp);

   mesg[8] = " ";
   mesg[9] = "<C>Press Any Key To Continue.";

   /* Create the pop up label.						*/
   infoLabel	= newCDKLabel (fscroll->screen, CENTER, CENTER, mesg, 8, TRUE, FALSE);
   drawCDKLabel (infoLabel, TRUE);
   wgetch (infoLabel->win);

   /* Clean up some memory.						*/
   destroyCDKLabel (infoLabel);
   freeChar (filename);
   freeChar (mesg[0]); freeChar (mesg[1]);
   freeChar (mesg[2]); freeChar (mesg[3]);
   freeChar (mesg[4]); freeChar (mesg[5]);
   freeChar (mesg[6]); freeChar (mesg[7]);

   /* Redraw the file selector.						*/
   drawCDKFselect (fselect, fselect->box);
}

/*
 * This allows the user to delete a file.
 */
void deleteFile (EObjectType objectType, void *object, void *clientData)
{
   /* Declare local variables.						*/
   CDKSCROLL	*fscroll	= (CDKSCROLL *)object;
   CDKFSELECT	*fselect	= (CDKFSELECT *)clientData;
   char		*buttons[]	= {"No", "Yes"};
   CDKDIALOG	*question;
   char		*mesg[10], *filename, temp[100];

   /* Get the filename which is to be deleted.				*/
   filename = chtype2Char (fscroll->item[fscroll->currentItem]);
   filename[strlen(filename)-1] = '\0';

   /* Create the dialog message.					*/
   mesg[0] = "<C>Are you sure you want to delete the file:";
   sprintf (temp, "<C></U>%s?", filename); mesg[1] = copyChar (temp);

   /* Create the dialog box.						*/
   question = newCDKDialog (fselect->screen, CENTER, CENTER,
			mesg, 2, buttons, 2, A_REVERSE,
			TRUE, TRUE, FALSE);

   /* Clean up this pointer. (we may need it again real soon.)		*/
   freeChar (mesg[1]);

   /* If the said yes then try to nuke it.				*/
   if (activateCDKDialog (question) == 1)
   {
      /* If we were successful, reload the scrolling list.		*/
      if (unlink (filename) == 0)
      {
         /* Set the file selector information.				*/
         setCDKFselect (fselect, fselect->pwd,
		fselect->fieldAttribute, fselect->fillerCharacter, fselect->highlight,
		fselect->dirAttribute, fselect->fileAttribute, 
		fselect->linkAttribute, fselect->sockAttribute, 
		fselect->box);
      }
      else
      {
         /* Pop up a message.						*/
         sprintf (temp, "<C>Can't delete file: <%s>", strerror (errno) );
         mesg[0] = copyChar (temp);
         mesg[1] = " ";
         mesg[2] = "<C>Press any key to continue.";
         popUpMessage (fselect->screen, mesg, 3);
         freeChar (mesg[0]);
      }
   }

   /* Clean up.								*/
   destroyCDKDialog (question);

   /* Redraw the file selector.						*/
   drawCDKFselect (fselect, fselect->box);
}
