#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/15 23:49:30 $
 * $Revision: 1.23 $
 */

/*
 * This creates a graph widget.
 */
CDKGRAPH *newCDKGraph (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char *title, char *xtitle, char *ytitle)
{
   /* Declare local variables.						*/
   CDKGRAPH *graph	= (CDKGRAPH *)malloc (sizeof (CDKGRAPH));
   int xpos		= xplace;
   int ypos		= yplace;

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, width+3, height);

   /* Create the graph pointer						*/
   graph->parent	= cdkscreen->window;
   graph->win		= newwin (height, width+3, ypos, xpos);
   graph->boxHeight	= height;
   graph->boxWidth	= width+3;
   graph->box		= NOBOX;
   graph->minx		= 0;
   graph->maxx		= 0;
   graph->xscale	= 0;
   graph->yscale	= 0;
   graph->displayType	= vLINE;

   /* Is the graph pointer NULL???					*/
   if (graph->win == (WINDOW *)NULL)
   {
      /* Clean up any memory used.					*/
      free (graph);

      /* Return a NULL pointer.						*/
      return ( (CDKGRAPH *)NULL );
   }

   /* Translate the graph title char * to a chtype *			*/
   if (title != (char *)NULL)
   {
      graph->title	= char2Chtype (title, &graph->titlelen, &graph->titlepos);
      graph->titlepos	= justifyString (graph->boxWidth, graph->titlelen, graph->titlepos);
   }
   else
   {
      graph->title	= char2Chtype ("<C></5>Untitled", &graph->titlelen, &graph->titlepos);
      graph->titlepos	= justifyString (graph->boxWidth, graph->titlelen, graph->titlepos);
   }

   /* Translate the X Axis title char * to a chtype *			*/
   if (xtitle != (char *)NULL)
   {
      graph->xtitle	= char2Chtype (xtitle, &graph->xtitlelen, &graph->xtitlepos);
      graph->xtitlepos	= justifyString (graph->boxHeight, graph->xtitlelen, graph->xtitlepos);
   } 
   else
   {
      graph->xtitle	= char2Chtype ("<C></5>X Axis", &graph->xtitlelen, &graph->xtitlepos);
      graph->xtitlepos	= justifyString (graph->boxHeight, graph->xtitlelen, graph->xtitlepos);
   }

   /* Translate the Y Axis title char * to a chtype *			*/
   if (ytitle != (char *)NULL)
   {
      graph->ytitle	= char2Chtype (ytitle, &graph->ytitlelen, &graph->ytitlepos);
      graph->ytitlepos	= justifyString (graph->boxWidth, graph->ytitlelen, graph->ytitlepos);
   } 
   else
   {
      graph->ytitle	= char2Chtype ("<C></5>Y Axis", &graph->ytitlelen, &graph->ytitlepos);
      graph->ytitlepos	= justifyString (graph->boxWidth, graph->ytitlelen, graph->ytitlepos);
   }

   /* Set some values of the graph structure.				*/
   graph->graphchar	= (chtype *)NULL;

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vGRAPH, graph);

   /* Return the graph pointer.						*/
   return (graph);
}

/*
 * This sets the values of the graph.
 */
int setCDKGraph (CDKGRAPH *graph, int *values, int count, char *graphChars, boolean startAtZero, EGraphDisplayType displayType)
{
   /* Declare local variables.						*/
   int min	= 99999999;
   int max	= (min * -1);
   int x	= 0;
   int charCount, temp;

   /* Keep the display type.						*/
   graph->displayType	= displayType;

   /* Remove the old graph char if it exists.				*/
   freeChtype (graph->graphchar);
   graph->graphchar = char2Chtype (graphChars, &charCount, &temp);

   /* Check if the number of characters back is the same as the number	*/
   /* of elements in the list.						*/
   if (charCount < count)
   {
      /* Oops. Better let them know.					*/
      freeChtype (graph->graphchar);
      return (FALSE);
   }

   /* Copy the X values.						*/
   for (x=0; x <count; x++)
   {
      /* Determine the min/max values of the graph.			*/
      min		= MIN (values[x], min);
      max		= MAX (values[x], max);

      /* Copy the value.						*/
      graph->values[x]	= values[x];
   }
   graph->count = count;
   graph->minx = min;
   graph->maxx = max;

   /* Check the start at zero status.					*/
   if (startAtZero)
   {
      graph->minx = 0;
   }

   /* Determine the scales.						*/
   graph->xscale = ((graph->maxx - graph->minx) / (graph->boxHeight - 5));
   graph->yscale = ((graph->boxWidth-4) / count);
   return (TRUE);
}

/*
 * This function draws the graph widget.
 */
void drawCDKGraph (CDKGRAPH *graph, boolean Box)
{
   /* Declare local variables.						*/
   int x		= 0;
   int y		= 0;
   int adj		= 0;
   chtype attrib	= ' '|A_REVERSE;
   char temp[100];

   /* Keep the box information.						*/
   graph->box		= Box;

   /* Erase the old object...						*/
   eraseCDKGraph (graph);

   /* Draw in the vertical axis						*/
   drawLine (graph->win, 2, 1, 2, graph->boxHeight-3, ACS_VLINE);

   /* Draw in the horizontal axis					*/
   drawLine (graph->win, 3, graph->boxHeight-3, graph->boxWidth, graph->boxHeight-3, ACS_HLINE);

   /* Draw in the graph title.						*/
   if (graph->title != (chtype *)NULL)
   {
      writeChtype (graph->win, graph->titlepos, 0, graph->title, HORIZONTAL, 0, graph->titlelen);
   }
   
   /* Draw in the X axis title.						*/
   if (graph->xtitle != (chtype *)NULL)
   {
      writeChtype (graph->win, 0, graph->xtitlepos, graph->xtitle, VERTICAL, 0, graph->xtitlelen);
      attrib	= graph->xtitle[0] & A_ATTRIBUTES;
   }
   
   /* Draw in the X axis high value.					*/
   sprintf (temp, "%d", graph->maxx);
   writeCharAttrib (graph->win, 1, 1, temp, attrib, VERTICAL, 0, strlen(temp));
   
   /* Draw in the X axis low value.					*/
   sprintf (temp, "%d", graph->minx);
   writeCharAttrib (graph->win, 1, graph->boxHeight-2-strlen(temp), temp, attrib, VERTICAL, 0, strlen(temp));
   
   /* Draw in the Y axis title.						*/
   if (graph->ytitle != (chtype *)NULL)
   {
      writeChtype (graph->win, graph->ytitlepos, graph->boxHeight-1, graph->ytitle, HORIZONTAL, 0, graph->ytitlelen);
      attrib	= graph->ytitle[0] & A_ATTRIBUTES;
   }

   /* Draw in the Y axis high value.					*/
   sprintf (temp, "%d", graph->count);
   writeCharAttrib (graph->win, graph->boxWidth-strlen(temp)-1, graph->boxHeight-2, temp, attrib, HORIZONTAL, 0, strlen(temp));
   
   /* Draw in the Y axis low value.					*/
   sprintf (temp, "0");
   writeCharAttrib (graph->win, 3, graph->boxHeight-2, temp, attrib, HORIZONTAL, 0, strlen(temp));
   
   /* Draw in the lines.						*/
   adj	= 3;
   for (y=0; y < graph->count; y++)
   {
       int colheight	= (graph->values[y] / graph->xscale) - 1;
       mvwaddch (graph->win, graph->boxHeight-3, adj-1, ACS_TTEE);

       /* If this is a plot graph, all we do is draw a dot.		*/
       if (graph->displayType == vPLOT)
       {
          int temploc = graph->boxHeight-4-colheight;
          mvwaddch (graph->win, temploc, adj, graph->graphchar[y]);
          adj += 2;
       } 
       else
       {
          for (x=0; x <= graph->yscale; x++)
          {
             /* Draw a line.						*/
             drawLine (graph->win, adj, graph->boxHeight-4-colheight, adj, graph->boxHeight-4, graph->graphchar[y]);
             adj++;
          }
       }
   }

   /* Draw in the axis corners.						*/
   mvwaddch (graph->win, 0, 2, ACS_URCORNER);
   mvwaddch (graph->win, graph->boxHeight-3, 2, ACS_LLCORNER);
   mvwaddch (graph->win, graph->boxHeight-3, graph->boxWidth, ACS_URCORNER);

   /* Refresh and lets see 'er.						*/
   wrefresh (graph->win);
   wrefresh (graph->parent);
}

/*
 * This function destroys the graph widget.
 */
void destroyCDKGraph (CDKGRAPH *graph)
{
   /* Erase the object.							*/
   eraseCDKGraph (graph);

   /* Clear up the char pointers.					*/
   freeChtype (graph->title);
   freeChtype (graph->xtitle);
   freeChtype (graph->ytitle);
   freeChtype (graph->graphchar);
      
   /* Clean up the windows.						*/
   delwin (graph->win);

   /* Unregister this object.						*/
   unregisterCDKObject (vGRAPH, graph);

   /* Finish cleaning up.						*/
   free (graph);
}

/*
 * This function erases the graph widget from the screen.
 */
void eraseCDKGraph (CDKGRAPH *graph)
{
   werase (graph->win);
   wrefresh (graph->win);
}
