#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.30 $
 */

/*
 * This creates a histogram widget.
 */
CDKHISTOGRAM *newCDKHistogram (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, int height, int width, int orient, char *label, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKHISTOGRAM *histogram	= (CDKHISTOGRAM *)malloc (sizeof (CDKHISTOGRAM));
   int boxwidth			= 0;
   int boxheight		= height + 2;
   int xpos			= xplace;
   int ypos			= yplace;
   int labelxoffset		= 0;
   int labelyoffset		= 0;
   int fieldxoffset		= 0;
   int fieldyoffset		= 0;
   int junk			= 0;
   char *plainLabel;

   /* Translate the char * to a chtype *				*/
   histogram->label	= char2Chtype (label, &histogram->labellen, &junk);
   plainLabel		= chtype2Char (histogram->label);
   boxwidth		= width + histogram->labellen + 3;

   /* Adjust the offset values given the location of the label.		*/
   alignlabel (plainLabel, lplace, width, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainLabel);

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Create the histogram pointer.					*/
   histogram->parent	= cdkscreen->window;
   histogram->win	= newwin (boxheight, boxwidth, ypos, xpos);
   histogram->shadowwin	= (WINDOW *)NULL;
   histogram->labelwin	= subwin (cdkscreen->window, 1, histogram->labellen+2, ypos+labelyoffset, xpos+labelxoffset);
   histogram->width	= width;
   histogram->height	= height;
   histogram->orient	= orient;
   histogram->shadow	= shadow;

   /* Is the window NULL.						*/
   if (histogram->win == (WINDOW *)NULL)
   {
      /* Remove the labelwin if its not NULL.				*/
      if ( histogram->labelwin != (WINDOW *)NULL)
      {
         delwin (histogram->labelwin);
      }

      /* Clean up any memory used.					*/
      freeChtype (histogram->label);
      free (histogram);

      /* Return a NULL pointer.						*/
      return ( (CDKHISTOGRAM *)NULL );
   }

   /* Set up some default values.					*/
   histogram->filler	= '#' | A_REVERSE;
   histogram->statsattr	= A_NORMAL;
   histogram->statspos	= TOP;
   histogram->viewtype	= vREAL;
   histogram->high	= 0;
   histogram->low	= 0;
   histogram->value	= 0;
   histogram->chars	= 0;
   histogram->lowx	= 0;
   histogram->lowy	= 0;
   histogram->highx	= 0;
   histogram->highy	= 0;
   histogram->labelx	= 0;
   histogram->labely	= 0;
   histogram->statsx	= 0;
   histogram->statsy	= 0;
   histogram->histx	= 0;
   histogram->histy	= 0;
   histogram->lowchar	= (char *)NULL;
   histogram->highchar	= (char *)NULL;
   histogram->statschar	= (char *)NULL;
   histogram->box	= box;

   /* Do we want a shadow???						*/
   if (shadow)
   {
      histogram->shadowwin	= subwin (cdkscreen->window, boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vHISTOGRAM, histogram);

   /* Return this thing.						*/
   return (histogram);
}

/*
 * This sets the values for the histogram.
 */
void setCDKHistogram (CDKHISTOGRAM *histogram, EHistogramDisplayType viewtype, int statspos, chtype statsattr, int low, int high, int value, chtype filler, boolean box)
{
   /* Declare local variables.						*/
   char		string[100];
   int		adj;
   float	temp;

   /* Keep the box info.						*/
   histogram->box	= box;

   /* We should error check the information we have.			*/
   histogram->low	= (low <= high ? low : 0);
   histogram->high	= (low <= high ? high : 0);
   histogram->value	= ( low <= value && value <= high ? value : 0);

   /* Set some of the basic values.					*/
   histogram->statspos	= statspos;
   histogram->statsattr	= statsattr;
   histogram->filler	= filler;
   histogram->viewtype	= viewtype;
   histogram->percent	= (histogram->high == 0 ? 0 : ( (float)histogram->value / (float)histogram->high ));
   if (histogram->orient == VERTICAL)
   {
      temp		= histogram->percent * (float)histogram->height;
   } 
   else
   {
      temp		= histogram->percent * (float)histogram->width;
   }
   histogram->chars	= ( (float)abs(temp) == (float)temp ? (int)temp : (int)temp + 1);

   /* We have a number of variables which determine the personality of	*/
   /* the histogram. We have to go through each one methodically, and 	*/
   /* set them correctly. This section does this.			*/
   if (histogram->viewtype != vNONE)
   {
      if (histogram->orient == VERTICAL)
      {
         /* Set the vertical adjustment var				*/
         adj = (histogram->label != (chtype *)NULL ? histogram->labellen + 1 : 0);

         /* Make a vertical histogram the exception.			*/
         if (histogram->statspos == LEFT || histogram->statspos == TOP)
         {
            /* Set the character strings correctly.			*/
            freeChar (histogram->lowchar);
            freeChar (histogram->highchar);
            freeChar (histogram->statschar);

            /* Set the low label attributes.				*/
            sprintf (string, "%d", histogram->low);
            histogram->lowchar		= copyChar (string);
            histogram->lowx		= histogram->height - (int)strlen (string) + 2;
            histogram->lowy		= adj;
            
            /* Set the high label attributes.				*/
            sprintf (string, "%d", histogram->high);
            histogram->highchar		= copyChar (string);
            histogram->highx		= 0;
            histogram->highy		= adj;
   
            /* Set the stats label attributes.				*/
            if (histogram->viewtype == vPERCENT)
            {
               sprintf (string, "%3.1f%%", (float) (histogram->percent * 100));
            }
            else if (histogram->viewtype == vFRACTION)
            {
               sprintf (string, "%d/%d", histogram->value, histogram->high);
            }
            else
            {
               sprintf (string, "%d", histogram->value);
            }
            histogram->statschar	= copyChar (string);
            histogram->statsx		= ((histogram->height - (int)strlen(string)) / 2) + 1;
            histogram->statsy		= adj;
         }
         else if (histogram->statspos == RIGHT || histogram->statspos == BOTTOM)
         {
            /* Set the character strings correctly.			*/
            freeChar (histogram->lowchar);
            freeChar (histogram->highchar);
            freeChar (histogram->statschar);
   
            /* Set the low label attributes.				*/
            sprintf (string, "%d", histogram->low);
            histogram->lowchar		= copyChar (string);
            histogram->lowx		= histogram->height - (int)strlen (string) + 2;
            histogram->lowy		= histogram->width + 1;
            
            /* Set the high label attributes.				*/
            sprintf (string, "%d", histogram->high);
            histogram->highchar		= copyChar (string);
            histogram->highx		= 0;
            histogram->highy		= histogram->width + 1;
   
            /* Set the stats label attributes.				*/
            if (histogram->viewtype == vPERCENT)
            {
               sprintf (string, "%3.2f%%", (float) (histogram->percent * 100));
            }
            else if (histogram->viewtype == vFRACTION)
            {
               sprintf (string, "%d/%d", histogram->value, histogram->high);
            }
            else
            {
               sprintf (string, "%d", histogram->value);
            }
            histogram->statschar	= copyChar (string);
            histogram->statsx		= ((histogram->height - (int)strlen(string)) / 2) + 1;
            histogram->statsy		= histogram->width + 1;
         }

         /* Set the histogram positions attributes.			*/
         histogram->histx		= histogram->height - histogram->chars + 1;
         histogram->histy		= adj;
      }
      else
      {
         /* Set the vertical adjustment var				*/
         adj = (histogram->label != (chtype *)NULL ? histogram->labellen + 1 : 0);

         /* Make horizontal the default.				*/
         if (histogram->statspos == TOP || histogram->statspos == LEFT)
         {
            /* Set the character strings correctly.			*/
            freeChar (histogram->lowchar);
            freeChar (histogram->highchar);
            freeChar (histogram->statschar);

            /* Set the low label attributes.    			*/
            sprintf (string, "%d", histogram->low);
            histogram->lowchar          = copyChar (string);
            histogram->lowx             = 0;
            histogram->lowy             = adj;
            
            /* Set the high label attributes.   			*/
            sprintf (string, "%d", histogram->high);
            histogram->highchar         = copyChar (string);
            histogram->highx            = 0;
            histogram->highy            = histogram->width - (int)strlen (string) + adj + 1;
   
            /* Set the stats label attributes.  			*/
            if (histogram->viewtype == vPERCENT)
            {
               sprintf (string, "%3.1f%%", (float) (histogram->percent * 100));
            }
            else if (histogram->viewtype == vFRACTION)
            {
               sprintf (string, "%d/%d", histogram->value, histogram->high);
            }
            else
            {
               sprintf (string, "%d", histogram->value);
            }
            histogram->statschar        = copyChar (string);
            histogram->statsx           = 0;
            histogram->statsy           = ((histogram->width - (int)strlen(string)) / 2) + adj + 1;
         }
         else if (histogram->statspos == BOTTOM || histogram->statspos == RIGHT)
         {
            /* Set the character strings correctly.			*/
            freeChar (histogram->lowchar);
            freeChar (histogram->highchar);
            freeChar (histogram->statschar);

            /* Set the low label attributes.    			*/
            sprintf (string, "%d", histogram->low);
            histogram->lowchar          = copyChar (string);
            histogram->lowx             = histogram->height + 1;
            histogram->lowy             = adj;
            
            /* Set the high label attributes.   			*/
            sprintf (string, "%d", histogram->high);
            histogram->highchar         = copyChar (string);
            histogram->highx            = histogram->height + 1;
            histogram->highy            = histogram->width - (int)strlen (string) + adj + 1;
   
            /* Set the stats label attributes.  			*/
            if (histogram->viewtype == vPERCENT)
            {
               sprintf (string, "%3.1f%%", (float) (histogram->percent * 100));
            }
            else if (histogram->viewtype == vFRACTION)
            {
               sprintf (string, "%d/%d", histogram->value, histogram->high);
            }
            else
            {
               sprintf (string, "%d", histogram->value);
            }
            histogram->statschar        = copyChar (string);
            histogram->statsx           = histogram->height + 1;
            histogram->statsy           = ((histogram->width - (int)strlen(string)) / 2) + adj + 1;
         }
         else if (histogram->statspos == CENTER)
         {
            /* Set the character strings correctly.			*/
            freeChar (histogram->lowchar);
            freeChar (histogram->highchar);
            freeChar (histogram->statschar);

            /* Set the low label attributes.    			*/
            sprintf (string, "%d", histogram->low);
            histogram->lowchar          = copyChar (string);
            histogram->lowx             = (histogram->height == 1 ? 1 : (histogram->height / 2) );
            histogram->lowy             = adj;
            
            /* Set the high label attributes.   			*/
            sprintf (string, "%d", histogram->high);
            histogram->highchar         = copyChar (string);
            histogram->highx            = (histogram->height == 1 ? 1 : (histogram->height / 2) );
            histogram->highy            = histogram->width - (int)strlen (string) + adj + 1;
   
            /* Set the stats label attributes.  			*/
            if (histogram->viewtype == vPERCENT)
            {
               sprintf (string, "%3.1f%%", (float) (histogram->percent * 100));
            }
            else if (histogram->viewtype == vFRACTION)
            {
               sprintf (string, "%d/%d", histogram->value, histogram->high);
            }
            else
            {
               sprintf (string, "%d", histogram->value);
            }
            histogram->statschar        = copyChar (string);
            histogram->statsx           = (histogram->height == 1 ? 1 : (histogram->height / 2) );
            histogram->statsy           = ((histogram->width - (int)strlen(string)) / 2) + adj + 1;
         }
      }
   }
}

/*
 * This function draws the histogram.
 */
void drawCDKHistogram (CDKHISTOGRAM *histogram, boolean Box)
{
   /* Declare the local variables.					*/
   int x	= 0;
   int y	= 0;

   /* Erase the old object...						*/
   eraseCDKHistogram (histogram);

   /* Box the widget if asked.						*/
   if (Box)
   {
      box (histogram->win, histogram->statsattr | ACS_VLINE, histogram->statsattr |ACS_HLINE);
   }

   /* Do we have a shadow to draw???					*/
   if (histogram->shadow)
   {
      drawShadow (histogram->shadowwin);
   }

   /* Draw the histogram.						*/
   if (histogram->orient == VERTICAL)
   {
       /* Declare the local vars.					*/
       int temp	= (int)strlen (histogram->highchar);
       chtype fattr, fchar, battr, bchar;

       /* Draw in the high label					*/
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->highx + x,
					histogram->highy,
					histogram->highchar[x] | histogram->statsattr);
       }

       /* Draw in the low label						*/
       temp =(int)strlen (histogram->lowchar);
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->lowx + x,
					histogram->lowy,
					histogram->lowchar[x] | histogram->statsattr);
       }

       /* Draw in the stats label					*/
       temp =(int)strlen (histogram->statschar);
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->statsx + x,
					histogram->statsy,
					histogram->statschar[x] | histogram->statsattr);
       }

       /* Draw in the histogram.					*/
       temp	= histogram->labellen;
       fattr	= histogram->filler & A_ATTRIBUTES;
       fchar	= histogram->filler & A_CHARTEXT;

       for (x=histogram->histx; x <= histogram->height; x++)
       {
          for (y=1; y <= histogram->width; y++)
          {
#ifdef WINCHBUG
             battr	= ' ' | A_REVERSE;
#else
             battr	= mvwinch (histogram->win, x, y+temp );
#endif
             fchar	= battr & A_ATTRIBUTES;
             bchar	= battr & A_CHARTEXT;
             if (bchar == ' ')
             {
                mvwaddch (histogram->win, x, y+temp, histogram->filler);
             }
             else
             {
                mvwaddch (histogram->win, x, y+temp, battr | fattr);
             }
          }
       }
       wrefresh (histogram->win);

       /* Draw in the label						*/
       writeChtype (histogram->labelwin, 0, 0, histogram->label, HORIZONTAL, 0, histogram->labellen);
       wrefresh (histogram->labelwin);
   }
   else
   {
       /* Declare the local vars.					*/
       int temp	= (int)strlen (histogram->highchar);
       chtype fattr, fchar, battr, bchar;
       int x, y;

       /* Draw in the high label					*/
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->highx,
					histogram->highy + x,
					histogram->highchar[x] | histogram->statsattr);
       }

       /* Draw in the low label						*/
       temp	= (int)strlen (histogram->lowchar);
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->lowx,
					histogram->lowy + x,
					histogram->lowchar[x] | histogram->statsattr);
       }

       /* Draw in the stats label					*/
       temp	= (int)strlen (histogram->statschar);
       for (x=0; x < temp; x++)
       {
          mvwaddch (histogram->win,	histogram->statsx,
					histogram->statsy + x,
					histogram->statschar[x] | histogram->statsattr);
       }

       /* Set up some variables we need for the histogram.		*/
       temp	= histogram->labellen;
       fattr	= histogram->filler & A_ATTRIBUTES;
       fchar	= histogram->filler & A_CHARTEXT;

       /* Draw in the histogram.					*/
       for (x=1; x <= histogram->height; x++)
       {
          for (y=0; y < histogram->chars; y++)
          {
             /* Get the character and split it into its elements	*/
#ifdef WINCHBUG
             battr	= ' ' | A_REVERSE;
#else
             battr	= mvwinch (histogram->win, x, y+temp );
#endif
             bchar	= battr & A_CHARTEXT;
             fchar	= battr & A_ATTRIBUTES;

             /* If there was nothing there, put the histogram		*/
             /* down.							*/
             if (bchar == ' ')
             {
                mvwaddch (histogram->win, x, y+temp, histogram->filler);
             }
             else
             {
                mvwaddch (histogram->win, x, y+temp, bchar | fattr);
             }
          }
       }
       wrefresh (histogram->win);

       /* Draw in the label						*/
       writeChtype (histogram->labelwin, 0, 0, histogram->label, HORIZONTAL, 0, histogram->labellen);
       wrefresh (histogram->labelwin);
   }
}

/*
 * This function destroys the histogram.
 */
void destroyCDKHistogram (CDKHISTOGRAM *histogram)
{
   /* Erase the object.							*/
   eraseCDKHistogram (histogram);

   /* Clean up the char pointers.					*/
   freeChtype (histogram->label);
   freeChar (histogram->lowchar);
   freeChar (histogram->highchar);
   freeChar (histogram->statschar);

   /* Clean up the windows.						*/
   delwin (histogram->win);
   delwin (histogram->labelwin);
   if (histogram->shadow)
   {
      delwin (histogram->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vHISTOGRAM, histogram);

   /* Finish cleaning up.						*/
   free (histogram);
}

/*
 * This function erases the histogram from the screen.
 */
void eraseCDKHistogram (CDKHISTOGRAM *histogram)
{
   werase (histogram->win);
   werase (histogram->labelwin);
   wrefresh (histogram->win);
   wrefresh (histogram->labelwin);

   /* Do we need to erase the shadow...					*/
   if (histogram->shadow)
   {
      werase (histogram->shadowwin);
      wrefresh (histogram->shadowwin);
   }
}
