#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.5 $
 */

/*
 * This creates a pointer to an itemlist widget.
 */
CDKITEMLIST *newCDKItemlist (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, char *label, char **item, int count, int defaultItem, boolean box, boolean shadow)
{
   /* Set up some variables.						*/
   CDKITEMLIST *itemlist = (CDKITEMLIST *)malloc (sizeof (CDKITEMLIST));
   char *plainlabel	= (char *)NULL;
   int boxwidth		= 0;
   int boxheight	= 3;
   int fieldWidth	= 0;
   int xpos		= xplace;
   int ypos		= yplace;
   int labelxoffset	= 0;
   int labelyoffset	= 0;
   int fieldxoffset	= 0;
   int fieldyoffset	= 0;
   int temp		= 0;
   int x;

   /* Translate the label char *pointer to a chtype pointer.		*/
   itemlist->label	= char2Chtype (label, &itemlist->labellen, &temp);
   plainlabel		= chtype2Char (itemlist->label);

   /* Go through the list and determine the widest item.		*/
   for (x=0; x < count; x++)
   {
      /* Copy the item to the list.					*/
      itemlist->item[x]	= char2Chtype (item[x], &itemlist->itemlen[x], &itemlist->itempos[x]);
      fieldWidth = (fieldWidth < itemlist->itemlen[x] ? itemlist->itemlen[x] : fieldWidth);
   }
   fieldWidth += 2;

   /* Now we need to justify the strings.				*/
   for (x=0; x < count; x++)
   {
      itemlist->itempos[x] = justifyString (fieldWidth, itemlist->itemlen[x], itemlist->itempos[x]);
   }

   /* Adjust the offset values given the location of the label.		*/
   alignlabel (plainlabel, lplace, fieldWidth, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainlabel);
   
   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Make the label window.						*/
   itemlist->win = newwin (boxheight, boxwidth, ypos, xpos);

   /* Is the window NULL ???						*/
   if (itemlist->win == (WINDOW *)NULL)
   {
      /* Clean up the pointers.						*/
      freeChtype (itemlist->label);
      free (itemlist);
      
      /* Exit with NULL.						*/
      return ( (CDKITEMLIST *)NULL );
   }

   /* Make the field window.						*/
   itemlist->fieldwin	= subwin (cdkscreen->window, 1, fieldWidth, ypos+fieldyoffset, xpos+fieldxoffset);
   itemlist->labelwin	= subwin (cdkscreen->window, 1, itemlist->labellen+1, ypos+labelyoffset, xpos+labelxoffset);
   keypad (itemlist->fieldwin, TRUE);

   /* Set up the rest of the structure.					*/
   itemlist->parent		= cdkscreen->window;
   itemlist->shadowwin		= (WINDOW *)NULL;
   itemlist->fieldWidth		= fieldWidth;
   itemlist->itemCount		= count-1;
   itemlist->box		= box;
   itemlist->shadow		= shadow;

   /* Set then default item.						*/
   if (defaultItem >= 0 && defaultItem <= itemlist->itemCount)
   {
      itemlist->currentItem	= defaultItem;
      itemlist->defaultItem	= defaultItem;
   }
   else
   {
      itemlist->currentItem	= 0;
      itemlist->defaultItem	= 0;
   }

   /* Do we want a shadow???						*/
   if (shadow)
   {
      itemlist->shadowwin = newwin (boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vITEMLIST, itemlist);

   /* Register this baby.               				*/
   registerCDKObject (cdkscreen, vITEMLIST, itemlist);

   /* Return the pointer to the structure				*/
   return (itemlist);
}

/*
 * This allows the user to play with the widget.
 */
int activateCDKItemlist (CDKITEMLIST *itemlist)
{
   /* Declare some local variables.					*/
   chtype input;

   /* Draw the widget.							*/
   drawCDKItemlist (itemlist, itemlist->box);

   /* Let 'em play....							*/
   for (;;)
   {
      /* Get the input from the user.					*/
      input = wgetch (itemlist->fieldwin);

      /* Check a predefined binding.					*/
      if ( ! checkCDKObjectBind (vITEMLIST, itemlist, input) )
      {
         switch (input)
         {
            case KEY_UP : case KEY_RIGHT : case ' ' : case '+' : case 'n' :
                 if (itemlist->currentItem < itemlist->itemCount )
                 {
                    itemlist->currentItem++;
                 }
                 else
                 {
                    itemlist->currentItem = 0;
                 }
                 break;

            case KEY_DOWN : case KEY_LEFT : case '-' : case 'p' :
                 if (itemlist->currentItem > 0 )
                 {
                    itemlist->currentItem--;
                 }
                 else
                 {
                    itemlist->currentItem = itemlist->itemCount;
                 }
                 break;

            case 'd' : case 'D' :
                 itemlist->currentItem = itemlist->defaultItem;
                 break;

            case '0' :
                 itemlist->currentItem = 0;
                 break;

            case '$' :
                 itemlist->currentItem = itemlist->itemCount;
                 break;

            case KEY_RETURN : case KEY_TAB : case CDK_NEXT :
                 return ( itemlist->currentItem );
                 break;

            case CDK_REFRESH :
                 refreshCDKScreen (itemlist->screen);
                 break;

            default :
                 Beep();
                 break;
         }
      }

      /* Redraw the field.					*/
      drawCDKItemlistField (itemlist);
   }
}

/*
 * This draws the widget on the screen.
 */
void drawCDKItemlist (CDKITEMLIST *itemlist, int Box)
{
   /* Erase the widget from the screen.					*/
   eraseCDKItemlist (itemlist);
 
   /* Did we ask for a shadow?						*/
   if (itemlist->shadow)
   {
      drawShadow (itemlist->shadowwin);
   }
 
   /* Box the widget if asked.						*/
   if (Box)
   {
      box (itemlist->win, ACS_VLINE, ACS_HLINE);
      wrefresh (itemlist->win);
   }
 
   /* Draw in the label to the widget.                                  */
   writeChtype (itemlist->labelwin, 0, 0, itemlist->label, HORIZONTAL, 0, itemlist->labellen);
   wrefresh (itemlist->labelwin);

   /* Draw in the field.						*/
   drawCDKItemlistField (itemlist);
}

/*
 * This function draws the contents of the field.
 */
void drawCDKItemlistField (CDKITEMLIST *itemlist)
{
   /* Declare local vars.						*/
   int currentItem = itemlist->currentItem;
   int x;

   /* Draw in the current item in the field.				*/
   werase (itemlist->fieldwin);
   for (x=0; x < itemlist->itemlen[currentItem]; x++)
   {
      mvwaddch (itemlist->fieldwin, 0, x+itemlist->itempos[currentItem], itemlist->item[currentItem][x]);
   }
   wrefresh (itemlist->fieldwin);
}

/*
 * This function removes the widget from the screen.
 */
void eraseCDKItemlist (CDKITEMLIST *itemlist)
{
   werase (itemlist->fieldwin);
   werase (itemlist->labelwin);
   werase (itemlist->win);
   wrefresh (itemlist->win);

   /* Do we want to erase the shadow.				*/
   if (itemlist->shadow)
   {
      werase (itemlist->shadowwin);
      wrefresh (itemlist->shadowwin);
   }
}

/*
 * This function destroys the widget and all the memory it used.
 */
void destroyCDKItemlist (CDKITEMLIST *itemlist)
{
   /* Declare local variables.					*/
   int x;

   /* Erase the object.						*/
   eraseCDKItemlist (itemlist);

   /* Clear out the character pointers.				*/
   freeChtype (itemlist->label);
   for (x=0; x <= itemlist->itemCount; x++)
   {
      freeChtype (itemlist->item[x]);
   }

   /* Delete the windows.					*/
   delwin (itemlist->fieldwin);
   delwin (itemlist->labelwin);
   delwin (itemlist->win);
   if (itemlist->shadow)
   {
      delwin (itemlist->shadowwin);
   }

   /* Unregister this object.					*/
   unregisterCDKObject (vITEMLIST, itemlist);

   /* Finish cleaning up.					*/
   free (itemlist);
}

/*
 * This function sets the contents of the list.
 */
void setCDKItemlist (CDKITEMLIST *itemlist, char **item, int count, int defaultItem, boolean box)
{
   /* Declare local variables.					*/
   int x;

   /* Free up the old memory.					*/
   for (x=0; x <= itemlist->itemCount; x++)
   {
      freeChtype (itemlist->item[x]);
   }

   /* Copy in the new information.				*/
   itemlist->itemCount = count-1;
   for (x=0; x <= itemlist->itemCount; x++)
   {
      /* Copy the new stuff in.					*/
      itemlist->item[x]	= char2Chtype (item[x], &itemlist->itemlen[x], &itemlist->itempos[x]);
      itemlist->itempos[x] = justifyString (itemlist->fieldWidth, itemlist->itemlen[x], itemlist->itempos[x]);
   }

   /* Set the box flag.						*/
   itemlist->box = box;

   /* Set the default item.					*/
   if (defaultItem >= 0 && defaultItem <= itemlist->itemCount)
   {
      itemlist->currentItem = defaultItem;
      itemlist->defaultItem = defaultItem;
   }

   /* Draw the field.						*/
   drawCDKItemlistField (itemlist);
}
