#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.28 $
 */

/*
 * This creates a label widget.
 */
CDKLABEL *newCDKLabel(CDKSCREEN *cdkscreen, int xplace, int yplace, char **mesg, int rows, boolean box, boolean shadow)
{
   /* Maintain the label information					*/
   CDKLABEL *label	= (CDKLABEL *)malloc (sizeof (CDKLABEL));
   int boxWidth		= 0;
   int boxHeight	= rows + 2;
   int xpos		= xplace;
   int ypos		= yplace;
   int x		= 0;

   /* Determine the box width.						*/
   for (x=0; x < rows; x++)
   {
      /* Translate the char * to a chtype.				*/
      label->info[x]	= char2Chtype (mesg[x], &label->infolen[x], &label->infopos[x]);
      boxWidth		= MAX(boxWidth, label->infolen[x]);
   }
   boxWidth += 2;

   /* Create the string alignments.					*/
   for (x=0; x < rows; x++)
   {
      label->infopos[x]	= justifyString (boxWidth, label->infolen[x], label->infopos[x]);
   }

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Create the label							*/
   label->parent	= cdkscreen->window;
   label->win		= newwin (boxHeight, boxWidth, ypos, xpos);
   label->shadowwin	= (WINDOW *)NULL;
   label->xpos		= xpos;
   label->ypos		= ypos;
   label->rows		= rows;
   label->width		= boxWidth;
   label->box		= box;
   label->shadow	= shadow;

   /* Is the window NULL???						*/
   if (label->win == (WINDOW *)NULL) 
   {
      /* Free up any memory used.					*/
      for (x=0; x < rows; x++)
      {
         freeChtype (label->info[x]);
      }
      free(label);

      /* Return a NULL pointer.						*/
      return ( (CDKLABEL *)NULL );
   }

   /* If a shadow was requested, then create the shadow window.		*/
   if (shadow)
   {
      label->shadowwin	= newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vLABEL, label);

   /* Return the label pointer.						*/
   return (label);
}

/*
 * This draws the label widget.
 */
void drawCDKLabel (CDKLABEL *label, boolean Box)
{
   /* Declare local variables.						*/
   int x	= 0;
   
   /* Keep the box information.						*/
   label->box	= Box;

   /* Erase the label widget from the screen.				*/
   eraseCDKLabel (label);

   /* Box the widget if asked.						*/
   if (label->box)
   {
      box (label->win, ACS_VLINE, ACS_HLINE);
   }

   /* Is there a shadow???						*/
   if (label->shadow)
   {
      drawShadow (label->shadowwin);
   }

   /* Draw in the message.						*/
   for (x=0; x < label->rows; x++)
   {
     writeChtype (label->win, label->infopos[x], x+1, label->info[x], HORIZONTAL, 0, label->infolen[x]);
   }
   wrefresh (label->win);
}

/*
 * This erases the label widget.
 */
void eraseCDKLabel (CDKLABEL *label)
{
   werase (label->win);
   wrefresh (label->win);

   /* Do we need to erase the shadow win.				*/
   if (label->shadow)
   {
      werase (label->shadowwin);
      wrefresh (label->shadowwin);
   }
}

/*
 * This destroys the label object pointer.
 */
void destroyCDKLabel (CDKLABEL *label)
{
   /* Declare local variables.						*/
   int x		= 0;

   /* Erase the old label.						*/
   eraseCDKLabel (label);

   /* Free up the character pointers.					*/
   for (x=0; x < label->rows ; x++)
   {
      freeChtype (label->info[x]);
   }

   /* Free up the window pointers.					*/
   delwin (label->win);

   /* Is there a shadow with this widget???				*/
   if (label->shadow)
   {
      delwin (label->shadowwin);
   }
 
   /* Unregister the object.						*/
   unregisterCDKObject (vLABEL, label);

   /* Free the object pointer.						*/
   free (label);
}

/*
 * This pauses until a user hits a key...
 */
char waitCDKLabel (CDKLABEL *label, char key)
{
   /* If the key is null, we'll accept anything.			*/
   if ( key == (char)NULL )
   {
      return (wgetch (label->win));
   }
   else
   {
      /* Only exit when a specific key is hit.				*/
      char newkey;
      for (;;)
      {
         newkey = wgetch(label->win);
         if (newkey == key)
         {
            return ( newkey );
         }
      }
   }
}
