#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.24 $
 */

/*
 * This creates a marquee widget.
 */
CDKMARQUEE *newCDKMarquee (CDKSCREEN *cdkscreen, int xplace, int yplace, int width, boolean shadow)
{
   CDKMARQUEE *marquee	= (CDKMARQUEE *)malloc (sizeof (CDKMARQUEE));
   int xpos		= xplace;
   int ypos		= yplace;
   int boxheight	= 3;
   int boxwidth		= width + 2;

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Create the marquee pointer.					*/
   marquee->parent	= cdkscreen->window;
   marquee->win		= newwin (boxheight, boxwidth, ypos, xpos);
   marquee->shadowwin	= (WINDOW *)NULL;
   marquee->active	= TRUE;
   marquee->width	= width;
   marquee->box		= NOBOX;
   marquee->shadow	= shadow;

   /* Is the window NULL???						*/
   if (marquee->win == (WINDOW *)NULL)
   {
      /* Clean up any memory.						*/
      free (marquee);

      /* Return a NULL pointer.						*/
      return ( (CDKMARQUEE *)NULL );
   }

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vMARQUEE, marquee);

   /* Return the marquee pointer.					*/
   return(marquee);
}

/* 
 * This activates the marquee.
 */
int activateCDKMarquee (CDKMARQUEE *marquee, char *mesg, int delay, int repeat, boolean box)
{
   /* Declear local variables.						*/
   chtype *message;
   int mesglength 	= 0;
   int startpos		= 0;
   int firstchar	= 0;
   int lastchar		= 1;
   int repeatCount	= 0;
   int viewsize		= 0;
#ifdef NOUSLEEP
   int sleeper		= 0;
#endif
   int x, y, junk;

   /* Keep the box info.						*/
   marquee->box = box;

   /* Make sure the message has some content.				*/
   if (mesg == (char *)NULL)
   {
      return (-1);
   }
   
   /* Translate the char * to a chtype *				*/
   message	= char2Chtype (mesg, &mesglength, &junk);

   /* Draw in the marquee.						*/
   drawCDKMarquee (marquee, box);

   /* Set up the variables.						*/
   viewsize = lastchar - firstchar;
   startpos = marquee->width - viewsize + 1;

   /* Start doing the marquee thing...					*/
   for (;;)
   {
      if (marquee->active)
      {
         /* Draw in the characters.					*/
         y = firstchar;
         for (x=startpos ; x < (startpos+viewsize) ; x++)
         {
            mvwaddch (marquee->win, 1, x, message[y]);
            y++;
         }
         wrefresh (marquee->win);
   
         /* Set my variables.						*/
         if (mesglength < marquee->width-2)
         {
            if (lastchar < mesglength)
            {
               lastchar ++;
               viewsize ++;
               startpos = marquee->width - viewsize + 1;
            }
            else if (lastchar == mesglength)
            {
               if (startpos > 1)
               {
                  /* This means the whole string is visible.		*/
                  startpos --;
                  viewsize = mesglength;
               }
               else
               {
                 /* We have to start chopping the viewsize		*/
                 startpos = 1;
                 firstchar ++;
                 viewsize --;
               }
            }
         }
         else
         {
            if (startpos > 1)
            {
               lastchar ++;
               viewsize ++;
               startpos --;
            }
            else
            {
               if (lastchar < mesglength)
               {
                  firstchar ++;
                  lastchar  ++;
                  viewsize = marquee->width;
                  startpos = 1;
               }
               else
               {
                  firstchar ++;
                  viewsize --;
                  startpos = 1;
               }
            }
         }

         /* OK, lets check if we have to start over.			*/
         if ( viewsize == 0 && firstchar == mesglength)
         {
            /* Check if we need to repeat or not.			*/
            repeatCount ++;
            if (repeat > 0 && repeatCount == repeat)
            {
               freeChtype (message);
               return (0);
            }

            /* Time to start over. 					*/
            mvwaddch (marquee->win, 1, 1, ' '|A_NORMAL);
            wrefresh (marquee->win);
#ifndef NOUSLEEP
            curs_set(0);
#endif
            firstchar = 0;
            lastchar = 1;
            viewsize = lastchar - firstchar;
            startpos = marquee->width - viewsize + 1;
         }
   
         /* Now sleep							*/
#ifdef NOUSLEEP
         for (sleeper = 0; sleeper <= (delay * 250) ; sleeper++) {}
#else
         usleep ( (delay * 10000) );
#endif
      }
      else
      {
         printf ("\r\n\r\n\r\n\r\nMarquee turned off\r\n");
         }
   }
}

/*
 * This draws the marquee widget on the screen.
 */
void drawCDKMarquee (CDKMARQUEE *marquee, boolean Box)
{
   /* Keep the box information.						*/
   marquee->box	= Box;

   /* Erase the old object...						*/
   eraseCDKMarquee (marquee);

   /* Do we need to draw a shadow???					*/
   if (marquee->shadow)
   {
      drawShadow (marquee->shadowwin);
   }

   /* Box it if needed.							*/
   if (Box)
   {
      box (marquee->win, ACS_VLINE, ACS_HLINE);
      wrefresh (marquee->win);
   }
}

/*
 * This destroys the marquee.
 */
void destroyCDKMarquee (CDKMARQUEE *marquee)
{
   /* Erase the object.							*/
   eraseCDKMarquee (marquee);

   /* Clean up the windows.						*/
   delwin (marquee->win);
   if (marquee->shadow)
   {
      delwin (marquee->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vMARQUEE, marquee);
   
   /* Finish cleaning up.						*/
   free (marquee);
}

/*
 * This erases the marquee.
 */
void eraseCDKMarquee (CDKMARQUEE *marquee)
{
   werase (marquee->win);
   wrefresh (marquee->win);
   if (marquee->shadow)
   {
      werase (marquee->shadowwin);
      wrefresh (marquee->shadowwin);
   }
}
