#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/23 22:28:56 $
 * $Revision: 1.73 $
 */

/* Decalre global vars.							*/
static char *PasteBuffer	= (char *)NULL;
void redrawTitles (CDKMATRIX *matrix, int row, int col);

/*
 * This function creates the matrix widget.
 */
CDKMATRIX *newCDKMatrix (CDKSCREEN *cdkscreen, int xplace, int yplace, int rows, int cols, int vrows, int vcols, char **rowtitles, char **coltitles, int *colwidths, int *colvalues, int rspace, int cspace, chtype filler, int dominant, MATRIXCB CDKMatrixCallBackFn, boolean boxMatrix, boolean boxCell, boolean shadow)
{
   /* Declare local variables.						*/
   CDKMATRIX *matrix	= (CDKMATRIX *)malloc (sizeof(CDKMATRIX));
   int boxheight	= 0;
   int boxwidth		= 0;
   int xpos		= xplace;
   int ypos		= yplace;
   int maxRowTitleWidth	= 0;
   int begx, begy, x, y, z, w, cellwidth;
   int rowspace		= MAX (0, rspace);
   int colspace		= MAX (0, cspace);

   /* Make sure that the number of rows/cols/vrows/vcols is not zero.	*/
   if (rows == 0 || cols == 0 || vrows == 0 || vcols == 0)
   {
      /* Free up any used memory.					*/
      free (matrix);

      /* Return a NULL pointer.						*/
      return ( (CDKMATRIX *)NULL );
   }

   /* Make sure the number of virtual cells is not larger than		*/
   /* the physical size.						*/
   vrows = (vrows > rows ? rows : vrows);
   vcols = (vcols > cols ? cols : vcols);

   /* Determine the height of the box.					*/
   for (x=1; x <= vrows; x++)
   {
      boxheight += 3 + rowspace;
   }

   /* Determine the maximum row title width				*/
   maxRowTitleWidth	= 0;
   for (x=1; x <= rows; x++)
   {
      matrix->rowtitle[x] = char2Chtype (rowtitles[x], &matrix->rowtitlelen[x], &matrix->rowtitlepos[x]);
      maxRowTitleWidth = MAX (maxRowTitleWidth, matrix->rowtitlelen[x]);
   }
   matrix->maxrt = maxRowTitleWidth + 2;
   boxwidth = matrix->maxrt;

   /* We need to rejustify the row title cell info.			*/
   for (x=1; x <= rows; x++)
   {
      matrix->rowtitlepos[x]	= justifyString (matrix->maxrt, matrix->rowtitlelen[x], matrix->rowtitlepos[x]);
   }

   /* Determine the width of the matrix.				*/
   for (x=1; x <= vcols; x++)
   {
      boxwidth += colwidths[x] + 2 + colspace;
   }
   boxwidth--;
   boxheight++;

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Make the pop-up window.						*/
   matrix->win = newwin (boxheight, boxwidth, ypos, xpos);

   /* Check if the new window is NULL or not.				*/
   if (matrix->win == (WINDOW *)NULL)
   {
      /* Free up any used memory.					*/
      for (z=1; z <= rows; z++)
      {
         freeChtype (matrix->rowtitle[z]);
      }
      free (matrix);

      /* Return a NULL pointer.						*/
      return ( (CDKMATRIX *)NULL );
   }

   /* Make the subwindows in the pop-up.				*/
   begx = xpos;
   begy = ypos + 1;

   /* Make the 'empty' 0x0 cell.					*/
   matrix->cell[0][0] = subwin (matrix->win, 3, matrix->maxrt, begy, begx);
   begx += matrix->maxrt + 1;

   /* Make the column titles.						*/
   for (x=1; x <= vcols; x++)
   {
      cellwidth = colwidths[x] + 2;
      matrix->cell[0][x] = subwin (matrix->win, 1, cellwidth, begy, begx);

      /* Is the subwindow NULL						*/
      if (matrix->cell[0][x] == (WINDOW *)NULL)
      {
         /* Free up any used memory.					*/
         for (z=1; z <= rows; z++)
         {
            freeChtype (matrix->rowtitle[z]);
         }

         /* We have to delete any windows created so far.		*/
         for (z=1; z < x; z++)
         {
            delwin ( matrix->cell[0][z]);
         }
         free (matrix);

         /* Return a NULL pointer.					*/
         return ( (CDKMATRIX *)NULL );
      }
      begx +=  cellwidth + colspace - 1;
   }
   begy ++;

   /* Make the main cell body						*/
   for (x=1; x <= vrows; x++)
   {
      /* Make the row titles						*/
      matrix->cell[x][0] = subwin (matrix->win, 3, matrix->maxrt, begy, xpos+1);

      /* Is the window NULL						*/
      if (matrix->cell[x][0] == (WINDOW *)NULL)
      {
         /* Free up any used memory.					*/
         for (z=1; z <= rows; z++)
         {
            freeChtype (matrix->rowtitle[z]);
         }

         /* We have to delete any windows created so far.		*/
         for (z=1; z <= vcols; z++)
         {
            delwin ( matrix->cell[0][z]);
         }
         for (z=1; z < x ; z++)
         {
            delwin ( matrix->cell[z][0]);
         }
         free (matrix);

         /* Return a NULL pointer.					*/
         return ( (CDKMATRIX *)NULL );
      }

      /* Set the start of the x position.				*/
      begx = xpos + matrix->maxrt + 1;

      /* Make the cells							*/
      for (y=1; y <= vcols; y++)
      {
         cellwidth = colwidths[y] + 2;
         matrix->cell[x][y] = subwin (matrix->win, 3, cellwidth, begy, begx);

         /* Is the cell NULL???						*/
         if (matrix->cell[x][y] == (WINDOW *)NULL)
         {
            /* Free up any used memory.					*/
            for (y=1; y <= rows; y++)
            {
               freeChtype (matrix->rowtitle[y]);
            }

            /* We have to delete any windows created so far.		*/
            for (y=1; y <= vcols; y++)
            {
               delwin ( matrix->cell[0][y]);
            }
            for (y=1; y < vrows ; y++)
            {
               delwin ( matrix->cell[y][0]);
            }
            for (w=1; w <= vrows; w++)
            {
               for (z=1; z <= y; z++)
               {
                  delwin ( matrix->cell[w][z]);
               }
            }
            free (matrix);

            /* Return a NULL pointer.					*/
            return ( (CDKMATRIX *)NULL );
         }
         begx += cellwidth + colspace - 1;
         keypad (matrix->cell[x][y], TRUE);
      }
      begy += rowspace + 2;
   }

   /* Copy the titles into the sturcture.				*/
   for (x=1; x <= cols; x++)
   {
      matrix->coltitle[x]	= char2Chtype (coltitles[x], &matrix->coltitlelen[x], &matrix->coltitlepos[x]);
      matrix->coltitlepos[x]	= justifyString (colwidths[x], matrix->coltitlelen[x], matrix->coltitlepos[x]);
      matrix->colwidths[x]	= colwidths[x];
   }

   /* Make room for the cell information.				*/
   for (x=1; x <= rows; x++)
   {
      for (y=1; y <= cols; y++)
      {
#ifdef NODYNALLOC
         matrix->info[x][y]	= (char *)malloc (sizeof (char) * 256);
#else
         matrix->info[x][y]	= (char *)malloc (sizeof (char) * (colwidths[y]+1));
#endif
         matrix->colvalues[y]	= colvalues[y];
         matrix->colwidths[y]	= colwidths[y];
         cleanChar (matrix->info[x][y], colwidths[y]+1, '\0');
      }
   }

   /* Keep the rest of the info.					*/
   matrix->parent	= cdkscreen->window;
   matrix->rows		= rows;
   matrix->cols		= cols;
   matrix->vrows	= vrows;
   matrix->vcols	= vcols;
   matrix->rowspace	= rowspace;
   matrix->colspace	= colspace;
   matrix->filler	= filler;
   matrix->dominant	= dominant;
   matrix->row		= 1;
   matrix->col		= 1;
   matrix->crow		= 1;
   matrix->ccol		= 1;
   matrix->trow		= 1;
   matrix->lcol		= 1;
   matrix->oldcrow	= 1;
   matrix->oldccol	= 1;
   matrix->oldvrow	= 1;
   matrix->oldvcol	= 1;
   matrix->boxMatrix	= boxMatrix;
   matrix->boxCell	= boxCell;
   matrix->shadow	= shadow;
   matrix->highlight	= A_REVERSE;
   matrix->callbackfn	= (void *)CDKMatrixCallBackFn;

   /* Do we want a shadow???						*/
   if (shadow)
   {
      matrix->shadowwin = newwin (boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vMATRIX, matrix);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vMATRIX, matrix);

   /* Return the matrix pointer						*/
   return (matrix);
}

/* 
 * This activates the matrix.
 */
void activateCDKMatrix (CDKMATRIX *matrix)
{
   /* Declare local variables.						*/
   int refreshCells	= FALSE;
   int movedCell	= FALSE;
   chtype input	= (chtype)NULL;
   int charcount, x, y, infolen;

   /* Draw the matrix							*/
   drawCDKMatrix (matrix, matrix->boxMatrix);
   
   /* Put the focus on the first cell					*/
   if (matrix->boxCell)
   {
      attrbox (	matrix->cell[matrix->crow][matrix->ccol],
		ACS_ULCORNER,	ACS_URCORNER,
		ACS_LLCORNER,	ACS_LRCORNER,
		ACS_HLINE,	ACS_VLINE,
		A_BOLD);
   }
   highlightCDKMatrixCell ( matrix );
   if (  matrix->colwidths[matrix->ccol] == 1)
   {
      wmove (matrix->cell[matrix->crow][matrix->ccol], 1, 1);
   }
   else
   {
      wmove (matrix->cell[matrix->crow][matrix->ccol], 1, strlen (matrix->info[matrix->row][matrix->col])+1);
   }
   wrefresh (matrix->cell[matrix->crow][matrix->ccol]);

   /* Start taking the input.						*/
   for (;;)
   {
      refreshCells	= FALSE;
      movedCell		= FALSE;
      input		= wgetch (matrix->cell[matrix->crow][matrix->ccol]);
      charcount		= strlen (matrix->info[matrix->row][matrix->col]);

      if ( ! checkCDKObjectBind (vMATRIX, matrix, input) )
      {
         switch (input)
         {
            case KEY_BACKSPACE : case DELETE : case '' :
                 if (matrix->colvalues[matrix->col] == vVIEWONLY)
                 {
                   Beep();
                 }
                 else
                 {
                    if (charcount > 0)
                    {
                       charcount--;
                       mvwdelch (matrix->cell[matrix->crow][matrix->ccol], 1, charcount+1);
                       mvwinsch (matrix->cell[matrix->crow][matrix->ccol], 1, charcount+1, matrix->filler);
                       wrefresh (matrix->cell[matrix->crow][matrix->ccol]);
                       matrix->info[matrix->row][matrix->col][charcount] = '\0';
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 break;
   
            case KEY_RIGHT : case KEY_TAB : case KEY_RETURN :
                 if (matrix->ccol != matrix->vcols)
                 {
                    /* We are moving to the right...			*/
                    matrix->col++;
                    matrix->ccol++;
                    movedCell	= TRUE;
                 }
                 else
                 {
                    /* We have to shift the columns to the right.	*/
                    if (matrix->col != matrix->cols)
                    {
                       matrix->lcol++;
                       matrix->col++;
   
                       /* Redraw the column titles.			*/
                       if (matrix->rows > matrix->vrows)
                       {
                          redrawTitles (matrix, FALSE, TRUE);
                       }
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       /* We are at the far right column, we need 	*/
                       /* shift down one row, if we can.		*/
                       if (matrix->row == matrix->rows)
                       {
                          Beep();
                       }
                       else
                       {
                          /* Set up the columns info.			*/
                          matrix->col	= 1;
                          matrix->lcol	= 1;
                          matrix->ccol	= 1;

                          /* Shift the rows...				*/
                          if (matrix->crow != matrix->vrows)
                          {
                             matrix->row++;
                             matrix->crow++;
                          }
                          else
                          {
                             matrix->row++;
                             matrix->trow++;
                          }
                          redrawTitles (matrix, TRUE, TRUE);
                          refreshCells	= TRUE;
                          movedCell	= TRUE;
                       }
                    }
                 }
                 break;
   
            case KEY_LEFT :
                 if (matrix->ccol != 1)
                 {
                    /* We are moving to the left...			*/
                    matrix->col--;
                    matrix->ccol--;
                    movedCell	= TRUE;
                 }
                 else
                 {
                    /* Are we at the far left???			*/
                    if (matrix->lcol != 1)
                    {
                       matrix->lcol--;
                       matrix->col--;
   
                       /* Redraw the column titles.			*/
                       if (matrix->cols > matrix->vcols)
                       {
                          redrawTitles (matrix, FALSE, TRUE);
                       }
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       /* Shift up one line if we can...		*/
                       if (matrix->row == 1)
                       {
                          Beep();
                       }
                       else
                       {
                          /* Set up the columns info.			*/
                          matrix->col	= matrix->cols;
                          matrix->lcol	= matrix->cols - matrix->vcols + 1;
                          matrix->ccol	= matrix->vcols;

                          /* Shift the rows...				*/
                          if (matrix->crow != 1)
                          {
                             matrix->row--;
                             matrix->crow--;
                          }
                          else
                          {
                             matrix->row--;
                             matrix->trow--;
                          }
                          redrawTitles (matrix, TRUE, TRUE);
                          refreshCells	= TRUE;
                          movedCell	= TRUE;
                       }
                    }
                 }
                 break;
      
            case KEY_UP :
                 if (matrix->crow != 1)
                 {
                    matrix->row--;
                    matrix->crow--;
                    movedCell	= TRUE;
                 }
                 else
                 {
                    if (matrix->trow != 1)
                    {
                       matrix->trow--;
                       matrix->row--;
                       
                       /* Redraw the row titles.			*/
                       if (matrix->rows > matrix->vrows)
                       {
                          redrawTitles (matrix, TRUE, FALSE);
                       }
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 break;
   
            case KEY_DOWN :
                 if (matrix->crow != matrix->vrows)
                 {
                    matrix->row ++;
                    matrix->crow ++;
                    movedCell	= TRUE;
                 }
                 else
                 {
                    if ((matrix->trow + matrix->vrows - 1) != matrix->rows )
                    {
                       matrix->trow++;
                       matrix->row++;
                       
                       /* Redraw the titles.				*/
                       if (matrix->rows > matrix->vrows)
                       {
                          redrawTitles (matrix, TRUE, FALSE);
                       }
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 break;
   
            case KEY_NPAGE : case '' :
                 if (matrix->rows > matrix->vrows)
                 {
                    if ((matrix->trow + ((matrix->vrows - 1) * 2)) <= matrix->rows )
                    {
                       matrix->trow	+= matrix->vrows - 1;
                       matrix->row	+= matrix->vrows - 1;
                       redrawTitles (matrix, TRUE, FALSE);
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_PPAGE : case '' :
                 if (matrix->rows > matrix->vrows)
                 {
                    if ((matrix->trow - ((matrix->vrows - 1) * 2)) >= 1)
                    {
                       matrix->trow	-= matrix->vrows - 1;
                       matrix->row	-= matrix->vrows - 1;
                       redrawTitles (matrix, TRUE, FALSE);
                       refreshCells	= TRUE;
                       movedCell	= TRUE;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
	 case '' :
              jumpToCell (matrix);
              drawCDKMatrix (matrix, matrix->boxMatrix);
              break;

   	 case CDK_PASTE :
              if (PasteBuffer == (char *)NULL || (int)strlen (PasteBuffer) > matrix->colwidths[matrix->ccol])
              {
                 char *mesg[] = {"Nothing in paste buffer."};
                 Beep();
                 popUpMessage (matrix->screen, mesg, 1);
              }
              else
              {
                 strcpy (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1], PasteBuffer);
                 drawCDKMatrixCell (matrix, matrix->crow, matrix->ccol, matrix->row, matrix->col, A_NORMAL, matrix->boxCell);
              }
              break;
   
   	 case CDK_COPY :
              {
                 char *mesg[] = {"Cell copied into paste buffer."};
                 freeChar (PasteBuffer);
                 PasteBuffer = copyChar (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1]);
                 popUpMessage (matrix->screen, mesg, 1);
              }
              break;
   
   	 case CDK_CUT :
              {
                 char *mesg[] = {"Cell copied into paste buffer."};
                 freeChar (PasteBuffer);
                 PasteBuffer = copyChar (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1]);
                 popUpMessage (matrix->screen, mesg, 1);
                 cleanChar (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1], matrix->colwidths[matrix->lcol+matrix->ccol-1], '\0');
                 drawCDKMatrixCell (matrix, matrix->crow, matrix->ccol, matrix->row, matrix->col, A_NORMAL, matrix->boxCell);
              }
              break;
   
   	 case CDK_ERASE :
              cleanChar (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1], matrix->colwidths[matrix->lcol+matrix->ccol-1], '\0');
              drawCDKMatrixCell (matrix, matrix->crow, matrix->ccol, matrix->row, matrix->col, A_NORMAL, matrix->boxCell);
              break;
   
   	 case CDK_NEXT :
              if (!matrix->boxCell)
              {
                 attrbox (	matrix->cell[matrix->oldcrow][matrix->oldccol],
              			' ',	' ',
              			' ',	' ',
              			' ',	' ',
              			A_NORMAL);
              }
              else
              {
                 drawCDKMatrixCell (matrix,
              			matrix->oldcrow,
              			matrix->oldccol,
              			matrix->oldvrow,
              			matrix->oldvcol,
              			A_NORMAL,
              			matrix->boxCell);
              }
              wrefresh (matrix->cell[matrix->crow][matrix->ccol]);
              return;
   	      break;
   
         case CDK_REFRESH :
              refreshCDKScreen (matrix->screen);
              break;
   
         default :
              ((MATRIXCB)matrix->callbackfn)( matrix, input );
              break;
         }
      }

      /* Did we change cells?						*/
      if (movedCell)
      {
         /* un-highlight the old box 					*/
         if (!matrix->boxCell)
         {
            attrbox (	matrix->cell[matrix->oldcrow][matrix->oldccol],
			' ',	' ',
			' ',	' ',
			' ',	' ',
			A_NORMAL);
         }
         else
         {
            drawCDKMatrixCell (matrix,
				matrix->oldcrow,
				matrix->oldccol,
				matrix->oldvrow,
				matrix->oldvcol,
				A_NORMAL,
				matrix->boxCell);
         }
         wrefresh (matrix->cell[matrix->oldcrow][matrix->oldccol]);

         /* Highlight the new cell.					*/
         attrbox (	matrix->cell[matrix->crow][matrix->ccol],
			ACS_ULCORNER,	ACS_URCORNER,
			ACS_LLCORNER,	ACS_LRCORNER,
			ACS_HLINE,	ACS_VLINE,
			A_BOLD);

         highlightCDKMatrixCell ( matrix );
      }

      /* Redraw each cell.						*/
      if (refreshCells)
      {
         /* Fill in the cells.						*/
         for (x=1; x <= matrix->vrows; x++)
         {
            for (y=1; y <= matrix->vcols; y++)
            {
               drawCDKMatrixCell (matrix, x, y,
					matrix->trow+x-1,
					matrix->lcol+y-1,
					A_NORMAL,
					matrix->boxCell);
            }
         }
      }

      /* Move to the correct position in the cell.			*/
      if (refreshCells || movedCell)
      {
         if (  matrix->colwidths[matrix->ccol] == 1)
         {
            wmove (matrix->cell[matrix->crow][matrix->ccol], 1, 1);
         }
         else
         {
            infolen = strlen (matrix->info[matrix->trow+matrix->crow-1][matrix->lcol+matrix->ccol-1]);
            wmove (matrix->cell[matrix->crow][matrix->ccol], 1, infolen+1);
         }
         wrefresh (matrix->cell[matrix->crow][matrix->ccol]);
      }

      /* Set the variables we need.					*/
      matrix->oldcrow	= matrix->crow;
      matrix->oldccol	= matrix->ccol;
      matrix->oldvrow	= matrix->row;
      matrix->oldvcol	= matrix->col;
   }
}

/* 
 * This allows the programmer to define their own key mappings.	
 */
void CDKMatrixCallBack ( CDKMATRIX *matrix, chtype input )
{
   /* Declare local variables.						*/
   EDisplayType disptype	= matrix->colvalues[matrix->col];
   int charcount		= strlen (matrix->info[matrix->row][matrix->col]);
   chtype newchar;

   /* Check the types							*/
   if (disptype == vINT && !isdigit((char)input))
   {
      Beep();
   }
   else if ( (	disptype == vCHAR || disptype == vUCHAR || \
		disptype == vLCHAR || disptype == vUHCHAR || \
		disptype == vLHCHAR) && isdigit((char)input) )
   {
      Beep();
   }
   else if (disptype == vVIEWONLY)
   {
      Beep();
   }
   else 
   {
      /* Check the width of the string.					*/
      if (charcount == matrix->colwidths[matrix->col])
      {
         Beep();
      }
      else
      {
         /* We will make any needed adjustments to the case of the character.	*/
         newchar = input;
         if ( (	disptype == vUCHAR || disptype == vUCHAR || \
		disptype == vUMIXED || disptype == vUHMIXED) \
		&& !isdigit((char)input) )
         {
            newchar = toupper(input);
         }
         else if ( (	disptype == vUCHAR || disptype == vUCHAR || \
			disptype == vUMIXED || disptype == vUHMIXED) && \
			!isdigit((char)input) )
         {
            newchar = tolower(input);
         }

         /* Update the screen.						*/
         wmove ( matrix->cell[matrix->crow][matrix->ccol], 1, strlen (matrix->info[matrix->row][matrix->col])+1);
         waddch (matrix->cell[matrix->crow][matrix->ccol], newchar);
         wrefresh (matrix->cell[matrix->crow][matrix->ccol]);

         /* Update the character pointer.				*/
         matrix->info[matrix->row][matrix->col][charcount++] = newchar;
         matrix->info[matrix->row][matrix->col][charcount] = '\0';
      }
   }
}

/*
 * Highlight the new field.	
 */
void highlightCDKMatrixCell ( CDKMATRIX *matrix )
{
   /* Declare local variables.						*/
   chtype highlight	= matrix->highlight;
   int x		= 0;
   int infolen		= strlen (matrix->info[matrix->row][matrix->col]);

   /* Given the dominance of the colors/attributes, we need to set the 	*/
   /* current cell attribute.						*/
   if (matrix->dominant == ROW)
   {
      highlight	= matrix->rowtitle[matrix->crow][0] & A_ATTRIBUTES;
   }
   else if (matrix->dominant == COL)
   {
      highlight	= matrix->coltitle[matrix->ccol][0] & A_ATTRIBUTES;
   }

   /* If the column is only one char.					*/
   for (x=1; x <= matrix->colwidths[matrix->ccol]; x++)
   {
      if (x <= infolen )
      {
        mvwaddch (matrix->cell[matrix->crow][matrix->ccol], 1, x, matrix->info[matrix->row][matrix->col][x-1]|highlight);
      }
      else
      {
        mvwaddch (matrix->cell[matrix->crow][matrix->ccol], 1, x, matrix->filler | highlight);
      }
   }
   wmove (matrix->cell[matrix->crow][matrix->ccol], 1, infolen+1);
   wrefresh (matrix->cell[matrix->crow][matrix->ccol]);
}

/*
 * This draws a cell within a matrix.
 */
void drawCDKMatrixCell (CDKMATRIX *matrix, int row, int col, int vrow, int vcol, chtype attr, boolean Box)
{
   /* Declare local variables.						*/
   WINDOW *cell		= matrix->cell[row][col];
   chtype highlight	= matrix->filler & A_ATTRIBUTES;
   int rows		= matrix->vrows;
   int cols		= matrix->vcols;
   int infolen		= strlen (matrix->info[vrow][vcol]);
   int x;

   /* Given the dominance of the colors/attributes, we need to set the 	*/
   /* current cell attribute.						*/
   if (matrix->dominant == ROW)
   {
      highlight	= matrix->rowtitle[row][0] & A_ATTRIBUTES;
   }
   else if (matrix->dominant == COL)
   {
      highlight	= matrix->coltitle[col][0] & A_ATTRIBUTES;
   }

   /* Draw in the cell info.						*/
   for (x=1; x <= matrix->colwidths[col]; x++)
   {
      if (x <= infolen)
      {
         mvwaddch (cell, 1, x, matrix->info[vrow][vcol][x-1]|highlight);
      }
      else
      {
         mvwaddch (cell, 1, x, matrix->filler|highlight);
      }
   }
   wmove (cell, 1, infolen+1);
   wrefresh (cell);

   /* Only draw the box iff the user asked for a box.			*/
   if (!Box)
   {
      return;
   }

   /* If the value of the column spacing is greater than 0 then these	*/
   /* are independent boxes.						*/
   if (matrix->colspace != 0 && matrix->rowspace != 0)
   {
      attrbox (cell,	ACS_ULCORNER,	ACS_URCORNER,
      			ACS_LLCORNER,	ACS_LRCORNER,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      return;
   }
   if (matrix->colspace != 0 && matrix->rowspace == 0)
   {
      if (row == 1)
      {
         attrbox (cell,	ACS_ULCORNER,	ACS_URCORNER,
      			ACS_LTEE,	ACS_RTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
      else if (row > 1 && row < rows)
      {
         attrbox (cell,	ACS_LTEE,	ACS_RTEE,
      			ACS_LTEE,	ACS_RTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
      else if (row == rows)
      {
         attrbox (cell,	ACS_LTEE,	ACS_RTEE,
      			ACS_LLCORNER,	ACS_LRCORNER,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
   }
   if (matrix->colspace == 0 && matrix->rowspace != 0)
   {
      if (col == 1)
      {
         attrbox (cell,	ACS_ULCORNER,	ACS_TTEE,
      			ACS_LLCORNER,	ACS_BTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
      else if (col > 1 && col < cols)
      {
         attrbox (cell,	ACS_TTEE,	ACS_TTEE,
      			ACS_BTEE,	ACS_BTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
      else if (col == cols)
      {
         attrbox (cell,	ACS_TTEE,	ACS_URCORNER,
      			ACS_BTEE,	ACS_LRCORNER,
      			ACS_HLINE,	ACS_VLINE,
			attr);
         return;
      }
   }

   /* Start drawing the matrix.						*/
   if (row == 1)
   {
      if (col == 1)
      {
         /* Draw the top left corner					*/
         attrbox (cell,	ACS_ULCORNER,	ACS_TTEE,
			ACS_LTEE,	ACS_PLUS,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col > 1 && col < cols)
      {
         /* Draw the top middle box					*/
         attrbox (cell,	ACS_TTEE,	ACS_TTEE,
      			ACS_PLUS,	ACS_PLUS,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col == cols)
      {
         /* Draw the top right corner					*/
         attrbox (cell,	ACS_TTEE,	ACS_URCORNER,
      			ACS_PLUS,	ACS_LTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
   }
   else if (row > 1 && row < rows)
   {
      if (col == 1)
      {
         /* Draw the middle left box					*/
         attrbox (cell,	ACS_LTEE,	ACS_PLUS,
     			ACS_LTEE,	ACS_PLUS,
     			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col > 1 && col < cols)
      {
         /* Draw the middle box						*/
         attrbox (cell,	ACS_PLUS,	ACS_PLUS,
      			ACS_PLUS,	ACS_PLUS,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col == cols)
      {
         /* Draw the middle right box					*/
         attrbox (cell,	ACS_PLUS,	ACS_RTEE,
      			ACS_PLUS,	ACS_RTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
   }
   else if (row == rows)
   {
      if (col == 1)
      {
         /* Draw the bottom left corner					*/
         attrbox (cell,	ACS_LTEE,	ACS_PLUS,
      			ACS_LLCORNER,	ACS_BTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col > 1 && col < cols)
      {
         /* Draw the bottom middle box					*/
         attrbox (cell,	ACS_PLUS,	ACS_PLUS,
      			ACS_BTEE,	ACS_BTEE,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
      else if (col == cols)
      {
         /* Draw the bottom right corner				*/
         attrbox (cell,	ACS_PLUS,	ACS_RTEE,
      			ACS_BTEE,	ACS_LRCORNER,
      			ACS_HLINE,	ACS_VLINE,
			attr);
      }
   }
}

/*
 * This function draws the matrix widget.
 */
void drawCDKMatrix (CDKMATRIX *matrix, boolean Box)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   /* Erase the old object...						*/
   eraseCDKMatrix (matrix);

   /* Did we ask for a shadow???					*/
   if (matrix->shadow)
   {
      drawShadow (matrix->shadowwin);
   }

   /* Should we box the matrix???					*/
   if (Box)
   {
      box (matrix->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (matrix->win);

   /* Draw in the column titles.					*/
   for (x=1; x <= matrix->vcols; x++)
   {
      writeChtype (matrix->cell[0][x],
			matrix->coltitlepos[matrix->lcol+x-1], 0,
			matrix->coltitle[matrix->lcol+x-1], 
			HORIZONTAL, 
			0, matrix->coltitlelen[matrix->lcol+x-1]);
      wrefresh (matrix->cell[0][x]);
   }

   /* Fill in the rest of the matrix.					*/
   for (x=1; x <= matrix->vrows; x++)
   {
      /* Draw in the row titles						*/
      writeChtype (matrix->cell[x][0],
			matrix->rowtitlepos[matrix->trow+x-1], 1,
			matrix->rowtitle[matrix->trow+x-1], 
			HORIZONTAL, 
			0, matrix->rowtitlelen[matrix->trow+x-1]);
      wrefresh (matrix->cell[x][0]);

      /* Draw in the cells..						*/
      for (y=1; y <= matrix->vcols; y++)
      {
         drawCDKMatrixCell (matrix, x, y, matrix->trow+x-1, matrix->lcol+y-1, A_NORMAL, matrix->boxCell);
      }
   }

   /* Refresh the matrix.						*/
   wrefresh (matrix->win);
}

/*
 * This function destroys the matrix widget.
 */
void destroyCDKMatrix (CDKMATRIX *matrix)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   /* Erase the object.							*/
   eraseCDKMatrix (matrix);

   /* Clear out the col titles.						*/
   for (x=1; x <= matrix->cols; x++)
   {
      freeChtype (matrix->coltitle[x]);
   }

   /* Clear out the row titles.						*/
   for (x=1; x <= matrix->rows; x++)
   {
      freeChtype (matrix->rowtitle[x]);
   }

   /* Clear out the matrix cells.					*/
   for (x=1; x <= matrix->rows; x++)
   {
      for (y=1; y <= matrix->cols; y++)
      {
         freeChar (matrix->info[x][y]);
      }
   }
 
   /* Clear the matrix windows.						*/
   delwin (matrix->cell[0][0]);
   for (x=1; x <= matrix->vrows; x++)
   {
      delwin (matrix->cell[x][0]);
   }
   for (x=1; x <= matrix->vcols; x++)
   {
      delwin (matrix->cell[0][x]);
   }
   for (x=1; x <= matrix->vrows; x++)
   {
      for (y=1; y <= matrix->vcols; y++)
      {
         delwin (matrix->cell[x][y]);
      }
   }
   delwin (matrix->win);

   /* Is there a shadow to destroy???					*/
   if (matrix->shadow)
   {
      delwin (matrix->shadowwin);
   }
   
   /* Unregister this object.						*/
   unregisterCDKObject (vMATRIX, matrix);

   /* Finish cleaning up.						*/
   free (matrix);
}

/*
 * This function erases the matrix widget from the screen.
 */
void eraseCDKMatrix (CDKMATRIX *matrix)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   /* Clear the matrix cells.						*/
   werase (matrix->cell[0][0]);
   wrefresh (matrix->cell[0][0]);
   for (x=1; x <= matrix->vrows; x++)
   {
      werase (matrix->cell[x][0]);
      wrefresh (matrix->cell[x][0]);
   }
   for (x=1; x <= matrix->vcols; x++)
   {
      werase (matrix->cell[0][x]);
      wrefresh (matrix->cell[0][x]);
   }
   for (x=1; x <= matrix->vrows; x++)
   {
      for (y=1; y <= matrix->vcols; y++)
      {
         werase (matrix->cell[x][y]);
         wrefresh (matrix->cell[x][y]);
      }
   }
   if (matrix->shadow)
   {
      werase (matrix->shadowwin);
      wrefresh (matrix->shadowwin);
   }
   werase (matrix->win);
   wrefresh (matrix->win);
}

/*
 * This function sets the values of the matrix widget.
 */
void setCDKMatrix (CDKMATRIX *matrix, char *info[MAXMATRIXROWS][MAXMATRIXCOLS], int rows, int *subSize)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   /* Clear out all the cells.						*/
   for (x=1; x <= matrix->rows; x++)
   {
      for (y=1; y <= matrix->cols; y++)
      {
         /* Clean the old info, and copy in the new.			*/
         if (matrix->info[x][y] != (char *)NULL)
         {
            /* Clean the cell information.				*/
            cleanChar (matrix->info[x][y], matrix->colwidths[y], '\0');
         }
      }
   }

   /* Copy in the new info.						*/
   for (x=1; x <= rows; x++)
   {
      for (y=1; y <= subSize[x]; y++)
      {
         /* Copy in the new information.				*/
         if (info[x][y] != (char *)NULL )
         {
            strncpy (matrix->info[x][y], info[x][y], matrix->colwidths[y]);
         }
      }
   }
}


/*
 * This cleans out the information cells in the matrix widget.
 */
void cleanCDKMatrix (CDKMATRIX *matrix)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   for (x=0; x < matrix->rows; x++)
   {
      for (y=0; y < matrix->cols; y++)
      {
         cleanChar (matrix->info[x][y], matrix->colwidths[y], '\0');
      }
   }
}

/*
 * This allows us to hyper-warp to a cell.
 */
int jumpToCell (CDKMATRIX *matrix)
{
   /* Declare local variables.					*/
   CDKSCALE *rowCell, *colCell;
   int newRow	= 0;
   int newCol	= 0;

   /* Create both the scales					*/
   rowCell	= newCDKScale (matrix->screen, 
				CENTER, CENTER, LEFT,
				"</5/B>Row: ", A_NORMAL, 5,
				1, 1, matrix->rows, 1, 1, TRUE, FALSE);
   colCell	= newCDKScale (matrix->screen, 
				CENTER, CENTER, LEFT,
				"</5/B>Col: ", A_NORMAL, 5,
				1, 1, matrix->cols, 1, 1, TRUE, FALSE);

   /* Activate 'em.						*/
   newRow	= activateCDKScale (rowCell); destroyCDKScale (rowCell);
   newCol	= activateCDKScale (colCell); destroyCDKScale (colCell);

   /* Hyper-warp....						*/
   if (newRow != matrix->row || newCol != matrix->col)
   {
      return (moveToCDKMatrixCell (matrix, newRow, newCol));
   }
   else
   {
      return (1);
   }
}

/*
 * This allows us to move to a given cell.
 */
int moveToCDKMatrixCell (CDKMATRIX *matrix, int newrow, int newcol)
{
   /* Declare local variables.					*/
   int rowShift	= newrow - matrix->row;
   int colShift	= newcol - matrix->col;

   /* Make sure we arent asking to move out of the matrix.	*/
   if (newrow > matrix->rows || newcol > matrix->cols || newrow <= 0 || newcol <= 0)
   {
      return (0);
   }
   
   /* Did we move up/down????					*/
   if (rowShift > 0)
   {
      /* We are moving down.					*/
      if (matrix->vrows == matrix->cols)
      {
         matrix->trow	= 1;
         matrix->crow	= newrow;
         matrix->row	= newrow;
      }
      else
      {
         if ((rowShift + matrix->vrows) < matrix->rows)
         {
            /* Just shift down by rowShift...			*/
            matrix->trow	+= rowShift;
            matrix->crow	= 1;
            matrix->row		+= rowShift;
         }
         else
         {
            /* We need to munge with the values...		*/
            matrix->trow	= matrix->rows - matrix->vrows + 1;
            matrix->crow	= ((rowShift + matrix->vrows) - matrix->rows) + 1;
            matrix->row		= newrow;
         }
      }
   }
   else if (rowShift < 0)
   {
      /* We are moving up.					*/
      if (matrix->vrows == matrix->rows)
      {
         matrix->trow	= 1;
         matrix->row	= newrow;
         matrix->crow	= newrow;
      }
      else
      {
         if ((rowShift + matrix->vrows) > 1)
         {
            /* Just shift up by rowShift...			*/
            matrix->trow	+= rowShift;
            matrix->row		+= rowShift;
            matrix->crow	= 1;
         }
         else
         {
            /* We need to munge with the values...		*/
            matrix->trow	= 1;
            matrix->crow	= 1;
            matrix->row		= 1;
         }
      }
   }

   /* Did we move left/right ????				*/
   if (colShift > 0)
   {
      /* We are moving right.					*/
      if (matrix->vcols == matrix->cols )
      {
         matrix->lcol	= 1;
         matrix->ccol	= newcol;
         matrix->col	= newcol;
      }
      else
      {
         if ((colShift + matrix->vcols) < matrix->cols)
         {
            matrix->lcol	+= colShift;
            matrix->ccol	= 1;
            matrix->col		+= colShift;
         }
         else
         {
            /* We need to munge with the values...		*/
            matrix->lcol	= matrix->cols - matrix->vcols + 1;
            matrix->ccol	= ((colShift + matrix->vcols) - matrix->cols) + 1;
            matrix->col		= newcol;
         }
      }
   }
   else if (colShift < 0)
   {
      /* We are moving left.					*/
      if (matrix->vcols == matrix->cols)
      {
            matrix->lcol	= 1;
            matrix->col		= newcol;
            matrix->ccol	= newcol;
      }
      else
      {
         if ((colShift + matrix->vcols) > 1)
         {
            /* Just shift left by colShift...			*/
            matrix->lcol	+= colShift;
            matrix->col		+= colShift;
            matrix->ccol	= 1;
         }
         else
         {
            matrix->lcol	= 1;
            matrix->col		= 1;
            matrix->ccol	= 1;
         }
      }
   }

   /* Keep the 'old' values around for redrawing sake.		*/
   matrix->oldcrow	= matrix->crow;
   matrix->oldccol	= matrix->ccol;
   matrix->oldvrow	= matrix->row;
   matrix->oldvcol	= matrix->col;

   /* Lets ...							*/
   return (1);
}

/*
 * This redraws the titles indicated...
 */
void redrawTitles (CDKMATRIX *matrix, int rowTitles, int colTitles)
{
   /* Declare local variables.					*/
   int x	= 0;

   /* Redraw the row titles.				*/
   if (rowTitles)
   {
      for (x=1; x <= matrix->vrows; x++)
      {
         werase (matrix->cell[x][0]);
         writeChtype (matrix->cell[x][0],
			matrix->rowtitlepos[matrix->trow+x-1], 1,
			matrix->rowtitle[matrix->trow+x-1], 
			HORIZONTAL, 
			0, matrix->rowtitlelen[matrix->trow+x-1]);
         wrefresh (matrix->cell[x][0]);
      }
   }

   /* Redraw the column titles.			*/
   if (colTitles)
   {
      for (x=1; x <=matrix->vcols; x++)
      {
         werase (matrix->cell[0][x]);
         writeChtype (matrix->cell[0][x],
			matrix->coltitlepos[matrix->lcol+x-1], 0,
			matrix->coltitle[matrix->lcol+x-1], 
			HORIZONTAL, 
			0, matrix->coltitlelen[matrix->lcol+x-1]);
      			wrefresh (matrix->cell[0][x]);
      }
   }
}
