#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/14 06:19:52 $
 * $Revision: 1.56 $
 */

/*
 * This creates a pointer to a muliple line entry widget.
 */
CDKMENTRY *newCDKMentry (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, char *label, chtype fieldattr, chtype filler, EDisplayType disptype, int fieldwidth, int fieldrows, int logical, int min, MENTRYCB callBackFunction, boolean box, boolean shadow)
{
   /* Set up some variables						*/
   CDKMENTRY *mentry	= (CDKMENTRY *)malloc (sizeof (CDKMENTRY));
   char *plainLabel;
   int boxwidth		= 0;
   int boxheight	= fieldrows + 2;
   int xpos		= xplace;
   int ypos		= yplace;
   int labelxoffset	= 0;
   int labelyoffset	= 0;
   int fieldxoffset	= 0;
   int fieldyoffset	= 0;
   int x		= 0;
   int junk		= 0;

   /* We need to translate the char * label to a chtype *		*/
   mentry->label	= char2Chtype (label, &mentry->labellen, &junk);
   plainLabel		= chtype2Char (mentry->label);

   /* Adjust the offset values given the location of the label.		*/
   alignlabel (plainLabel, lplace, fieldwidth, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainLabel);
   
   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Make the label window.						*/
   mentry->win		= newwin (boxheight, boxwidth, ypos, xpos);

   /* Is the window NULL???						*/
   if (mentry->win == (WINDOW *)NULL)
   {
      /* Free up any memory used.					*/
      freeChtype (mentry->label);
      free (mentry);
      
      /* Return a NULL pointer.						*/
      return ( (CDKMENTRY *)NULL );
   }

   /* Create the label window.						*/
   mentry->labelwin	= subwin (cdkscreen->window, fieldrows, (int)strlen(label)+1, ypos+labelyoffset, xpos+labelxoffset);

   /* Is the window NULL???						*/
   if (mentry->labelwin == (WINDOW *)NULL)
   {
      /* Free up any memory used.					*/
      delwin (mentry->win);
      freeChtype (mentry->label);
      free (mentry);
      
      /* Return a NULL pointer.						*/
      return ( (CDKMENTRY *)NULL );
   }

   /* Make the field window.						*/
   mentry->field	= subwin (cdkscreen->window, fieldrows, fieldwidth, ypos+fieldyoffset+x, xpos+fieldxoffset);

   /* Is the window NULL???						*/
   if (mentry->win == (WINDOW *)NULL)
   {
      /* Free up any memory used.					*/
      delwin (mentry->labelwin);
      delwin (mentry->win);
      freeChtype (mentry->label);
      free (mentry);
      
      /* Return a NULL pointer.						*/
      return ( (CDKMENTRY *)NULL );
   }

   /* Turn on the keypad.						*/
   keypad (mentry->field, TRUE);

   /* Set up the rest of the structure.			 		*/
   mentry->parent	= cdkscreen->window;
#ifdef AIX
   mentry->totalwidth	= 1000;
#else
   mentry->totalwidth	= (fieldwidth*fieldrows) + 1;
#endif

   /* Create the info char * pointer.					*/
   mentry->info		= (char *)malloc (sizeof (char) * (mentry->totalwidth+3));
   cleanChar (mentry->info, mentry->totalwidth+3, '\0');

   mentry->shadowwin	= (WINDOW *)NULL;
   mentry->fieldattr	= fieldattr;
   mentry->fieldwidth	= fieldwidth;
   mentry->rows		= fieldrows;
   mentry->filler	= filler;
   mentry->box		= box;
   mentry->currentrow	= 0;
   mentry->currentcol	= 0;
   mentry->toprow	= 0;
   mentry->shadow	= shadow;
   mentry->disptype	= disptype;
   mentry->min		= min;
   mentry->logical	= logical;
   mentry->callbackfn	= (void *)callBackFunction;

   /* Do we need to create a shadow.					*/
   if (shadow)
   {
      mentry->shadowwin	= newwin (boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vMENTRY, mentry);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vMENTRY, mentry);

   /* Return the pointer to the structure.				*/
   return (mentry);
}

/*
 * This actually manages the mentry widget...
 */
char *activateCDKMentry (CDKMENTRY *mentry)
{
   /* Declare local variables.						*/
   int adj	= 0;
   int temp	= 0;
   int x	= 0;
   chtype input;

   /* Add the filler character on the end of the info.			*/
   mentry->info[(int)strlen(mentry->info)] = mentry->filler;

   /* Draw the field.							*/
   drawCDKMentry (mentry, mentry->box);

   /* Let them play with the widget....					*/
   for (;;)
   {
      input = wgetch (mentry->field);

      /* Check for a key binding...					*/
      if (! checkCDKObjectBind (vMENTRY, mentry, input))
      {
         switch (input)
         {
            case KEY_LEFT :
                 if (mentry->currentcol != 0)
                 {
                    mentry->currentcol--;
                    wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                    wrefresh (mentry->field);
                 }
                 else
                 {
                    if (mentry->currentrow == 0)
                    {
                       if (mentry->toprow == 0)
                       {
                          Beep();
                       }
                       else
                       {
                          /* Move up one row.				*/
                          mentry->currentcol = mentry->fieldwidth-1;
                          mentry->toprow--;
                          redrawCDKMentry ( mentry );
                       }
                    }
                    else
                    {
                       mentry->currentrow--;
                       mentry->currentcol = mentry->fieldwidth - 1;
                       wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                       wrefresh (mentry->field);
                    }
                 }
                 break;
   
            case KEY_RIGHT :
                 if (mentry->currentcol < (mentry->fieldwidth - 1))
                 {
                    if ((((mentry->toprow + mentry->currentrow) * mentry->fieldwidth) + mentry->currentcol+1) > (int)strlen (mentry->info)-1)
                    {
                       Beep();
                    }
                    else
                    {
                       mentry->currentcol++;
                       wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                       wrefresh (mentry->field);
                    }
                 }
                 else
                 {
                    if (mentry->currentrow == mentry->rows - 1)
                    {
                       if ( (mentry->toprow + mentry->currentrow + 1) <= mentry->logical)
                       {
                          mentry->currentcol = 0;
                          mentry->toprow++;
                          redrawCDKMentry ( mentry );
                       }
                       else
                       {
                          Beep();
                       }
                    }
                    else
                    {
                       mentry->currentcol = 0;
                       mentry->currentrow++;
                       wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                       wrefresh (mentry->field);
                    }
                 }
                 break;

            case KEY_DOWN :
                 if (mentry->currentrow != (mentry->rows-1))
                 {
                    if ((((mentry->toprow + mentry->currentrow + 1) * mentry->fieldwidth) + mentry->currentcol) > (int)strlen (mentry->info)-1)
                    {
                       Beep();
                    }
                    else
                    {
                       mentry->currentrow++;
                       wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                       wrefresh (mentry->field);
                    }
                 }
                 else
                 {
                    if ( mentry->toprow >= mentry->logical - mentry->rows)
                    {
                       Beep();
                    }
                    else
                    {
                       mentry->toprow++;
                       redrawCDKMentry ( mentry );
                    }
                 }
                 break;

            case KEY_UP :
                 if (mentry->currentrow != 0)
                 {
                    mentry->currentrow--;
                    wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                    wrefresh (mentry->field);
                 }
                 else
                 {
                    if (mentry->toprow == 0)
                    {
                       Beep();
                    }
                    else
                    {
                       mentry->toprow--;
                       redrawCDKMentry ( mentry );
                    }
                 }
                 break;

            case DELETE : case KEY_BACKSPACE :
                 if (mentry->disptype == vVIEWONLY)
                 {
                    Beep();
                 }
                 else
                 {
                    /* Add the character to the character array.	*/
                    adj	= ( (mentry->currentrow + mentry->toprow) * mentry->fieldwidth) + mentry->currentcol;
                    temp	= (int)strlen (mentry->info);
   
                    /* If there is nothing left to delete, then beep.	*/
                    if ( (int)strlen (mentry->info) <= 1)
                    {
                       Beep();
                    }
                    else
                    { 
                       if (adj != temp-1)
                       {
                          /* We are deleting from the middle of the string. */
                          for (x = adj; x < temp; x++)
                          {
                             mentry->info[x]	= mentry->info[x+1];
                          }
                          mentry->info[temp]	= '\0';
      
                          /* We have to redraw the screen.		*/
                          redrawCDKMentry ( mentry );
                       }
                       else
                       {
                          /* We are deleting from the end of the string.*/
                          mentry->info[temp-1] = '\0';
                          mentry->info[temp-2] = mentry->filler;
                       
                          /* Delete from the screen now.		*/
                          if (mentry->currentcol == 0)
                          {
                             if (mentry->currentrow == 0)
                             {
                                if (mentry->toprow != 0)
                                {
                                   mentry->toprow--;
                                   mentry->currentcol = mentry->fieldwidth-1;
                                   redrawCDKMentry ( mentry );
                                   mvwaddch (mentry->field, mentry->currentrow, mentry->currentcol, mentry->filler);
                                   wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                                   wrefresh (mentry->field);
                                }
                                else
                                {
                                   Beep();
                                }
                             }
                             else
                             {
                                mentry->currentrow --;
                                mentry->currentcol = mentry->fieldwidth-1;
                                mvwaddch (mentry->field, mentry->currentrow, mentry->currentcol, mentry->filler);
                                wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                                wrefresh (mentry->field);
                             }
                          }
                          else
                          {
                             mentry->currentcol--;
                             mvwaddch (mentry->field, mentry->currentrow, mentry->currentcol, mentry->filler);
                             wmove (mentry->field, mentry->currentrow, mentry->currentcol);
                             wrefresh (mentry->field);
                          }
                       }
                    }
                 }
                 break;

            case KEY_RETURN : case KEY_TAB : case CDK_NEXT :
                 if ((int)strlen (mentry->info) < mentry->min+1) 
                 {
                    Beep();
                 }
                 else
                 {
                    mentry->info[(int)strlen(mentry->info)-1] = '\0';
                    return (mentry->info);
                 }
                 break;

            case CDK_REFRESH :
                 refreshCDKScreen (mentry->screen);
                 break;

            default :
                 /* Make sure we havent gone out of bounds.		*/
                 if ( (int)strlen (mentry->info) <= mentry->totalwidth)
                 {
                    if (mentry->disptype == vVIEWONLY)
                    {
                       Beep();
                    }
                    else
                    {
                       ((MENTRYCB)mentry->callbackfn)(mentry, input);
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
         }
      }
   }
}

/*
 * This function redraws the multiple line entry field.
 */
void redrawCDKMentry (CDKMENTRY *mentry)
{
   /* Declare local variables.					*/
   int currchar	= (mentry->fieldwidth * mentry->toprow);
   int length	= 0;
   int lastpos	= 0;
   int x, y;

   /* Set up some variables.					*/
   if (mentry->info == (char *)NULL)
   {
      return;
   }

   /* The information isn't NULL, redraw the field.		*/
   length	= (int)strlen (mentry->info);
   lastpos	= (mentry->info[length] == mentry->filler ? length-1 : length);

   /* Start redrawing the fields.				*/
   for (x=0; x < mentry->rows; x++)
   {
      for (y=0; y < mentry->fieldwidth ; y++)
      {
         if (currchar < lastpos)
         {
            mvwaddch (mentry->field, x, y, mentry->info[currchar++]|mentry->fieldattr);
         }
         else
         {
            mvwaddch (mentry->field, x, y, mentry->filler);
         }
      }
   }
   wmove (mentry->field, mentry->currentrow, mentry->currentcol);
   wrefresh (mentry->field);
}

/*
 * This is a generic character parser for the mentry field. It is used as a 
 * callback function, so any personal modifications can be made by creating
 * a new function and calling that one the mentry activation.
 */
void CDKMentryCallBack (CDKMENTRY *mentry, chtype character)
{
   /* Declare local variables.						*/
   int adj	= ( (mentry->currentrow + mentry->toprow) * mentry->fieldwidth) + mentry->currentcol;
   int temp	= (int)strlen (mentry->info);
   chtype newchar;
   int x;

   /* Check the type of character we are looking for.			*/
   if (mentry->disptype == vINT && !isdigit((char)character))
   {
      Beep();
   }
   else if ( (	mentry->disptype == vCHAR || mentry->disptype == vUCHAR || \
		mentry->disptype == vLCHAR || mentry->disptype == vUHCHAR || \
		mentry->disptype == vLHCHAR) && isdigit((char)character) )
   {
      Beep();
   }
   else
   {
      /* We will make any adjustments to the case of the character.	*/
      newchar = character;
      if ( (	mentry->disptype == vUCHAR || mentry->disptype == vUHCHAR || \
		mentry->disptype == vUMIXED || mentry->disptype == vUHMIXED) && \
		!isdigit((char)character) )
      {
         newchar = toupper (character);
      }
      else if ( (	mentry->disptype == vLCHAR || mentry->disptype == vLHCHAR ||
			mentry->disptype == vLMIXED || mentry->disptype == vLHMIXED) &&
			!isdigit((char)character) )
      {
         newchar = tolower (character);
      }

      /* Check if we are adding onto the end or we are adding		*/
      /* onto the end of the string.					*/
      if (adj != temp-1)
      {
         /* We are adding in the middle of the string.			*/
         for (x = temp+1; x > adj; x--)
         {
            mentry->info[x]	= mentry->info[x-1];
         }
         mentry->info[adj]	= newchar;

         /* We have to redraw the screen.				*/
         redrawCDKMentry ( mentry );
      }
      else
      {
         /* We are adding onto the end of the string.			*/
         mentry->info[temp-1]	= newchar;
         mentry->info[temp]	= mentry->filler;
   
         /* Add the character on the screen				*/
         mvwaddch (mentry->field, mentry->currentrow, mentry->currentcol++, newchar|mentry->fieldattr);
         wrefresh (mentry->field);
   
         /* Have we gone out of bounds.					*/
         if (mentry->currentcol == mentry->fieldwidth)
         {
            /* Update the row and col values.				*/
            mentry->currentcol = 0;
            mentry->currentrow++;
   
            /* If we have gone outside of the visual boundries, we	*/
            /* need to scroll the window.				*/
            if ( mentry->currentrow == mentry->rows )
            {
               /* We have to redraw the screen.				*/
               mentry->currentrow--;
               mentry->toprow++;
               redrawCDKMentry ( mentry );
            }
            wmove (mentry->field, mentry->currentrow, mentry->currentcol );
            wrefresh (mentry->field);
         }
      }
   }
}

/*
 * This function draws the multiple line entry field.
 */
void drawCDKMentry (CDKMENTRY *mentry, boolean Box)
{
   /* Keep the box information.						*/
   mentry->box	= Box;

   /* Erase the mentry widget from the screen.				*/
   eraseCDKMentry (mentry);

   /* Box the widget if asked.						*/
   if (Box)
   {
      box (mentry->win, ACS_VLINE, ACS_HLINE);
      wrefresh (mentry->win);
   }

   /* Do we need to draw in the shadow???				*/
   if (mentry->shadow)
   {
      drawShadow (mentry->shadowwin);
   }

   /* Draw in the label to the widget.					*/
   writeChtype (mentry->labelwin, 0, 0, mentry->label, HORIZONTAL, 0, mentry->labellen);
   wrefresh (mentry->labelwin);

   /* Draw the mentry field.						*/
   redrawCDKMentry (mentry);
}

/*
 * This function erases the multiple line entry field from the screen.
 */
void eraseCDKMentry (CDKMENTRY *mentry)
{
   /* Erase the mentry field.						*/
   werase (mentry->field);
   werase (mentry->labelwin);
   werase (mentry->win);
   wrefresh (mentry->win);
   
   /* Do we need to erase the shadow.					*/
   if (mentry->shadowwin)
   {
      werase (mentry->shadowwin);
      wrefresh (mentry->shadowwin);
   }
}

/*
 * This function destroys a multiple line entry field widget.
 */
void destroyCDKMentry (CDKMENTRY *mentry)
{
   /* Erase the object.							*/
   eraseCDKMentry (mentry);

   /* Clean up the char pointers.					*/
   freeChtype (mentry->label);
   freeChar (mentry->info);

   /* Clean up the windows.						*/
   delwin (mentry->field);
   delwin (mentry->win);
   delwin (mentry->labelwin);
   if (mentry->shadow)
   {
      delwin (mentry->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vMENTRY, mentry);

   /* Finish cleaning up.						*/
   free (mentry);
}

/*
 * This removes the old information in the entry field and keeps the
 * new information given.
 */
void setCDKMentry (CDKMENTRY *mentry, char *newValue, int min, boolean box)
{
   /* Declare local variables.						*/
   int len		= 0;
   int copychars	= 0;

   /* Keep the values passed in.					*/
   mentry->box	= box;
   mentry->min	= min;

   /* Just to be sure, if lets make sure the new value isn't NULL.	*/
   if (newValue == (char *)NULL)
   { 
      /* Then we want to just erase the old value.			*/
      cleanChar (mentry->info, mentry->totalwidth, '\0');
      return;
   }

   /* Determine how many characters we need to copy.			*/
   len		= (int)strlen (newValue);
   copychars	= (len < mentry->totalwidth ? len : mentry->totalwidth);

   /* OK, erase the old value, and copy in the new value.		*/
   cleanChar (mentry->info, mentry->totalwidth, '\0');
   strncpy (mentry->info, newValue, copychars);
}

/*
 * This erases the information in the multiple line entry widget.
 */
void cleanCDKMentry (CDKMENTRY *mentry)
{
   char plainchar	= mentry->filler & A_CHARTEXT;
   cleanChar (mentry->info, mentry->totalwidth, '\0');
   mentry->info[0]	= plainchar;
}

