#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.34 $
 */

/*
 * This creates a new menu widget.
 */
CDKMENU *newCDKMenu (CDKSCREEN *cdkscreen, char *menulist[MAXMENUITEMS][MAXSUBITEMS], int menuitems, int *subsize, int *menuloc, chtype titleattr, chtype subtitleattr)
{
   /* Declare local variables.						*/
   CDKMENU *menu	= (CDKMENU *)malloc (sizeof (CDKMENU));
   int rightcount	= menuitems-1;
   int rightloc		= (cdkscreen->window->_maxx) - 1;
   int leftloc		= 0;
#ifdef NODYNALLOC
   int maxwidth[MAXMENUITEMS];
#else
   int maxwidth[menuitems];
#endif
   int x, y, max, junk;

   /* Start making a copy of the information.				*/
   rightcount			= menuitems-1;
   menu->parent			= cdkscreen->window;
   menu->menuitems		= menuitems;
   menu->titleattr		= titleattr;
   menu->subtitleattr		= subtitleattr;
   menu->currentTitle		= 0;
   menu->currentSubtitle	= 0;

   /* Create the pull down menus.					*/
   for (x=0; x < menuitems; x++)
   {
      max			= -1;
      for (y=1; y < subsize[x] ; y++)
      {
         if (menuloc[x] == LEFT)
         {
            menu->sublist[x][y-1]		= char2Chtype (menulist[x][y], &menu->sublistlen[x][y-1], &junk);
            max	= MAX (max, menu->sublistlen[x][y-1]);
         }
         else
         {
            menu->sublist[rightcount][y-1]	= char2Chtype (menulist[x][y], &menu->sublistlen[rightcount][y-1], &junk);
            max	= MAX (max, menu->sublistlen[rightcount][y-1]);
         }
      }

      if (menuloc[x] == LEFT)
      {
         /* Its a menu item on the left, add it to the left side 	*/
         /* of the menu.						*/
         menu->title[x]		= char2Chtype (menulist[x][0], &menu->titlelen[x], &junk);
         menu->subsize[x]	= subsize[x] - 1;
         maxwidth[x] 		= max+1;
         menu->titlewin[x]	= subwin (cdkscreen->window, 1, menu->titlelen[x]+2, 0, leftloc);
         menu->pullwin[x]	= subwin (cdkscreen->window, subsize[x]+1, max+2, 1, leftloc);
         leftloc		+= menu->titlelen[x] + 1;

         /* Allow the windows to use the keypad.			*/
         keypad (menu->titlewin[x], TRUE);
         keypad (menu->pullwin[x], TRUE);
      }
      else 
      {
         /* Its a menu item on the right, add it to the right side	*/
         /* of the menu.						*/
         rightloc			-= max + 3;
         menu->title[rightcount]	= char2Chtype (menulist[x][0], &menu->titlelen[rightcount], &junk);
         menu->subsize[rightcount]	= subsize[x] - 1;
         maxwidth[rightcount] 		= max+1;
         menu->titlewin[rightcount]	= subwin (cdkscreen->window, 1, menu->titlelen[x]+2, 0, rightloc);
         menu->pullwin[rightcount]	= subwin (cdkscreen->window, subsize[x]+1, max+2, 1, rightloc);

         /* Allow the windows to use the keypad.			*/
         keypad (menu->titlewin[rightcount], TRUE);
         keypad (menu->pullwin[rightcount], TRUE);
         rightcount--;
      }
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vMENU, menu);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vMENU, menu);

   /* Return the menu object.						*/
   return (menu);
}

/*
 * This activates the CDK Menu.
 */
int activateCDKMenu (CDKMENU *menu)
{
   /* Declare local variables.						*/
   chtype input;

   /* Draw in the screen.						*/
   refreshCDKScreen (menu->screen);

   /* Display the menu titles.						*/
   drawCDKMenu(menu);

   /* Turn the cursor off.						*/
   curs_set (0);

   /* Highlight the current title and window.				*/
   drawCDKMenuSubwin (menu);

   /* Start taking input from the keyboard.				*/
   for (;;)
   {
      /* Get the input.							*/
      input = wgetch (menu->titlewin[menu->currentTitle]);

      /* Check for key bindings.					*/
      if ( ! checkCDKObjectBind (vMENU, menu, input) )
      {
         switch (input)
         {
            case KEY_LEFT :
                 /* Erase the menu sub-window				*/
                 eraseCDKMenuSubwin (menu);
                 refreshCDKScreen (menu->screen);

                 /* Set the values.					*/
                 menu->currentSubtitle		= 0;
                 if (menu->currentTitle == 0)
                 {
                    menu->currentTitle		= menu->menuitems - 1;
                 }
                 else
                 {
                    menu->currentTitle--;
                 }

                 /* Draw the new menu sub-window.			*/
                 drawCDKMenuSubwin (menu);
                 break;

            case KEY_RIGHT : case KEY_TAB :
                 /* Erase the menu sub-window				*/
                 eraseCDKMenuSubwin (menu);
                 refreshCDKScreen (menu->screen);

                 /* Set the values.					*/
                 menu->currentSubtitle		= 0;
                 if (menu->currentTitle == menu->menuitems-1)
                 {
                    menu->currentTitle		= 0;
                 }
                 else
                 {
                    menu->currentTitle++;
                 }

                 /* Draw the new menu sub-window.			*/
                 drawCDKMenuSubwin (menu);
                 break;

            case KEY_UP :
                 /* Erase the old subtitle.				*/
                 writeChtype (	menu->pullwin[menu->currentTitle],
				1, menu->currentSubtitle+1,
				menu->sublist[menu->currentTitle][menu->currentSubtitle],
				HORIZONTAL,
				0, menu->sublistlen[menu->currentTitle][menu->currentSubtitle]);

                 /* Set the values.					*/
                 if (menu->currentSubtitle == 0)
                 {
                    menu->currentSubtitle	= menu->subsize[menu->currentTitle] - 1;
                 }
                 else
                 {
                    menu->currentSubtitle--;
                 }

                 /* Draw the new sub-title.				*/
                 writeChtypeAttrib (
				menu->pullwin[menu->currentTitle],
				1, menu->currentSubtitle+1,
				menu->sublist[menu->currentTitle][menu->currentSubtitle],
				menu->subtitleattr,
				HORIZONTAL,
				0, menu->sublistlen[menu->currentTitle][menu->currentSubtitle]);
                 wrefresh (menu->pullwin[menu->currentTitle]);
                 break;

            case KEY_DOWN : case SPACE :
                 /* Erase the old subtitle.				*/
                 writeChtype (	menu->pullwin[menu->currentTitle],
				1, menu->currentSubtitle+1,
				menu->sublist[menu->currentTitle][menu->currentSubtitle],
				HORIZONTAL,
				0, menu->sublistlen[menu->currentTitle][menu->currentSubtitle]);

                 /* Set the values.					*/
                 if (menu->currentSubtitle == menu->subsize[menu->currentTitle]-1)
                 {
                    menu->currentSubtitle	= 0;
                 }
                 else
                 {
                    menu->currentSubtitle++;
                 }

                 /* Draw the new sub-title.				*/
                 writeChtypeAttrib (
				menu->pullwin[menu->currentTitle],
				1, menu->currentSubtitle+1,
				menu->sublist[menu->currentTitle][menu->currentSubtitle],
				menu->subtitleattr,
				HORIZONTAL,
				0, menu->sublistlen[menu->currentTitle][menu->currentSubtitle]);
                 wrefresh (menu->pullwin[menu->currentTitle]);
                 break;
 
            case KEY_RETURN :
                 cleanUpMenu (menu);
                 return ((menu->currentTitle * 100) + menu->currentSubtitle);
                 break;

            case CDK_NEXT :
                 cleanUpMenu (menu);
                 return (-1);
                 break;
     
            case CDK_REFRESH :
                 refreshCDKScreen (menu->screen);
                 break;
         }
      }
   }
}

/* This draws a menu item subwindow					*/
void drawCDKMenuSubwin (CDKMENU *menu)
{
   /* Declare local variables.						*/
   int x;

   /* Box the window							*/
   box (menu->pullwin[menu->currentTitle], ACS_VLINE, ACS_HLINE);
   mvwaddch (menu->pullwin[menu->currentTitle], 0, 0, ACS_LTEE);

   /* Draw in the items.						*/
   for (x=0; x < menu->subsize[menu->currentTitle] ; x++)
   {
      writeChtype (	menu->pullwin[menu->currentTitle],
			1, x+1,
			menu->sublist[menu->currentTitle][x],
			HORIZONTAL,
			0, menu->sublistlen[menu->currentTitle][x]);
   }

   /* Highlight the current sub-menu item.				*/
   writeChtypeAttrib (	menu->pullwin[menu->currentTitle],
			1, menu->currentSubtitle+1,
			menu->sublist[menu->currentTitle][menu->currentSubtitle],
			menu->subtitleattr,
			HORIZONTAL,
			0, menu->sublistlen[menu->currentTitle][menu->currentSubtitle]);
   wrefresh (menu->pullwin[menu->currentTitle]);

   /* Highlight the title.						*/
   writeChtypeAttrib (	menu->titlewin[menu->currentTitle],
			0, 0,
			menu->title[menu->currentTitle],
			menu->titleattr,
			HORIZONTAL,
			0,
			menu->titlelen[menu->currentTitle]);
   wrefresh (menu->titlewin[menu->currentTitle]);
}

/* This erases a menu item subwindow.					*/
void eraseCDKMenuSubwin (CDKMENU *menu)
{
   /* Erase the sub-menu.						*/
   werase (menu->pullwin[menu->currentTitle]);
   wrefresh (menu->pullwin[menu->currentTitle]);

   /* Redraw the sub-menu title.					*/
   writeChtype(	menu->titlewin[menu->currentTitle],
		0, 0,
		menu->title[menu->currentTitle],
		HORIZONTAL,
		0,
		menu->titlelen[menu->currentTitle]);
   wrefresh (menu->titlewin[menu->currentTitle]);
}

/*
 * This function draws the menu.
 */
void drawCDKMenu (CDKMENU *menu)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the old object.						*/
   eraseCDKMenu (menu);

   /* Draw in the menu titles.						*/
   for (x=0; x < menu->menuitems; x++)
   {
      writeChtype (menu->titlewin[x], 0, 0, menu->title[x], HORIZONTAL, 0, menu->titlelen[x]);
      wrefresh (menu->titlewin[x]);
   }
}

/*
 * This function destroys a menu widget.
 */
void destroyCDKMenu (CDKMENU *menu)
{
   /* Declare local variables.						*/
   int x	= 0;
   int y	= 0;

   /* Erase the object.							*/
   eraseCDKMenu (menu);

   /* Clean up both the winodws and the char pointers.			*/
   for (x=0; x < menu->menuitems; x++)
   {
      /* Clean the windows.						*/
      delwin (menu->titlewin[x]);
      delwin (menu->pullwin[x]);

      /* Delete the character pointers.					*/
      freeChtype (menu->title[x]);
      for (y=0; y < menu->subsize[x] ; y++)
      {
         freeChtype (menu->sublist[x][y]);
      }
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vMENU, menu);

   /* Finish cleaning up.						*/
   free (menu);
}

/*
 * This function erases the menu widget from the screen.
 */
void eraseCDKMenu (CDKMENU *menu)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the menu items.						*/
   for (x=0 ; x < menu->menuitems; x++)
   {
      werase (menu->titlewin[x]);
      wrefresh (menu->titlewin[x]);
      werase (menu->pullwin[x]);
      wrefresh (menu->pullwin[x]);
   }
}

/*
 * This sets certain features of the menu.
 */
void setCDKMenu (CDKMENU *menu, int menuitem, int submenuitem, chtype titlehighlight, chtype subtitlehighlight)
{
   menu->currentTitle		= menuitem;
   menu->currentSubtitle	= submenuitem;
   menu->titleattr		= titlehighlight;
   menu->subtitleattr		= subtitlehighlight;
}

/* 
 * This exits the menu.
 */
void cleanUpMenu (CDKMENU *menu)
{
   /* Reset the cursor.							*/
   curs_set (1);

   /* Erase the sub-menu.						*/
   eraseCDKMenuSubwin (menu);
   wrefresh (menu->pullwin[menu->currentTitle]);

   /* Refresh the screen.						*/
   refreshCDKScreen (menu->screen);
}
