#include "cdk.h"

/*
 * $Author: mikeg $
 * $Date: 1995/12/14 19:49:11 $
 * $Revision: 1.29 $
 */

/*
 * This function creates the radio widget.
 */
CDKRADIO *newCDKRadio (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char *title, char **list, int listSize, chtype choiceChar, int defItem, chtype highlight, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKRADIO *radio	= (CDKRADIO *)malloc (sizeof (CDKRADIO));
   int boxWidth		= width + 5;
   int boxHeight	= height;
   int xpos		= xplace;
   int ypos		= yplace;
   int widestItem	= 0;
   int x		= 0;

   /* Translate the char * title to a chtype *				*/
   if (title != (char *)NULL)
   {
      radio->title	= char2Chtype (title, &radio->titlelen, &radio->titlepos);
      radio->titlepos	= justifyString (width, radio->titlelen, radio->titlepos);
      radio->titleadj	= 2;
      radio->listSize	= listSize;
      radio->viewSize	= boxHeight - 3;
      radio->lastItem	= listSize - 1;
      radio->maxTopItem	= listSize - radio->viewSize;
   }
   else
   {
      radio->title	= (chtype *)NULL;
      radio->titlepos	= 0;
      radio->titleadj	= 1;
      radio->titlelen	= 0;
      radio->listSize	= listSize;
      radio->viewSize	= boxHeight - 2;
      radio->lastItem	= listSize - 1;
      radio->maxTopItem	= listSize - radio->viewSize;
   }

   /* Is the view size smaller then the window???			*/
   if (listSize < (boxHeight-1-radio->titleadj))
   {
      radio->viewSize	= listSize;
      radio->listSize	= listSize;
      radio->lastItem	= listSize;
      radio->maxTopItem	= -1;
   }

   /* Each item in the needs to be converted to chtype *		*/
   for (x=0; x < listSize; x++)
   {
      radio->item[x]	= char2Chtype (list[x], &radio->itemlen[x], &radio->itempos[x]);
      radio->itempos[x]	= justifyString (width, radio->itemlen[x], radio->itempos[x]) + 3;
      widestItem	= MAX(widestItem, radio->itemlen[x]);
   }

   /* Determine how many characters we can shift to the right		*/
   /* before all the items have been scrolled off the screen.		*/
   if (boxWidth > widestItem)
   {
      radio->maxLeftChar = 0;
   }
   else
   {
      radio->maxLeftChar = widestItem-(boxWidth-5);
   }

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the radio window						*/
   radio->win		= newwin (boxHeight, boxWidth, ypos, xpos);

   /* Is the window NULL???						*/
   if (radio->win == (WINDOW *)NULL)
   {
      /* Clean up memory.						*/
      for (x=0; x < listSize; x++)
      {
         freeChtype (radio->item[x]);
      }
      freeChtype (radio->title);
      free (radio);

      /* Return a NULL pointer.						*/
      return ( (CDKRADIO *)NULL );
   }

   /* Turn on the keypad.						*/
   keypad (radio->win, TRUE);

   /* Set the rest of the variables					*/
   radio->parent	= cdkscreen->window;
   radio->boxHeight	= boxHeight;
   radio->boxWidth	= boxWidth;
   radio->widestItem	= widestItem;
   radio->currentTop	= 0;
   radio->currentItem	= 0;
   radio->currentHigh	= 0;
   radio->leftChar	= 0;
   radio->selectedItem	= 0;
   radio->highlight	= highlight;
   radio->choiceChar	= choiceChar;
   radio->defItem	= defItem;
   radio->box		= box;
   radio->shadow	= shadow;

   /* Do we need to create the shadow???				*/
   if (shadow)
   {
      radio->shadowwin	= newwin (boxHeight, boxWidth+1, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vRADIO, radio);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vRADIO, radio);

   /* Return the radio list						*/
   return (radio);
}

/*
 * This actually manages the radio widget.
 */
int activateCDKRadio (CDKRADIO *radio)
{
   /* Declare local variables.						*/
   int x		= 0;
   char emptyString[100];
   chtype input;

   /* Create a string full of spaces.					*/
   cleanChar (emptyString, radio->boxWidth, '\0');
   cleanChar (emptyString, radio->boxWidth-2, ' ');

   /* Turn off the cursor.						*/
   curs_set (0);

   /* Draw the radio list						*/
   drawCDKRadio (radio, radio->box);

   /* Do this until KEY_RETURN is hit.					*/
   for (;;)
   {
      input = wgetch (radio->win);

      /* Check for a predefined key binding.				*/
      if ( ! checkCDKObjectBind (vRADIO, radio, input) )
      {
         switch (input)
         {
            case KEY_UP :
                 if (radio->currentHigh == 0)
                 {
                    if (radio->currentTop != 0)
                    {
                       radio->currentTop--;
                       radio->currentItem--;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    radio->currentItem--;
                    radio->currentHigh--;
                 }
                 break;
   
            case KEY_DOWN : case KEY_TAB : 
                 if (radio->currentHigh == radio->viewSize - 1)
                 {
                    /* We need to scroll down one line.			*/
                    if (radio->currentTop < radio->maxTopItem)
                    {
                       radio->currentTop++;
                       radio->currentItem++;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    radio->currentItem++;
                    radio->currentHigh++;
                 }
                 break;
   
            case KEY_RIGHT :
                 if (radio->leftChar >= radio->maxLeftChar)
                 {
                    Beep();
                 }
                 else
                 {
                    radio->leftChar ++;
                 }
                 break;
   
            case KEY_LEFT :
                 if (radio->leftChar == 0)
                 {
                    Beep();
                 }
                 else
                 {
                    radio->leftChar --;
                 }
                 break;
   
            case KEY_PPAGE : case '' :
                 if (radio->currentTop > 0)
                 {
                    if ( radio->currentTop >= (radio->viewSize -1) )
                    {
                       radio->currentTop	-= (radio->viewSize - 1);
                       radio->currentItem	-= (radio->viewSize - 1);
                    }
                    else
                    {
                       radio->currentTop 	= 0;
                       radio->currentItem	= radio->currentHigh;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_NPAGE : case '' :
                 if (radio->currentTop < radio->maxTopItem)
                 {
                    if ( (radio->currentTop + radio->viewSize - 1) <= radio->maxTopItem)
                    {
                       radio->currentTop	+= (radio->viewSize - 1);
                       radio->currentItem	+= (radio->viewSize - 1);
                    }
                    else
                    {
                       radio->currentTop 	= radio->maxTopItem;
                       radio->currentItem	= radio->lastItem;
                       radio->currentHigh	= radio->viewSize-1;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case 'g' : case '1' :
                 radio->currentTop	= 0;
                 radio->currentItem	= 0;
                 radio->currentHigh	= 0;
                 break;
   
            case 'G' :
                 radio->currentTop 	= radio->maxTopItem;
                 radio->currentItem	= radio->lastItem;
                 radio->currentHigh	= radio->viewSize-1;
                 break;
   
            case '$' :
                 radio->leftChar	= radio->maxLeftChar;
                 break;
   
            case '|' :
                 radio->leftChar	= 0;
                 break;
   
            case SPACE :
                 radio->selectedItem	= radio->currentItem;
                 break;
   
            case KEY_RETURN :
                 return (radio->selectedItem);
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (radio->screen);
                 break;
   
            default :
                 break;
         }
      }
      
      /* Redraw the list						*/
      for (x=0; x < radio->viewSize; x++)
      {
         writeChar (radio->win, 1,
			radio->titleadj+x,
			emptyString,
			HORIZONTAL, 0,
			strlen(emptyString));

         writeChtype (radio->win,
			radio->itempos[x+radio->currentTop],
			x+radio->titleadj,
			radio->item[x+radio->currentTop],
			HORIZONTAL,
			radio->leftChar,
			radio->itemlen[x+radio->currentTop]);

         /* Draw in the selected choice...				*/
         if (x+radio->currentTop == radio->selectedItem)
         {
            mvwaddch ( radio->win, x+radio->titleadj, 1, '[');
            mvwaddch ( radio->win, x+radio->titleadj, 2, radio->choiceChar);
            mvwaddch ( radio->win, x+radio->titleadj, 3, ']');
         }
         else
         {
            mvwaddch ( radio->win, x+radio->titleadj, 1, '[');
            mvwaddch ( radio->win, x+radio->titleadj, 2, ' ');
            mvwaddch ( radio->win, x+radio->titleadj, 3, ']');
         }
      }

      /* Highlight the current item.					*/
      writeChtypeAttrib (radio->win,
			radio->itempos[radio->currentItem],
			radio->currentHigh+radio->titleadj,
			radio->item[radio->currentItem],
			radio->highlight,
			HORIZONTAL,
			radio->leftChar,
			radio->itemlen[radio->currentItem]);

      /* Box it if needed.						*/
      if (radio->box)
      {
         box (radio->win, ACS_VLINE, ACS_HLINE);
      }
      wrefresh (radio->win);
   }
}

/*
 * This function draws the radio widget.
 */
void drawCDKRadio (CDKRADIO *radio, boolean Box)
{
   /* Declare local variables.						*/
   int x		= 0;

   /* Erase the old object...						*/
   eraseCDKRadio (radio);

   /* Do we need to draw in the shadow???				*/
   if (radio->shadow)
   {
      drawShadow (radio->shadowwin);
   }

   /* Draw in the title if there is one					*/
   if (radio->title != (chtype *)NULL)
   {
      writeChtype (radio->win, radio->titlepos, 1, radio->title, HORIZONTAL, 0, radio->titlelen);
   }

   /* Draw in each item until we can't anymore				*/
   for (x=0; x < radio->viewSize; x++)
   {
      /* Draw in the item.						*/
      writeChtype (radio->win,
			radio->itempos[x+radio->currentTop],
			x+radio->titleadj+radio->currentTop,
			radio->item[x+radio->currentTop],
			HORIZONTAL,
			radio->leftChar,
			radio->itemlen[x+radio->currentTop]);

      /* Draw in the selection field.					*/
      if (x+radio->currentTop == radio->selectedItem)
      {
         mvwaddch ( radio->win, x+radio->titleadj, 1, '[');
         mvwaddch ( radio->win, x+radio->titleadj, 2, radio->choiceChar);
         mvwaddch ( radio->win, x+radio->titleadj, 3, ']');
      }
      else
      {
         mvwaddch ( radio->win, x+radio->titleadj, 1, '[');
         mvwaddch ( radio->win, x+radio->titleadj, 2, ' ');
         mvwaddch ( radio->win, x+radio->titleadj, 3, ']');
      }
   }
   writeChtypeAttrib (radio->win,
			radio->itempos[radio->currentItem],
			radio->currentHigh+radio->titleadj,
			radio->item[radio->currentItem],
			radio->highlight,
			HORIZONTAL, radio->leftChar,
			radio->itemlen[radio->currentItem]);

   /* Box it if needed							*/
   if (Box)
   {
      box (radio->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (radio->win);
}

/*
 * This function destroys the radio widget.
 */
void destroyCDKRadio (CDKRADIO *radio)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the object.							*/
   eraseCDKRadio (radio);

   /* Clear up the char pointers.					*/
   freeChtype (radio->title);
   for (x=0; x < radio->listSize; x++)
   {
      freeChtype (radio->item[x]);
   }

   /* Clean up the windows.						*/
   delwin (radio->win);
   if (radio->shadow)
   {
      delwin (radio->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vRADIO, radio);

   /* Finish cleaning up.						*/
   free (radio);
}

/*
 * This function erases the radio widget.
 */
void eraseCDKRadio (CDKRADIO *radio)
{
   werase (radio->win);
   wrefresh (radio->win);
   if (radio->shadow)
   {
      werase (radio->shadowwin);
      wrefresh (radio->shadowwin);
   }
}

/*
 * This sets certain attributes of the radio list.
 */
void setCDKRadio (CDKRADIO *radio, chtype highlight, chtype choiceChar, int Box)
{
   radio->highlight	= highlight;
   radio->choiceChar	= choiceChar;
   radio->box		= Box;
}
