#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/10 02:16:34 $
 * $Revision: 1.40 $
 */

/*
 * This function creates a scale widget.
 */
CDKSCALE *newCDKScale (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, char *label, chtype fieldattr, int fieldwidth, int start, int low, int high, int inc, int fastinc, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKSCALE *scale	= (CDKSCALE *) malloc (sizeof (CDKSCALE));
   int boxheight	= 3;
   int boxwidth		= fieldwidth + 2;
   int xpos		= xplace;
   int ypos		= yplace;
   int labelxoffset	= 0;
   int labelyoffset	= 0;
   int fieldxoffset	= 0;
   int fieldyoffset	= 0;
   char *plainlabel;
   int temp;

   /* Translate the label char *pointer to a chtype pointer.            */
   if (label != (char *)NULL)
   {
      scale->label	= char2Chtype (label, &scale->labellen, &temp);
      plainlabel	= chtype2Char (scale->label);
      boxwidth		= scale->labellen + fieldwidth + 3;
   }

   /* Adjust the offset values given the location of the label.		*/
   alignlabel (label, lplace, fieldwidth+2, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainlabel);

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Create the scale field.						*/
   scale->parent	= cdkscreen->window;
   scale->win		= subwin (cdkscreen->window, boxheight, boxwidth, ypos, xpos);
   scale->shadowwin	= (WINDOW *)NULL;
   scale->fieldwin	= subwin (cdkscreen->window, 1, fieldwidth+1, ypos+fieldyoffset, xpos+fieldxoffset);
   scale->labelwin	= subwin (cdkscreen->window, 1, strlen(label)+1, ypos+labelyoffset, xpos+labelxoffset);
   scale->fieldwidth	= fieldwidth;
   scale->fieldattr	= fieldattr;
   scale->current	= low;
   scale->low		= low;
   scale->high		= high;
   scale->current	= start;
   scale->inc		= inc;
   scale->fastinc	= fastinc;
   scale->box		= box;
   scale->shadow	= shadow;

   /* Is the main window NULL???					*/
   if (scale->win == (WINDOW *) NULL)
   {
      /* Clean up an used memory.					*/
      if (scale->fieldwin == (WINDOW *)NULL)
      {
         delwin (scale->fieldwin);
      }
      if (scale->labelwin == (WINDOW *)NULL)
      {
         delwin (scale->labelwin);
      }
      freeChtype (scale->label);
      free (scale);
      
      /* Return a NULL pointer.						*/
      return ( (CDKSCALE *) NULL );
   }

   /* Do we want a shadow???						*/
   if (scale->shadow)
   {
      scale->shadowwin	= subwin (cdkscreen->window, boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Make sure we can use cursor keys in the widget.			*/
   keypad (scale->fieldwin, TRUE);

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vSCALE, scale);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vSCALE, scale);

   /* Return the pointer.						*/
   return (scale);
}

/*
 * This allows the person to use the scale field.
 */
int activateCDKScale (CDKSCALE *scale)
{
   /* Declare local variables.						*/
   chtype	input;
   char		temp[10];

   /* Draw the widget.							*/
   drawCDKScale (scale, scale->box);

   /* Lets get the input.						*/
   for (;;)
   {
      /* Get the input from the user.					*/
      input = wgetch (scale->fieldwin);

      if ( ! checkCDKObjectBind(vSCALE, scale, input))
      {
         switch (input)
         {
            case KEY_LEFT : case 'd' : case '-' : case KEY_DOWN :
                 if (scale->current > scale->low)
                 {
                    scale->current -= scale->inc;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
         
            case KEY_RIGHT : case 'u' : case '+' : case KEY_UP :
                 if (scale->current < scale->high)
                 {
                    scale->current += scale->inc;
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case KEY_PPAGE : case 'U' : case '' :
                 if ( (scale->current + scale->fastinc) < scale->high)
                 {
                    scale->current += scale->fastinc;
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case KEY_NPAGE : case 'D' : case '' :
                 if ( (scale->current - scale->fastinc) > scale->low)
                 {
                    scale->current -= scale->fastinc;
                 }
                 else
                 {
                    Beep();
                 }
                 break;

            case KEY_HOME : case 'g' : case '0' :
                 scale->current = scale->low;
                 break;

            case KEY_END : case 'G' : case '$' :
                 scale->current = scale->high;
                 break;

            case KEY_RETURN : case CDK_NEXT :
                return (scale->current);
                break;

            case CDK_REFRESH :
                refreshCDKScreen (scale->screen);
                break;

            default :
                 break;
         }
      }

      /* Draw in the value in the field.				*/
      sprintf (temp, "%d", scale->current);
      werase (scale->fieldwin);
      writeCharAttrib (scale->fieldwin, scale->fieldwidth-strlen(temp)-1, 0, temp, scale->fieldattr, HORIZONTAL, 0, strlen(temp));
      wrefresh (scale->fieldwin);
   }
}

/*
 * This function draws the scale widget.
 */
void drawCDKScale (CDKSCALE *scale, boolean Box)
{
   /* Declare local variables.						*/
   char		temp[10];

   /* Keep the box information.						*/
   scale->box	= Box;

   /* Erase the widget from the screen.					*/
   eraseCDKScale (scale);

   /* Draw the shadow.							*/
   if (scale->shadow)
   {
      drawShadow (scale->shadowwin);
   }

   /* Box the widget if asked.						*/
   if (Box)
   {
      box (scale->win, ACS_VLINE, ACS_HLINE);
      wrefresh (scale->win);
   }

   /* Draw the label.							*/
   writeChtype (scale->labelwin, 0, 0, scale->label, HORIZONTAL, 0, scale->labellen);
   wrefresh (scale->labelwin);

   /* Draw the value in the field.					*/
   sprintf (temp, "%d", scale->current);
   writeCharAttrib (scale->fieldwin, scale->fieldwidth-strlen(temp)-1, 0, temp, scale->fieldattr, HORIZONTAL, 0, strlen(temp));
   wrefresh (scale->fieldwin);
}

/*
 * This function destroys the scale widget.
 */
void destroyCDKScale (CDKSCALE *scale)
{
   /* Erase the object.							*/
   eraseCDKScale (scale);

   /* Clean up the char pointers.					*/
   freeChtype (scale->label);

   /* Clean up the windows.						*/
   delwin (scale->fieldwin);
   delwin (scale->labelwin);
   delwin (scale->win);
   if (scale->shadow)
   {
      delwin (scale->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vSCALE, scale);

   /* Finish cleaning up.						*/
   free (scale);
}

/*
 * This function erases the scale widget from the screen.
 */
void eraseCDKScale (CDKSCALE *scale)
{
   werase (scale->win);
   wrefresh (scale->win);
   if (scale->shadow)
   {
      werase (scale->shadowwin);
      wrefresh (scale->shadowwin);
   }
}

/*
 * This function sets the low/high/current values of the scale widget.
 */
void setCDKScale (CDKSCALE *scale, int low, int high, int value, boolean box)
{
   /* Make sure the values aren't out of bounds.			*/
   if (low <= high)
   {
      scale->low	= low;
      scale->high	= high;
      scale->current	= value;
   }
   else if (low > high)
   {
      scale->low	= high;
      scale->high	= low;
      scale->current	= value;
   }
   scale->box		= box;
}
