#include "cdk.h"

/*
 * $Author: mikeg $
 * $Date: 1995/12/14 19:16:14 $
 * $Revision: 1.25 $
 */

/*
 * This function creates a new scrolling list widget.
 */
CDKSCROLL *newCDKScroll (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char *title, char **list, int listSize, boolean numbers, chtype highlight, boolean box, boolean shadow)
{
   /* Declare local variables.					*/
   CDKSCROLL*scroll		= (CDKSCROLL *)malloc (sizeof (CDKSCROLL));
   int boxWidth			= width;
   int boxHeight		= height;
   int xpos			= xplace;
   int ypos			= yplace;

   /* Translate the char * items to chtype *			*/
   if (title != (char *)NULL)
   {
      /* Title? Set the required variables.			*/
      scroll->title		= char2Chtype (title, &scroll->titlelen, &scroll->titlepos);
      scroll->titlepos		= justifyString (width, scroll->titlelen, scroll->titlepos);
      scroll->titleadj		= 2;
      scroll->listSize		= listSize;
      scroll->viewSize		= boxHeight - 3;
      scroll->lastItem		= listSize - 1;
      scroll->maxTopItem	= listSize - scroll->viewSize;
   }
   else
   {
      /* No title? Set the required variables.			*/
      scroll->title		= (chtype *)NULL;
      scroll->titlepos		= 0;
      scroll->titlelen		= 0;
      scroll->titleadj		= 1;
      scroll->listSize		= listSize;
      scroll->viewSize		= boxHeight - 2;
      scroll->lastItem		= listSize - 1;
      scroll->maxTopItem	= listSize - scroll->viewSize;
   }

   /* Is the view size smaller then the window???		*/
   if (listSize < (boxHeight-1-scroll->titleadj))
   {
      scroll->viewSize		= listSize;
      scroll->listSize		= listSize;
      scroll->lastItem		= listSize;
      scroll->maxTopItem	= -1;
   }

   /* Rejustify the x and y positions if we need to.   	 	*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the scrolling window					*/
   scroll->win		= newwin (boxHeight, boxWidth, ypos, xpos);

   /* Is the scrolling window NULL??				*/
   if (scroll->win == (WINDOW *)NULL)
   {
      /* Clean up any memory.					*/
      freeChtype (scroll->title);
      free(scroll);

      /* Return a NULL pointer.					*/
      return ( (CDKSCROLL *) NULL );
   }

   /* Turn the keypad on for the window.			*/
   keypad (scroll->win, TRUE);

   /* Set the rest of the variables				*/
   scroll->parent		= cdkscreen->window;
   scroll->shadowwin		= (WINDOW *)NULL;
   scroll->boxHeight		= boxHeight;
   scroll->boxWidth		= boxWidth;
   scroll->maxLeftChar		= 0;
   scroll->currentTop		= 0;
   scroll->currentItem		= 0;
   scroll->currentHigh		= 0;
   scroll->leftChar		= 0;
   scroll->highlight		= highlight;
   scroll->box			= box;
   scroll->shadow		= shadow;

   /* Create the scrolling list item list and needed variables.	*/
   createCDKScrollItemList (scroll, numbers, list, listSize);

   /* Do we need to create a shadow???				*/
   if (scroll->shadow)
   {
      scroll->shadowwin		= newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.					*/
   cleanCDKObjectBindings (vSCROLL, scroll);

   /* Register this baby.					*/
   registerCDKObject (cdkscreen, vSCROLL, scroll);

   /* Return the scrolling list					*/
   return (scroll);
}

/*
 * This actually does all the 'real' work of managing the scrolling list.
 */
int activateCDKScroll (CDKSCROLL *scroll)
{
   /* Declare some local vars					*/
   int x		= 0;
   char emptyString[100];
   chtype input;

   /* Create the empty string...				*/
   cleanChar (emptyString, scroll->boxWidth-2, ' ');

   /* Turn off the cursor.					*/
   curs_set (0);

   /* Draw the scrolling list					*/
   drawCDKScroll (scroll, scroll->box);

   /* Do this until KEY_RETURN is hit.				*/
   for (;;)
   {
      input = wgetch (scroll->win);

      /* Check for a predefined key binding.			*/
      if ( ! checkCDKObjectBind (vSCROLL, scroll, input) )
      {
         switch (input)
         {
            case KEY_UP :
                 if (scroll->currentHigh == 0)
                 {
                    if (scroll->currentTop != 0)
                    {
                       scroll->currentTop--;
                       scroll->currentItem--;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    scroll->currentItem--;
                    scroll->currentHigh--;
                 }
                 break;
   
            case KEY_DOWN : case KEY_TAB : 
                 if (scroll->currentHigh == scroll->viewSize - 1)
                 {
                    /* We need to scroll down one line.		*/
                    if (scroll->currentTop < scroll->maxTopItem)
                    {
                       scroll->currentTop++;
                       scroll->currentItem++;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    scroll->currentItem++;
                    scroll->currentHigh++;
                 }
                 break;
   
            case KEY_RIGHT :
                 if (scroll->leftChar >= scroll->maxLeftChar)
                 {
                    Beep();
                 }
                 else
                 {
                    scroll->leftChar ++;
                 }
                 break;
   
            case KEY_LEFT :
                 if (scroll->leftChar == 0)
                 {
                    Beep();
                 }
                 else
                 {
                    scroll->leftChar --;
                 }
                 break;
   
            case KEY_PPAGE : case '' :
                 if (scroll->currentTop > 0)
                 {
                    if ( scroll->currentTop >= (scroll->viewSize -1) )
                    {
                       scroll->currentTop	-= (scroll->viewSize - 1);
                       scroll->currentItem	-= (scroll->viewSize - 1);
                    }
                    else
                    {
                       scroll->currentTop 	= 0;
                       scroll->currentItem	= scroll->currentHigh;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_NPAGE : case '' :
                 if (scroll->currentTop < scroll->maxTopItem)
                 {
                    if ( (scroll->currentTop + scroll->viewSize - 1) <= scroll->maxTopItem)
                    {
                       scroll->currentTop	+= (scroll->viewSize - 1);
                       scroll->currentItem	+= (scroll->viewSize - 1);
                    }
                    else
                    {
                       scroll->currentTop	= scroll->maxTopItem;
                       scroll->currentItem	= scroll->lastItem;
                       scroll->currentHigh	= scroll->viewSize-1;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case 'g' : case '1' :
                 scroll->currentTop	= 0;
                 scroll->currentItem	= 0;
                 scroll->currentHigh	= 0;
                 break;
   
            case 'G' :
                 scroll->currentTop	= scroll->maxTopItem;
                 scroll->currentItem	= scroll->lastItem;
                 scroll->currentHigh	= scroll->viewSize-1;
                 break;
   
            case '$' :
                 scroll->leftChar	= scroll->maxLeftChar;
                 break;
   
            case '|' :
                 scroll->leftChar	= 0;
                 break;
   
            case CDK_NEXT :
                 curs_set (1);
                 return (-1);
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (scroll->screen);
                 break;
   
            case KEY_RETURN :
                 curs_set (1);
                 return (scroll->currentItem);
                 break;
   
            default :
                 break;
         }
      }
      
      /* Redraw the list						*/
      for (x=0; x < scroll->viewSize; x++)
      {
         writeChar (scroll->win, 1, scroll->titleadj+x, emptyString, HORIZONTAL, 0, strlen(emptyString));
         writeChtype (scroll->win,
		scroll->itempos[x+scroll->currentTop],
		x+scroll->titleadj,
		scroll->item[x+scroll->currentTop],
		HORIZONTAL, scroll->leftChar,
		scroll->itemlen[x+scroll->currentTop]);
      }

      /* Rehighlight the current menu item.			*/
      writeChtypeAttrib (scroll->win,
			scroll->itempos[scroll->currentItem],
			scroll->currentHigh+scroll->titleadj,
			scroll->item[scroll->currentItem],
			scroll->highlight,
			HORIZONTAL, scroll->leftChar,
			scroll->itemlen[scroll->currentItem]);
      wrefresh (scroll->win);
   }
}

/*
 * This function draws the scrolling list widget.
 */
void drawCDKScroll (CDKSCROLL *scroll, boolean Box)
{
   /* Declare local variables.					*/
   int x		= 0;

   /* Erase the old scrolling list.				*/
   eraseCDKScroll (scroll);

   /* Draw in the shadow if we need to.				*/
   if (scroll->shadow)
   {
      drawShadow (scroll->shadowwin);
   }

   /* Draw in the title if there is one.			*/
   if (scroll->title != (chtype *)NULL)
   {
      writeChtype (scroll->win, scroll->titlepos, 1, scroll->title, HORIZONTAL, 0, scroll->titlelen);
   }

   /* Draw in the scolling list items.				*/
   for (x=0; x < scroll->viewSize; x++)
   {
      writeChtype (scroll->win,
			scroll->itempos[x+scroll->currentTop],
			x+scroll->titleadj+scroll->currentTop,
			scroll->item[x+scroll->currentTop],
			HORIZONTAL, scroll->leftChar,
			scroll->itemlen[x+scroll->currentTop]);
   }
   writeChtypeAttrib (scroll->win,
			scroll->itempos[scroll->currentItem],
			scroll->currentHigh+scroll->titleadj,
			scroll->item[scroll->currentItem],
			scroll->highlight,
			HORIZONTAL, scroll->leftChar,
			scroll->itemlen[scroll->currentItem]);

   /* Box it if needed.						*/
   if (Box)
   {
      box (scroll->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (scroll->win);
}

/*
 * This function destroys
 */
void destroyCDKScroll (CDKSCROLL *scroll)
{
   /* Declare local variables.					*/
   int x	= 0;

   /* Erase the object.						*/
   eraseCDKScroll (scroll);

   /* Clean up the char pointers.				*/
   freeChtype (scroll->title);
   for (x=0; x < scroll->listSize; x++)
   {
      freeChtype (scroll->item[x]);
   }

   /* Clean up the windows.					*/
   delwin (scroll->win);
   if (scroll->shadow)
   {
      delwin (scroll->shadowwin);
   }

   /* Unregister this object.					*/
   unregisterCDKObject (vSCROLL, scroll);

   /* Finish cleaning up.					*/
   free (scroll);
}

/*
 * This function erases the scrolling list from the screen.
 */
void eraseCDKScroll (CDKSCROLL *scroll)
{
   werase (scroll->win);
   wrefresh (scroll->win);
   if (scroll->shadow)
   {
      werase (scroll->shadowwin);
      wrefresh (scroll->shadowwin);
   }
}

/*
 * This function creates massages the scrolling list information and 
 * sets up the needed variables for the scrolling list to work correctly.
 */
void createCDKScrollItemList (CDKSCROLL *scroll, boolean numbers, char **list, int listSize)
{
   /* Declare local variables.					*/
   int widestItem	= 0;
   int x		= 0;
   char temp[100];

   /* Create the items in the scrolling list.			*/
   if (numbers == NUMBERS)
   {
      for (x=0 ; x < listSize; x++)
      {
         sprintf (temp, "%4d. %s", x+1, list[x]);
         scroll->item[x]	= char2Chtype (temp, &scroll->itemlen[x], &scroll->itempos[x]);
         scroll->itempos[x]	= justifyString (scroll->boxWidth, scroll->itemlen[x], scroll->itempos[x]);
         widestItem		= MAX (scroll->itemlen[x], widestItem);
      }
   }
   else
   {
      for (x=0 ; x < listSize; x++)
      {
         scroll->item[x]	= char2Chtype (list[x], &scroll->itemlen[x], &scroll->itempos[x]);
         scroll->itempos[x]	= justifyString (scroll->boxWidth, scroll->itemlen[x], scroll->itempos[x]);
         widestItem		= MAX (scroll->itemlen[x], widestItem);
      }
   }

   /* Determine how many characters we can shift to the right	*/
   /* before all the items have been scrolled off the screen.	*/
   if (scroll->boxWidth > widestItem)
   {
      scroll->maxLeftChar = 0;
   }
   else
   {
      scroll->maxLeftChar = widestItem-(scroll->boxWidth-2);
   }
}

/*
 * This sets certain attributes of the scrolling list.
 */
void setCDKScroll (CDKSCROLL *scroll, char **list, int listSize, boolean numbers, chtype highlight, boolean box)
{
   /* Declare some vars.					*/
   char emptyString[100];
   int x = 0;

   /* Clean out the old list.					*/
   for (x=0; x < scroll->listSize; x++)
   {
      freeChtype (scroll->item[x]);
      scroll->itemlen[x] = 0;
      scroll->itempos[x] = 0;
   }

   /* Clean up the display.					*/
   cleanChar (emptyString, scroll->boxWidth-1, ' ');
   for (x=0; x < scroll->viewSize ; x++ )
   {
      writeChar (scroll->win, 1, scroll->titleadj+x, emptyString, HORIZONTAL, 0, strlen(emptyString));
   }

   /* Readjust all of the variables ...				*/
   if (scroll->title != (chtype *)NULL)
   {
      scroll->listSize		= listSize;
      scroll->viewSize		= scroll->boxHeight - 3;
      scroll->lastItem		= listSize - 1;
      scroll->maxTopItem	= listSize - scroll->viewSize;
   }
   else
   {
      scroll->listSize		= listSize;
      scroll->viewSize		= scroll->boxHeight - 2;
      scroll->lastItem		= listSize - 1;
      scroll->maxTopItem	= listSize - scroll->viewSize;
   }

   /* Is the view size smaller then the window???               */
   if (listSize < (scroll->boxHeight-1-scroll->titleadj))
   {
      scroll->viewSize		= listSize;
      scroll->listSize		= listSize;
      scroll->lastItem		= listSize;
      scroll->maxTopItem	= -1;
   }

   /* Set some vars.						*/
   scroll->highlight		= highlight;
   scroll->box			= box;
   scroll->currentTop		= 0;
   scroll->currentItem		= 0;
   scroll->currentHigh		= 0;
   scroll->leftChar		= 0;

   /* Set up the new list.					*/
   createCDKScrollItemList (scroll, numbers, list, listSize);
}
