#include "cdk.h"

/*
 * $Author: mikeg $
 * $Date: 1995/12/14 19:49:11 $
 * $Revision: 1.36 $
 */

/*
 * This function creates a selection widget.
 */
CDKSELECTION *newCDKSelection (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char *title, char **list, int listSize, char **choices, int choiceCount, chtype highlight, boolean box, boolean shadow)
{
   /* Declare local variables.					*/
   CDKSELECTION *selection	= (CDKSELECTION *)malloc (sizeof (CDKSELECTION));
   int widestitem		= -1;
   int boxWidth			= width;
   int boxHeight		= height;
   int xpos			= xplace;
   int ypos			= yplace;
   int x			= 0;
   int junk;

   /* Translate the char * title to a chtype *			*/
   if (title != (char *)NULL)
   {
      /* Title? Set the required variables.			*/
      selection->title		= char2Chtype (title, &selection->titlelen, &selection->titlepos);
      selection->titlepos	= justifyString (width, selection->titlelen, selection->titlepos);
      selection->titleadj	= 2;
      selection->listSize	= listSize;
      selection->viewSize	= boxHeight - 3;
      selection->lastItem	= listSize - 1;
      selection->maxTopItem	= listSize - selection->viewSize;
   }
   else
   {
      /* No title? Set the required variables.			*/
      selection->title		= (chtype *)NULL;
      selection->titlepos	= 0;
      selection->titlelen	= 0;
      selection->titleadj	= 1;
      selection->listSize	= listSize;
      selection->viewSize	= boxHeight - 2;
      selection->lastItem	= listSize - 1;
      selection->maxTopItem	= listSize - selection->viewSize;
   }
   selection->maxchoicelen	= -1;

   /* Is the view size smaller then the window???		*/
   if (listSize < (boxHeight-1-selection->titleadj))
   {
      selection->viewSize	= listSize;
      selection->listSize	= listSize;
      selection->lastItem	= listSize;
      selection->maxTopItem	= -1;
   }

   /* Rejustify the x and y positions if we need to.	    	*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the selection window					*/
   selection->win = newwin (boxHeight, boxWidth, ypos, xpos);

   /* Is the window NULL??					*/
   if (selection->win == (WINDOW *)NULL)
   {
      /* Clean up any memory used.				*/
      freeChtype (selection->title);
      free (selection);

      /* Return a NULL pointer.					*/
      return ( (CDKSELECTION *)NULL );
   }

   /* Turn the keypad on for this window.			*/
   keypad (selection->win, TRUE);

   /* Set the rest of the variables				*/
   selection->parent		= cdkscreen->window;
   selection->boxHeight		= boxHeight;
   selection->boxWidth		= boxWidth;
   selection->currentTop	= 0;
   selection->currentHigh	= 0;
   selection->currentItem	= 0;
   selection->maxLeftChar	= 0;
   selection->leftChar		= 0;
   selection->highlight		= highlight;
   selection->choiceCount	= choiceCount;
   selection->box		= box;
   selection->shadow		= shadow;

   /* Each choice has to be converted from char * to chtype *		*/
   for (x=0; x < choiceCount; x++)
   {
      selection->choice[x]	= char2Chtype (choices[x], &selection->choicelen[x], &junk);
      selection->maxchoicelen	= MAX(selection->maxchoicelen, selection->choicelen[x]);
   }

   /* Each item in the needs to be converted to chtype *                */
   for (x=0; x < listSize; x++)
   {
      selection->item[x]	= char2Chtype (list[x], &selection->itemlen[x], &selection->itempos[x]);
      selection->itempos[x]	= justifyString ( (width-selection->maxchoicelen), selection->itemlen[x], selection->itempos[x])+selection->maxchoicelen;
      selection->selections[x]	= 0;
      widestitem		= MAX (widestitem, selection->itemlen[x]);
   }

   /* Determine how many characters we can shift to the right   */
   /* before all the items have been scrolled off the screen.   */
   if (selection->boxWidth > widestitem)
   {
      selection->maxLeftChar = 0;
   }
   else
   {
      selection->maxLeftChar = widestitem - (selection->boxWidth-2-selection->maxchoicelen);
   }

   /* Do we need to create a shadow.				*/
   if (selection->shadow)
   {
      selection->shadowwin = newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.					*/
   cleanCDKObjectBindings (vSELECTION, selection);

   /* Register this baby.					*/
   registerCDKObject (cdkscreen, vSELECTION, selection);

   /* Return the selection list					*/
   return (selection);
}

/*
 * This actually manages the selection widget...
 */
void activateCDKSelection (CDKSELECTION *selection)
{
   /* Declare local variables.					*/
   char emptyString[1000];
   int x		= 0;
   chtype input;

   /* Empty out the string.					*/
   cleanChar (emptyString, selection->boxWidth, ' ');

   /* Draw the selection list					*/
   drawCDKSelection (selection, selection->box);
   wrefresh (selection->win);

   /* Turn off the cursor.					*/
   curs_set (0);

   /* Do this until KEY_RETURN is hit.				*/
   for (;;)
   {
      input = wgetch (selection->win);
      if ( ! checkCDKObjectBind (vSELECTION, selection, input) )
      {
         switch (input)
         {
            case KEY_UP :
                 if (selection->currentHigh == 0)
                 {
                    if (selection->currentTop != 0)
                    {
                       selection->currentTop--;
                       selection->currentItem--;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    selection->currentItem--;
                    selection->currentHigh--;
                 }
                 break;
   
            case KEY_DOWN : case KEY_TAB : 
                 if (selection->currentHigh == selection->viewSize - 1)
                 {
                    /* We need to scroll down one line.		*/
                    if (selection->currentTop < selection->maxTopItem)
                    {
                       selection->currentTop++;
                       selection->currentItem++;
                    }
                    else
                    {
                       Beep();
                    }
                 }
                 else
                 {
                    selection->currentItem++;
                    selection->currentHigh++;
                 }
                 break;
   
            case KEY_RIGHT :
                 if (selection->leftChar >= selection->maxLeftChar)
                 {
                    Beep();
                 }
                 else
                 {
                    selection->leftChar ++;
                 }
                 break;
   
            case KEY_LEFT :
                 if (selection->leftChar == 0)
                 {
                    Beep();
                 }
                 else
                 {
                    selection->leftChar --;
                 }
                 break;
   
            case KEY_PPAGE : case '' :
                 if (selection->currentTop > 0)
                 {
                    if ( selection->currentTop >= (selection->viewSize -1) )
                    {
                       selection->currentTop	-= (selection->viewSize - 1);
                       selection->currentItem	-= (selection->viewSize - 1);
                    }
                    else
                    {
                       selection->currentTop 	= 0;
                       selection->currentItem	= selection->currentHigh;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_NPAGE : case '' :
                 if (selection->currentTop < selection->maxTopItem)
                 {
                    if ( (selection->currentTop + selection->viewSize - 1) <= selection->maxTopItem)
                    {
                       selection->currentTop	+= (selection->viewSize - 1);
                       selection->currentItem	+= (selection->viewSize - 1);
                    }
                    else
                    {
                       selection->currentTop 	= selection->maxTopItem;
                       selection->currentItem	= selection->lastItem;
                       selection->currentHigh	= selection->viewSize-1;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case 'g' : case '1' :
                 selection->currentTop	= 0;
                 selection->currentItem	= 0;
                 selection->currentHigh	= 0;
                 break;
   
            case 'G' :
                 selection->currentTop 	= selection->maxTopItem;
                 selection->currentItem	= selection->lastItem;
                 selection->currentHigh	= selection->viewSize-1;
                 break;
   
            case '$' :
                 selection->leftChar	= selection->maxLeftChar;
                 break;
   
            case '|' :
                 selection->leftChar	= 0;
                 break;
   
            case SPACE :
                 if (selection->selections[selection->currentItem] == selection->choiceCount-1)
                 {
                    selection->selections[selection->currentItem]	= 0;
                 }
                 else
                 {
                    selection->selections[selection->currentItem]++;
                 }
                 break;
   
            case KEY_RETURN : case CDK_NEXT :
                 curs_set (1);
                 return;
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (selection->screen);
                 break;
   
            default :
                 break;
         }
      }
      
      /* Redraw the list						*/
      for (x=0; x < selection->viewSize; x++)
      {
         writeChar (selection->win, 1,
			selection->titleadj+x,
 			emptyString,
			HORIZONTAL, 0,
			strlen(emptyString));

         /* Draw in the selection item.				*/
         writeChtype (selection->win,
		selection->itempos[x+selection->currentTop],
		x+selection->titleadj,
		selection->item[x+selection->currentTop],
		HORIZONTAL,
		selection->leftChar,
		selection->itemlen[x+selection->currentTop]);

         /* Draw in the choice value.				*/
         writeChtype (selection->win,
		1,
		x+selection->titleadj,
		selection->choice[selection->selections[x+selection->currentTop]],
		HORIZONTAL,
		0,
		selection->choicelen[selection->selections[x+selection->currentTop]]);
      }

      /* Draw in the current highlight.				*/
      writeChtypeAttrib (selection->win,
                        selection->itempos[selection->currentItem],
                        selection->currentHigh+selection->titleadj,
                        selection->item[selection->currentItem],
                        selection->highlight,
                        HORIZONTAL, selection->leftChar,
                        selection->itemlen[selection->currentItem]);
      wrefresh (selection->win);
   }
}

/*
 * This function draws the selection list.
 */
void drawCDKSelection (CDKSELECTION *selection, boolean Box)
{
   /* Erase the old object...					*/
   eraseCDKSelection (selection);

   /* Draw in the shadow if we need to.				*/
   if (selection->shadow)
   {
      drawShadow (selection->shadowwin);
   }

   /* Draw in the title if there is one.			*/
   if (selection->title != (chtype *)NULL)
   {
      writeChtype (selection->win, selection->titlepos, 1, selection->title, HORIZONTAL, 0, selection->titlelen);
   }

   /* Redraw the list						*/
   drawCDKSelectionList (selection);

   /* Box it if needed						*/
   if (Box)
   {
      box (selection->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (selection->win);
}

/*
 * This function destroys the selection list.
 */
void destroyCDKSelection (CDKSELECTION *selection)
{
   /* Declare local variables.					*/
   int x	= 0;

   /* Erase the object.						*/
   eraseCDKSelection (selection);

   /* Clean up the char pointers.				*/
   freeChtype (selection->title);
   for (x=0; x < selection->listSize; x++)
   {
      freeChtype (selection->item[x]);
   }
   for (x=0; x < selection->choiceCount; x++)
   {
      freeChtype (selection->choice[x]);
   }

   /* Clean up the windows.					*/
   delwin (selection->win);
   if (selection->shadow)
   {
      delwin (selection->shadowwin);
   }

   /* Unregister this object.					*/
   unregisterCDKObject (vSELECTION, selection);

   /* Finish cleaning up.					*/
   free (selection);
}

/*
 * This function erases the selection list from the screen.
 */
void eraseCDKSelection (CDKSELECTION *selection)
{
   werase (selection->win);
   wrefresh (selection->win);
   if (selection->shadow)
   {
      werase (selection->shadowwin);
      wrefresh (selection->shadowwin);
   }
}

/*
 * This function sets a couple of the selection list attributes.
 */
void setCDKSelection (CDKSELECTION *selection, chtype highlight, int *defChoices, boolean box)
{
   /* Declare local variables.					*/
   int x	= 0;

   /* Set the simple ones first....				*/
   selection->highlight	= highlight;
   selection->box	= box;

   /* OK, on to the more difficult...				*/
   for (x=0; x < selection->choiceCount; x++)
   {
      selection->selections[x]	= defChoices[x];
   }
}

/*
 * This function draws the selection list window.
 */
void drawCDKSelectionList (CDKSELECTION *selection)
{
   /* Declare local variables.					*/
   int x	= 0;

   /* Redraw the list...					*/
   for (x=0; x < selection->viewSize; x++)
   {
      /* Draw in the selection item.				*/
      writeChtype (selection->win,
		selection->itempos[x+selection->currentTop],
		x+selection->titleadj,
		selection->item[x+selection->currentTop],
		HORIZONTAL,
		selection->leftChar,
		selection->itemlen[x+selection->currentTop]);

      /* Draw in the choice value.				*/
      writeChtype (selection->win,
		1,
		x+selection->titleadj,
		selection->choice[selection->selections[x+selection->currentTop]],
		HORIZONTAL,
		0,
		selection->choicelen[selection->selections[x+selection->currentTop]]);
   }

   /* Draw in the current highlight.				*/
   writeChtypeAttrib (selection->win,
                        selection->itempos[selection->currentItem],
                        selection->currentHigh+selection->titleadj,
                        selection->item[selection->currentItem],
                        selection->highlight,
                        HORIZONTAL, selection->leftChar,
                        selection->itemlen[selection->currentItem]);
}
