#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/23 22:55:50 $
 * $Revision: 1.22 $
 */

/*
 * This function creates a scrolling window widget.
 */
CDKSWINDOW *newCDKSwindow (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char *title, int saveLines, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKSWINDOW	*swindow	= (CDKSWINDOW *)malloc (sizeof (CDKSWINDOW));
   int boxWidth			= width;
   int boxHeight		= height+2;
   int xpos			= xplace;
   int ypos			= yplace;

   /* Translate the title char * to chtype *				*/
   if (title != (char *)NULL)
   {
      swindow->title		= char2Chtype (title, &swindow->titlelen, &swindow->titlepos);
      swindow->titlepos		= justifyString (boxWidth, swindow->titlelen, swindow->titlepos);
      swindow->titleadj		= 2;
   }
   else
   {
      swindow->title		= (chtype *)NULL;
      swindow->titlepos		= 0;
      swindow->titleadj		= 1;
   }

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the scrolling window						*/
   swindow->win			= newwin (boxHeight, boxWidth, ypos, xpos);

   /* Is the window NULL??						*/
   if (swindow->win == (WINDOW *)NULL)
   {
      /* Clean up.							*/
      freeChtype (swindow->title);
      free(swindow);
 
      /* Return a NULL pointer.						*/
      return ( (CDKSWINDOW *)NULL );
   }

   /* Turn on the keypad for this window.				*/
   keypad (swindow->win, TRUE);

   /* Set the rest of the variables					*/
   swindow->parent		= cdkscreen->window;
   swindow->shadowwin		= (WINDOW *)NULL;
   swindow->boxHeight		= boxHeight;
   swindow->boxWidth		= boxWidth;
   swindow->viewSize		= height-1;
   swindow->currentTop		= 0;
   swindow->maxTopLine		= 0;
   swindow->leftChar		= 0;
   swindow->maxLeftChar		= 0;
   swindow->itemCount		= 0;
   swindow->widestLine		= -1;
   swindow->saveLines		= saveLines;
   swindow->box			= box;
   swindow->shadow		= shadow;

   /* Do we need to create a shadow???					*/
   if (swindow->shadow)
   {
      swindow->shadowwin	= newwin (boxHeight, boxWidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vSWINDOW, swindow);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vSWINDOW, swindow);

   /* Return the scrolling window					*/
   return (swindow);
}

/*
 * This sets all the lines inside the scrolling window.
 */
void setCDKSwindow (CDKSWINDOW *swindow, char **info, int lines, boolean box)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* First lets clean all the lines in the window.			*/
   cleanCDKSwindow(swindow);

   /* Now lets set all the lines inside the window.			*/
   for (x=0; x < lines; x++)
   {
      swindow->info[x]		= char2Chtype (info[x], &swindow->infolen[x], &swindow->infopos[x]);
      swindow->infopos[x]	= justifyString (swindow->boxWidth, swindow->infolen[x], swindow->infopos[x]);
      swindow->widestLine	= MAX (swindow->widestLine, swindow->infolen[x]);
   }
   swindow->itemCount	= lines;
   swindow->maxLeftChar	= swindow->widestLine - (swindow->boxWidth - 2);
   swindow->box		= box;
}

/*
 * This adds a line to the scrolling window.
 */
void addCDKSwindow  (CDKSWINDOW *swindow, char *info, int insertPos)
{
   /* Declare variables.						*/
   int x = 0;

   /* Are we adding to the top or bottom?				*/
   if (insertPos == TOP)
   {
      /* We need to 'bump' everything down one line...			*/
      for (x=swindow->itemCount; x > 0; x--)
      {
         /* Clean up the old memory.					*/
         freeChtype (swindow->info[x]);

         /* Copy in the new row.					*/
         swindow->info[x] = copyChtype (swindow->info[x-1]);
         swindow->infopos[x] = swindow->infopos[x-1];
         swindow->infolen[x] = swindow->infolen[x-1];
      }

      /* Free up the memry from the top row.				*/
      freeChtype (swindow->info[0]);

      /* Add it into the scrolling window.				*/
      swindow->info[0] = char2Chtype (info, &swindow->infolen[0], &swindow->infopos[0]);
      swindow->infopos[0] = justifyString (swindow->boxWidth, swindow->infolen[0], swindow->infopos[0]);

      /* Set some variables.						*/
      swindow->currentTop = 0;
      swindow->itemCount++;

      /* Set the maximum top line.					*/
      swindow->maxTopLine = swindow->itemCount - swindow->viewSize;
      swindow->maxTopLine = (swindow->maxTopLine < 0 ? 0 : swindow->maxTopLine);

      /* Lets determine the widest line and the maximum leftmost character. */
      swindow->widestLine = MAX (swindow->widestLine, swindow->infolen[0]);
      swindow->maxLeftChar = swindow->widestLine - (swindow->boxWidth - 2);
   }
   else
   { 
      /* Add to the bottom.						*/
      swindow->info[swindow->itemCount]	 = char2Chtype (info, &swindow->infolen[swindow->itemCount], &swindow->infopos[swindow->itemCount]);
      swindow->infopos[swindow->itemCount] = justifyString (swindow->boxWidth, swindow->infolen[swindow->itemCount], swindow->infopos[swindow->itemCount]);

      /* Lets determine the widest line and the maximum leftmost character. */
      swindow->widestLine = MAX (swindow->widestLine, swindow->infolen[swindow->itemCount]);
      swindow->maxLeftChar = swindow->widestLine - (swindow->boxWidth - 2);

      /* Increment the item count and zero out the next row.		*/
      swindow->itemCount++;
      swindow->info[swindow->itemCount] = '\0';

      /* Set the maximum top line.					*/
      swindow->maxTopLine = swindow->itemCount - swindow->viewSize;
      swindow->maxTopLine = (swindow->maxTopLine < 0 ? 0 : swindow->maxTopLine);

      /* Do we have to scroll???					*/
      if (swindow->itemCount > swindow->viewSize)
      {
         swindow->currentTop++;
      }
   }

   /* Lets redraw ourselves....						*/
   drawCDKSwindow (swindow, swindow->box);
}

/*
 * This removes all the lines inside the scrolling window.
 */
void cleanCDKSwindow (CDKSWINDOW *swindow)
{
   /* Declare local variables.						*/
   int x;

   /* Clean up the memory used ...					*/
   for (x=0; x < swindow->itemCount; x++)
   {
      freeChtype (swindow->info[x]);
   }

   /* Reset some variables.						*/
   swindow->itemCount	= 0;
   swindow->maxLeftChar	= 0;
   swindow->widestLine	= 0;
   swindow->currentTop	= 0;
   swindow->maxTopLine	= 0;
}

/*
 * This trims lines from the scrolling window.
 */
void trimCDKSwindow (CDKSWINDOW *swindow, int start, int finish)
{
   int begin = 0;
   int end = (finish <= swindow->itemCount ? finish : swindow->itemCount);
   int x;

   /* Make sure the start is lower than the end.			*/
   if (start > finish)
   {
      return;
   }

   /* Set the start value.						*/
   begin = (start >= 0? start : 0);

   /* Start nuking elements from the window.				*/
   for (x=begin; x <= end; x++)
   {
      freeChtype (swindow->info[x]);
   }
}

/*
 * This allows the user to play inside the scolling window.
 */
void activateCDKSwindow (CDKSWINDOW *swindow)
{
   /* Declare local variables.						*/
   char emptyString[200];
   chtype input;
   int x;

   /* Create the empty string...					*/
   cleanChar (emptyString, swindow->boxWidth, ' ');

   /* Draw the window....						*/
   drawCDKSwindow (swindow, swindow->box);

   /* Do this until KEY_RETURN is hit.					*/
   for (;;)
   {
      input = wgetch (swindow->win);
      if ( ! checkCDKObjectBind (vSWINDOW, swindow, input) )
      {
         switch (input)
         {
            case KEY_UP :
                 if (swindow->currentTop > 0)
                 {
                    swindow->currentTop--;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_DOWN :
                 if (swindow->currentTop >= 0 && swindow->currentTop < swindow->maxTopLine)
                 {
                    swindow->currentTop++;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_RIGHT :
                 if (swindow->leftChar < swindow->maxLeftChar)
                 {
                    swindow->leftChar ++;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_LEFT :
                 if (swindow->leftChar > 0)
                 {
                    swindow->leftChar --;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_PPAGE : case '' : case 'b' : case 'B' :
                 if (swindow->currentTop != 0)
                 {
                    if ( swindow->currentTop >= swindow->viewSize )
                    {
                       swindow->currentTop	= swindow->currentTop - (swindow->viewSize - 1);
                    }
                    else
                    {
                       swindow->currentTop 	= 0;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_NPAGE : case '' : case ' ' : case 'f' : case 'F' :
                 if ( swindow->currentTop != swindow->maxTopLine)
                 {
                    if ( (swindow->currentTop + swindow->viewSize) < swindow->maxTopLine )
                    {
                       swindow->currentTop	= swindow->currentTop + (swindow->viewSize - 1);
                    }
                    else
                    {
                       swindow->currentTop	= swindow->maxTopLine;
                    }
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_HOME : case '|' :
                 swindow->leftChar = 0;
                 break;
   
            case KEY_END : case '$' :
                 swindow->leftChar = swindow->maxLeftChar + 1;
                 break;
   
            case 'g' : case '1' :
                 swindow->currentTop = 0;
                 break;
   
            case 'G' : 
                 swindow->currentTop = swindow->maxTopLine;
                 break;
   
            case 'L' :
                 x = (int) ( (swindow->itemCount + swindow->currentTop) / 2);
                 if ( x < swindow->maxTopLine)
                 {
                    swindow->currentTop = x;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case 'l' :
                 x = (int) (swindow->currentTop / 2);
                 if ( x >= 0 )
                 {
                    swindow->currentTop = x;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_RETURN : case KEY_TAB : case CDK_NEXT :
                 return;
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (swindow->screen);
                 break;
   
            default :
                 break;
         }
      }

      /* Redraw the list						*/
      for (x=0; x < swindow->viewSize; x++)
      {
         /* Empty the line.						*/
         writeChar (swindow->win, 1, swindow->titleadj+x, emptyString, HORIZONTAL, 1, swindow->boxWidth-1);

         /* Draw in the line.						*/
         if (swindow->info[x] != (chtype *)NULL)
         {
            writeChtype (swindow->win,
			swindow->infopos[x+swindow->currentTop],
			swindow->titleadj + x,
			swindow->info[x+swindow->currentTop],
			HORIZONTAL,
			swindow->leftChar,
			swindow->infolen[x+swindow->currentTop]);
         }
      }
      wrefresh (swindow->win);
   }
}

/*
 * This function draws the swindow window widget.
 */
void drawCDKSwindow (CDKSWINDOW *swindow, boolean Box)
{
   /* Declare local variables.						*/
   int temp		= 0;
   int x		= 0;

   /* Keep the box information.						*/
   swindow->box	= Box;

   /* Erase the old object...						*/
   eraseCDKSwindow (swindow);

   /* Do we need to draw in the shadow.					*/
   if (swindow->shadow)
   {
      drawShadow (swindow->shadowwin);
   }

   /* Draw in the title if there is one					*/
   if (swindow->title != (chtype *)NULL)
   {
      writeChtype (swindow->win, swindow->titlepos, 1, swindow->title, HORIZONTAL, 0, swindow->titlelen);
   }

   /* Draw in each item until we can't anymore.				*/
   temp = (swindow->itemCount < swindow->viewSize ? swindow->itemCount : swindow->viewSize);
   for (x=0; x < temp; x++)
   {
      writeChtype (swindow->win,
			swindow->infopos[x+swindow->currentTop],
			swindow->titleadj+x,
			swindow->info[x+swindow->currentTop],
			HORIZONTAL, 
			swindow->leftChar,
			swindow->infolen[x+swindow->currentTop]);
   }

   /* Box it if needed							*/
   if (Box)
   {
      box (swindow->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (swindow->win);
}

/*
 * This function destroys the scrolling window widget.
 */
void destroyCDKSwindow (CDKSWINDOW *swindow)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the object.							*/
   eraseCDKSwindow (swindow);

   /* Clear out the character pointers.					*/
   for (x=0; x < swindow->itemCount-1; x++)
   {
      freeChtype (swindow->info[x]);
   }
   freeChtype (swindow->title);

   /* Delete the windows.						*/
   delwin (swindow->win);
   if (swindow->shadow)
   {
      delwin (swindow->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vSWINDOW, swindow);

   /* Finish cleaning up.						*/
   free (swindow);
}

/*
 * This function erases the scrolling window widget.
 */
void eraseCDKSwindow (CDKSWINDOW *swindow)
{
   werase (swindow->win);
   wrefresh (swindow->win);
   if (swindow->shadow)
   {
      werase (swindow->shadowwin);
      wrefresh (swindow->shadowwin);
   }
}

/*
 * This function returns the current information in the swindow.
 */
void getCDKSwindow (CDKSWINDOW *swindow, char **info)
{
   int x;
   for (x=0; x < swindow->itemCount; x++)
   {
      info[x] = chtype2Char (swindow->info[x]);
   }
}
