#include "cdk.h"

/*
 * $Author: mikeg $
 * $Date: 1995/12/21 21:17:18 $
 * $Revision: 1.25 $
 */

/*
 * This creates a template widget.
 */
CDKTEMPLATE *newCDKTemplate (CDKSCREEN *cdkscreen, int xplace, int yplace, int lplace, char *label, char *plate, char *overlay, TEMPLATECB callBackFn, boolean box, boolean shadow)
{
   /* Set up some variables						*/
   CDKTEMPLATE *template	= (CDKTEMPLATE *)malloc (sizeof (CDKTEMPLATE));
   int boxwidth			= 0;
   int boxheight		= 3;
   int xpos			= xplace;
   int ypos			= yplace;
   int labelxoffset		= 0;
   int labelyoffset		= 0;
   int fieldxoffset		= 0;
   int fieldyoffset		= 0;
   int fieldwidth		= strlen (plate) + 2;
   int junk			= 0;
   char *plainLabel;
   
   /* Translate the char * label to a chtype *				*/
   template->label	= char2Chtype (label, &template->labellen, &junk);
   plainLabel		= chtype2Char (template->label);

   /* Translate the char * overlay to a chtype *			*/
   if (overlay != (char *)NULL)
   {
      template->overlay		= char2Chtype (overlay, &template->overlaylen, &junk);
      template->fieldattr	= template->overlay[0] & A_ATTRIBUTES;
   }
   else
   {
      template->overlay		= (chtype *)NULL;
      template->overlaylen	= 0;
      template->fieldattr	= A_NORMAL;
   }
   
   /* Align the x and y offsets for the label.				*/
   alignlabel (plainLabel, lplace, fieldwidth, &boxwidth, &boxheight, &labelxoffset, &labelyoffset, &fieldxoffset, &fieldyoffset);
   freeChar (plainLabel);

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxwidth, boxheight);

   /* Make the template window						*/
   template->win		= newwin (boxheight, boxwidth, ypos, xpos);
   
   /* Is the template window NULL??					*/
   if (template->win == (WINDOW *)NULL)
   {
      /* Clean up any used memory.					*/
      freeChtype (template->label);
      freeChtype (template->overlay);
      free (template);

      /* Return a NULL pointer.						*/
      return ( (CDKTEMPLATE *)NULL );
   }

   /* Make the field window.						*/
   template->field		= subwin (cdkscreen->window, 1, fieldwidth, ypos+fieldyoffset, xpos+fieldxoffset);
   template->labelwin		= subwin (cdkscreen->window, 1, strlen(label)+1, ypos+labelyoffset, xpos+labelxoffset);
   keypad (template->field, TRUE);

   /* Set up the info field.						*/
   template->info		= (char *)malloc (sizeof(char) * ((int)strlen(plate)+4));
   cleanChar (template->info, strlen(plate)+4, '\0');

   /* Set up the rest of the structure 					*/
   template->parent		= cdkscreen->window;
   template->shadowwin		= (WINDOW *)NULL;
   template->plate		= copyChar (plate);
   template->fieldwidth		= fieldwidth;
   template->platePos		= 0;
   template->screenPos		= 0;
   template->infoPos		= 0;
   template->box		= box;
   template->shadow		= shadow;
   template->callbackfn		= (void *)callBackFn;

   /* Do we need to create a shaodw???					*/
   if (shadow)
   {
      template->shadowwin	= newwin (boxheight, boxwidth, ypos+1, xpos+1);
   }

   /* Clean the key bindings.						*/
   cleanCDKObjectBindings (vTEMPLATE, template);

   /* Register this baby.               				*/
   registerCDKObject (cdkscreen, vTEMPLATE, template);

   /* Return the pointer to the structure				*/
   return (template);
}

/*
 * This actually manages the template widget...
 */
char *activateCDKTemplate (CDKTEMPLATE *template)
{
   /* Declare local variables.						*/
   int charcount	= 0;
   int filler		= 0;
   int infolen		= 0;
   chtype input;

   /* Draw the object.							*/
   drawCDKTemplate (template, template->box);

   /* If there is info already, adjust the variables...			*/
   if (template->info != (char *)NULL)
   {
      infolen		= strlen (template->info);
      while (charcount++ < infolen)
      {
         if (
		template->plate[charcount] != '#' &&
		template->plate[charcount] != 'A' &&
		template->plate[charcount] != 'C' &&
		template->plate[charcount] != 'c' &&
		template->plate[charcount] != 'M' &&
		template->plate[charcount] != 'X' &&
		template->plate[charcount] != 'x')
         {
            filler++;
         }
      }

      /* Now set the variables.						*/
      template->screenPos	= filler + infolen;
      template->platePos	= filler + infolen;
      template->infoPos		= infolen;
   }
   else
   {
      /* Now set the variables.						*/
      template->screenPos	= 0;
      template->platePos	= 0;
      template->infoPos		= 0;
   }
   wmove (template->field, 0, template->screenPos);
   wrefresh (template->field);

   /* Do this forever...						*/
   for (;;)
   {
      /* Get the input from the user.           			*/
      input	= wgetch (template->field);

      /* Check a predefined binding...          			*/
      if ( ! checkCDKObjectBind (vTEMPLATE, template, input) )
      {
         switch (input)
         {
            case KEY_LEFT : case KEY_RIGHT : case KEY_UP : case KEY_DOWN :
               Beep();
            break;

            case KEY_RETURN : case KEY_TAB : case CDK_NEXT :
                 return (template->info);
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (template->screen);
                 break;
            
            default :
                 ((TEMPLATECB)template->callbackfn)(template, input);
                 break;
         }
      }
   }
}

/*
 * This is the standard callback proc for the template.
 */
void CDKTemplateCallBack (CDKTEMPLATE *template, chtype input)
{
   /* Check the character input.					*/
   if (input == DELETE || input == KEY_BACKSPACE)
   {
      /* Make sure we don't go out of bounds.				*/
      if (template->platePos == 0)
      {
         Beep();
         return;
      }

      /* Remove it and redisplay....					*/
      template->info[--template->infoPos] = '\0';
      template->platePos--;
      template->screenPos--;

      /* Move the cursor						*/
      while (	template->plate[template->platePos] != '#' &&
		template->plate[template->platePos] != 'A' &&
		template->plate[template->platePos] != 'C' &&
		template->plate[template->platePos] != 'c' &&
		template->plate[template->platePos] != 'M' &&
		template->plate[template->platePos] != 'X' &&
		template->plate[template->platePos] != 'x' &&
		template->platePos > 0)
      { 
         template->platePos--;
         template->screenPos--;
      }
      mvwaddch (template->field, 0, template->screenPos, template->overlay[template->platePos] | template->fieldattr);
      wmove (template->field, 0, template->screenPos);
   }
   else
   {
      /* To be safe, we should cast the chtype to a char		*/
      char newchar	= (char)input;

      /* Make sure we don't go out of bounds.				*/
      if (template->platePos >= template->fieldwidth)
      {
         Beep();
         return;
      }

      /* We should check to see if what they typed in matches		*/
      /* what the plate states.						*/
      if (	isdigit(newchar) && ( \
		template->plate[template->platePos] == 'A' || \
		template->plate[template->platePos] == 'C' || \
		template->plate[template->platePos] == 'c') )
      {
         Beep();
         return;
      }
      if (	!isdigit(newchar) && template->plate[template->platePos] == '#' )
      {
         Beep();
         return;
      }

      /* Do we need to convert the case???				*/
      if ( template->plate[template->platePos] == 'C' || template->plate[template->platePos] == 'X')
      {
         newchar	= toupper ( (char)input );
      }
      else if ( template->plate[template->platePos] == 'c' || template->plate[template->platePos] == 'x')
      {
         newchar	= tolower ( (char)input );
      }

      /* Add it and redisplay....					*/
      template->info[template->infoPos++]	= newchar;
      template->info[template->infoPos]		= '\0';
      mvwaddch (template->field, 0, template->screenPos, newchar | template->fieldattr);
      template->platePos++;
      template->screenPos++;

      /* Move the cursor						*/
      while (	template->plate[template->platePos] != '#' &&
		template->plate[template->platePos] != 'A' &&
		template->plate[template->platePos] != 'C' &&
		template->plate[template->platePos] != 'c' &&
		template->plate[template->platePos] != 'M' &&
		template->plate[template->platePos] != 'X' &&
		template->plate[template->platePos] != 'x'  &&
		template->platePos < template->fieldwidth )
      {
         template->platePos++;
         template->screenPos++;
      }
      wmove (template->field, 0, template->screenPos);
      wrefresh (template->field);
   }
}

/*
 * This takes the overlay and the info, and mixes the two, for a
 * 'mixed' character string...
 */
char *mixCDKTemplate (CDKTEMPLATE *template)
{
   /* Reconstruct the info.						*/
   char *mixedString	= (char *)NULL;
   char *originalString	= (char *)NULL;
   int infolen		= 0;
   int x		= 0;
   int pos		= 0;

   /* Get the chtype * overlay pointer and convert it to a char *	*/
   originalString = chtype2Char (template->overlay);
   infolen = strlen (originalString);
   
   /* Create a char * pointer.						*/
   mixedString		= (char *)malloc (sizeof(char) * (infolen+3));
   cleanChar (mixedString, infolen+3, '\0');

   /* Start copying.							*/
   while (pos < infolen)
   {
      if (	template->plate[x] == '#' || \
		template->plate[x] == 'A' || \
		template->plate[x] == 'C' || \
		template->plate[x] == 'c' || \
		template->plate[x] == 'M' || \
		template->plate[x] == 'X' || \
		template->plate[x] == 'x')
      {
         mixedString[x++]	= template->info[pos++];
      }
      else
      {
         mixedString[x]		= originalString[x];
         x++;
      }
   }

   /* Clean up any memory.						*/
   freeChar (originalString);

   /* Return the new string.						*/
   return (mixedString);
}

/*
 * This takes a mixed string and returns a non-mixed string.
 */
char *unmixCDKTemplate (CDKTEMPLATE *template, char *info)
{
   /* Reconstruct the info.						*/
   char *unmixedString	= (char *)NULL;
   int infolen		= strlen (info);
   int x		= 0;
   int pos		= 0;

   /* Create a char * pointer.						*/
   unmixedString	= (char *)malloc (sizeof(char) * (infolen+2));
   cleanChar (unmixedString, infolen+2, '\0');

   /* Start copying.							*/
   while (pos < infolen)
   {
      if (	template->plate[pos] == '#' || \
		template->plate[pos] == 'A' || \
		template->plate[pos] == 'C' || \
		template->plate[pos] == 'c' || \
		template->plate[pos] == 'M' || \
		template->plate[pos] == 'X' || \
		template->plate[pos] == 'x')
      {
         unmixedString[x++] = info[pos++];
      }
      else
      {
         pos++;
      }
   }

   /* Return the new string.						*/
   return (unmixedString);
}

/*
 * This function draws
 */
void drawCDKTemplate (CDKTEMPLATE *template, boolean Box)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Keep the box information.						*/
   template->box	= Box;

   /* Erase the old object...						*/
   eraseCDKTemplate (template);

   /* Do we need to draw the shadow.					*/
   if (template->shadow)
   {
      drawShadow (template->shadowwin);
   }

   /* Box it if needed							*/
   if (Box)
   {
      box (template->win, ACS_VLINE, ACS_HLINE);
   }
   wrefresh (template->win);

   /* Draw in the label and the entry object.				*/
   writeChtype (template->labelwin, 0, 0, template->label, HORIZONTAL, 0, template->labellen);
   wrefresh (template->labelwin);

   /* Draw in the template...						*/
   if (template->overlay != (chtype *)NULL)
   {
      writeChtype (template->field, 0, 0, template->overlay, HORIZONTAL, 0, template->overlaylen);
   }

   if (template->info != (char *) NULL)
   {
      int infolen	= strlen (template->info);
      int pos	= 0;
      for (x=0; x < template->fieldwidth; x++)
      {
         if ((	template->plate[x] == '#' ||
			template->plate[x] == 'A' ||
			template->plate[x] == 'C' ||
			template->plate[x] == 'c' ||
			template->plate[x] == 'M' ||
			template->plate[x] == 'X' ||
			template->plate[x] == 'x') &&
			pos < infolen)
         {
            mvwaddch (template->field, 0, x, template->info[pos++] );
         }
      }
      wmove (template->field, 0, infolen);
   }
   wrefresh (template->field);
}

/*
 * This function destroys
 */
void destroyCDKTemplate (CDKTEMPLATE *template)
{
   /* Erase the object.							*/
   eraseCDKTemplate (template);

   /* Clear out the character pointers.					*/
   freeChtype (template->label);
   freeChtype (template->overlay);
   freeChar (template->plate);
   freeChar (template->info);

   /* Delete the windows.						*/
   delwin (template->win);
   delwin (template->field);
   delwin (template->labelwin);
   if (template->shadow)
   {
      delwin (template->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vTEMPLATE, template);

   /* Finish cleaning up.						*/
   free (template);
}

/*
 * This function erases
 */
void eraseCDKTemplate (CDKTEMPLATE *template)
{
   werase (template->field);
   werase (template->labelwin);
   werase (template->win);
   wrefresh (template->win);
   if (template->shadow)
   {
      werase (template->shadowwin);
      wrefresh (template->shadowwin);
   }
}

/*
 * This function sets the value given to the template.
 */
void setCDKTemplate (CDKTEMPLATE *template, char *newValue, boolean box)
{
   /* Declare local variables.						*/
   int len		= 0;
   int copychars	= 0;

   /* Keep some variables.						*/
   template->box	= box;

   /* Just to be sure, if lets make sure the new value isn't NULL.	*/
   if (newValue == (char *)NULL)
   { 
      /* Then we want to just erase the old value.			*/
      cleanChar (template->info, template->fieldwidth, '\0');
      return;
   }

   /* Determine how many characters we need to copy.			*/
   len		= strlen (newValue);
   copychars	= (len < template->fieldwidth ? len : template->fieldwidth);

   /* OK, erase the old value, and copy in the new value.		*/
   cleanChar (template->info, template->fieldwidth, '\0');
   strncpy (template->info, newValue, copychars);
}

/*
 * This erases the information in the template widget.
 */
void cleanCDKTemplate (CDKTEMPLATE *template)
{
   cleanChar (template->info, template->fieldwidth, '\0');
}
