#include "cdk.h"

/*
 * $Author: glover $
 * $Date: 1995/12/29 18:49:35 $
 * $Revision: 1.46 $
 */

/* Make local global vars.						*/
#define		DOWN		0
#define		UP		1
static char *	SearchPattern	= (char *)NULL;
int		SearchDirection	= DOWN;

/* Declare local functions. This kinda makes 'em 'private'		*/
int searchForWord (CDKVIEWER *viewer, char *pattern, int direction);
int jumpToLine (CDKVIEWER *viewer);
void popUpLabel (CDKVIEWER *viewer, char *mesg);
void getAndStorePattern (CDKSCREEN *screen, int width);

/*
 * This function creates a new viewer object.
 */
CDKVIEWER *newCDKViewer (CDKSCREEN *cdkscreen, int xplace, int yplace, int height, int width, char **buttons, int buttonCount, boolean box, boolean shadow)
{
   /* Declare local variables.						*/
   CDKVIEWER *viewer	= (CDKVIEWER *)malloc (sizeof (CDKVIEWER));
   int boxWidth		= width;
   int boxHeight	= height + 3;
   int xpos		= xplace;
   int ypos		= yplace;
   int buttonWidth	= 0;
   int buttonAdj	= 0;
   int buttonPos	= 1;
   int x		= 0;

   /* Rejustify the x and y positions if we need to.			*/
   alignxy (cdkscreen->window, &xpos, &ypos, boxWidth, boxHeight);

   /* Make the viewer window						*/
   viewer->win		= newwin (boxHeight, boxWidth+1, ypos, xpos);

   /* Is the window NULL?						*/
   if (viewer->win == (WINDOW *)NULL)
   {
      /* Clean up any used memory.					*/
      free (viewer);

      /* Return a NULL pointer.						*/
      return ( (CDKVIEWER *)NULL );
   }

   /* Turn the keypad on for the viewer.				*/
   keypad (viewer->win, TRUE);

   /* Create the buttons						*/
   viewer->buttonCount		= buttonCount;
   for (x=0; x < buttonCount; x++)
   {
      viewer->button[x]		= char2Chtype (buttons[x], &viewer->buttonlen[x], &buttonAdj);
      buttonWidth		+= viewer->buttonlen[x] + 1;
   }
   buttonAdj	= (int)((boxWidth-buttonWidth)/ (buttonCount +1));
   buttonPos	= 1 + buttonAdj;
   for (x=0; x < buttonCount; x++)
   {
      viewer->buttonpos[x]	= buttonPos;
      buttonPos			+= buttonAdj + viewer->buttonlen[x];
   }

   /* Set the rest of the variables					*/
   viewer->parent		= cdkscreen->window;
   viewer->shadowwin		= (WINDOW *)NULL;
   viewer->title		= char2Chtype ("</U>*** Untitled ***<!U>", &viewer->titlelen, &viewer->titlepos);
   viewer->titlepos		= justifyString (boxWidth, viewer->titlelen, viewer->titlepos);
   viewer->buttonHighlight	= A_REVERSE;
   viewer->boxHeight		= boxHeight;
   viewer->boxWidth		= boxWidth - 2;
   viewer->viewSize		= height - 2;
   viewer->box			= box;
   viewer->shadow		= shadow;
   viewer->currentButton	= 0;
   viewer->currentTop		= 0;
   viewer->length		= 0;
   viewer->leftChar		= 0;
   viewer->maxLeftChar		= 0;
   viewer->maxTopLine		= 0;
   viewer->characters		= 0;

   /* Do we need to create a shadow???					*/
   if (shadow)
   {
      viewer->shadowwin		= newwin (boxHeight, boxWidth+1, ypos+1, xpos+1);
   }

   /* Clean the key bindings						*/
   cleanCDKObjectBindings (vVIEWER, viewer);

   /* Register this baby.						*/
   registerCDKObject (cdkscreen, vVIEWER, viewer);

   /* Return the viewer list						*/
   return (viewer);
}

/*
 * This allows the user to scroll through the info.
 */
int activateCDKViewer (CDKVIEWER *viewer, char *title, char **info, int infoSize, chtype buttonHighlight, boolean attrInterp, boolean box)
{
   /* Declare local variables.						*/
   int widestLine		= -1;
   int x			= 0;
   int y			= 0;

   /* Keep some semi-perinant info					*/
   viewer->box			= box;
   viewer->infoSize		= infoSize;
   viewer->interpret		= attrInterp;
   viewer->buttonHighlight	= buttonHighlight;

   /* Set up the needed vars for the viewer list.    			*/
   if (infoSize <= viewer->viewSize)
   {
      viewer->maxTopLine        = -1;
   }
   else
   {
      viewer->maxTopLine        = viewer->infoSize-1;
   }

   /* Set up the title of the viewer.					*/
   if (title != (char *)NULL)
   {
      freeChtype (viewer->title);
      viewer->title		= char2Chtype (title, &viewer->titlelen, &viewer->titlepos);
      viewer->titlepos		= justifyString (viewer->boxWidth, viewer->titlelen, viewer->titlepos);
   }

   /* Did they ask for attribute interpretation???			*/
   if (!attrInterp)
   {
      /* Copy the information given, without interpretation.		*/
      for (x=0; x <= infoSize; x++)
      {
         if (info[x] == (char *) NULL)
         {
            viewer->info[x]	= (chtype *)NULL;
            viewer->infolen[x]	= 0;
            viewer->infopos[x]	= 0;
         }
         else 
         {
            int len		= strlen (info[x]);
            widestLine		= MAX(widestLine,len);

            /* Init memory and clean it.				*/
            viewer->info[x]	= (chtype *)malloc (sizeof(chtype) * (len+3));
            cleanChtype (viewer->info[x], len+3, '\0');

            /* Copy from one to the other.				*/
            for (y=0; y < len; y++)
            {
               viewer->info[x][y]	= info[x][y] | A_NORMAL;
            }
            viewer->infolen[x]	= len;
            viewer->infopos[x]	= 0;
            viewer->characters	+= len;
         }
      }
   }
   else
   {
      /* Copy the information given, with interpretation.		*/
      for (x=0; x <= infoSize; x++)
      {
         if (info[x] == (char *)NULL)
         {
            viewer->info[x]	= (chtype *)NULL;
            viewer->infolen[x]	= 0;
            viewer->infopos[x]	= 0;
         }
         else
         {
            viewer->info[x]	= char2Chtype (info[x], &viewer->infolen[x], &viewer->infopos[x]);
            viewer->infopos[x]	= justifyString (viewer->boxWidth, viewer->infolen[x], viewer->infopos[x]);
            widestLine		= MAX(widestLine,viewer->infolen[x]);
            viewer->characters	+= viewer->infolen[x];
         }
      }
   }
   
   /* Determine how many characters we can shift to the right		*/
   /* before all the items have been viewer off the screen.		*/
   if ( widestLine > viewer->boxWidth )
   {
      viewer->maxLeftChar	= (widestLine - viewer->boxWidth);
   }
   else
   {
      viewer->maxLeftChar	= 0;
   }

   /* Now lets manage this thingy...					*/
   return (manageCDKViewer (viewer));
}

/*
 * This function actually controls the vewer...
 */
int manageCDKViewer (CDKVIEWER *viewer)
{
   /* Declare local variables.						*/
   CDKLABEL *fileInfoLabel;
   chtype input;
   char *fileInfo[10], temp[500];
   int x ,REFRESH;

#ifdef NODYNALLOC
   chtype emptyString[256];
#else
   chtype emptyString[viewer->boxWidth];
#endif

   /* Create a string full of spaces.					*/
   cleanChtype (emptyString, viewer->boxWidth-1, '\0');
   cleanChtype (emptyString, viewer->boxWidth-3, ' ');

   /* Create the information about the file stats.			*/
   sprintf (temp, "<C></5>  </U>Viewer File Statistics<!U>  <!5>");	fileInfo[0] = copyChar(temp);
   sprintf (temp, "<C></5>                          <!5>");	fileInfo[1] = copyChar(temp);
   sprintf (temp, "</5/R>Character Count:<!R> %-4d     <!5>", viewer->characters);	fileInfo[2] = copyChar(temp);
   sprintf (temp, "</5/R>Line Count     :<!R> %-4d     <!5>", viewer->infoSize);	fileInfo[3] = copyChar(temp);
   sprintf (temp, "<C></5>                          <!5>");	fileInfo[4] = copyChar(temp);
   sprintf (temp, "<C></5>Press Any Key To Continue.<!5>");	fileInfo[5] = copyChar(temp);

   /* Set the current button.						*/
   viewer->currentButton	= 0;

   /* Draw the viewer list						*/
   drawCDKViewer (viewer, viewer->box);

   /* Do this until KEY_RETURN is hit.					*/
   for (;;)
   {
      /* Reset the refresh flag.					*/
      REFRESH	= FALSE;

      /* Get the user input.						*/
      input = wgetch (viewer->win);
      if ( ! checkCDKObjectBind (vVIEWER, viewer, input) )
      {
         switch (input)
         {
            case KEY_TAB : 
	         if (viewer->buttonCount > 1)
                 {
                    if (viewer->currentButton == viewer->buttonCount -1 )
                    {
                       viewer->currentButton	= 0;
                    }
                    else
                    {
                       viewer->currentButton++;
                    }
                    REFRESH	= TRUE;
                 }
                 break;
   
            case KEY_UP :
                 if (viewer->currentTop > 0)
                 {
                    viewer->currentTop--;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_DOWN :
                 if (viewer->currentTop < viewer->maxTopLine)
                 {
                    viewer->currentTop++;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_RIGHT :
                 if (viewer->leftChar < viewer->maxLeftChar)
                 {
                    viewer->leftChar ++;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_LEFT :
                 if (viewer->leftChar > 0)
                 {
                    viewer->leftChar --;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_PPAGE : case '' : case 'b' : case 'B' :
                 if (viewer->currentTop > 0)
                 {
                    if ( (viewer->currentTop - (viewer->viewSize-1)) > 0)
                    {
                       viewer->currentTop	= viewer->currentTop - (viewer->viewSize - 1);
                    }
                    else
                    {
                       viewer->currentTop 	= 0;
                    }
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_NPAGE : case '' : case ' ' : case 'f' : case 'F' :
                 if ( viewer->currentTop < viewer->maxTopLine)
                 {
                    if ( (viewer->currentTop + viewer->viewSize) < viewer->maxTopLine )
                    {
                       viewer->currentTop	= viewer->currentTop + (viewer->viewSize - 1);
                    }
                    else
                    {
                       viewer->currentTop	= viewer->maxTopLine;
                    }
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case KEY_HOME : case '|' :
                 viewer->leftChar = 0;
                 REFRESH	= TRUE;
                 break;
   
            case KEY_END : case '$' :
                 viewer->leftChar = viewer->maxLeftChar;
                 REFRESH	= TRUE;
                 break;
   
            case 'g' : case '1' :
                 viewer->currentTop = 0;
                 REFRESH	= TRUE;
                 break;
   
            case 'G' : 
                 viewer->currentTop = viewer->maxTopLine;
                 REFRESH	= TRUE;
                 break;
   
            case 'L' :
                 x = (int) ( (viewer->infoSize + viewer->currentTop) / 2);
                 if ( x < viewer->maxTopLine)
                 {
                    viewer->currentTop = x;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case 'l' :
                 x = (int) (viewer->currentTop / 2);
                 if ( x >= 0 )
                 {
                    viewer->currentTop = x;
                    REFRESH	= TRUE;
                 }
                 else
                 {
                    Beep();
                 }
                 break;
   
            case '?' :
                 SearchDirection	= UP;
                 getAndStorePattern (viewer->screen, viewer->boxWidth-2);
                 if ( ! searchForWord(viewer, SearchPattern, SearchDirection))
                 {
                    sprintf (temp, "</U/5>Pattern '%s' not found.<!U!5>", SearchPattern);
                    popUpLabel (viewer, temp);
                 }
                 REFRESH	= TRUE;
                 break;
   
            case '/' :
                 SearchDirection	= DOWN;
                 getAndStorePattern (viewer->screen, viewer->boxWidth-2);
                 if ( ! searchForWord(viewer, SearchPattern, SearchDirection))
                 {
                    sprintf (temp, "</U/5>Pattern '%s' not found.<!U!5>", SearchPattern);
                    popUpLabel (viewer, temp);
                 }
                 REFRESH	= TRUE;
                 break;
   
            case 'n' :
                 if ( SearchPattern == (char *)NULL )
                 {
                    popUpLabel (viewer, "</5>There is no pattern in the buffer.<!5>");
                 }
                 else
                 {
                    if ( ! searchForWord(viewer, SearchPattern, SearchDirection))
                    {
                       sprintf (temp, "</5>Pattern '%s' not found.<!5>", SearchPattern);
                       popUpLabel (viewer, temp);
                    }
                 }
                 REFRESH	= TRUE;
                 break;
   
            case ':' :
                 viewer->currentTop	= jumpToLine (viewer);
                 REFRESH		= TRUE;
                 break;
   
            case 'i' : case 's' :
                 fileInfoLabel	= newCDKLabel (viewer->screen, CENTER, CENTER, fileInfo, 6, TRUE, FALSE);
                 drawCDKLabel (fileInfoLabel, BOX);
                 wgetch (fileInfoLabel->win);
                 destroyCDKLabel (fileInfoLabel);
                 REFRESH		= TRUE;
                 break;
   
            case KEY_RETURN : case CDK_NEXT :
                 freeChar (fileInfo[0]);
                 freeChar (fileInfo[1]);
                 freeChar (fileInfo[2]);
                 freeChar (fileInfo[3]);
                 freeChar (fileInfo[4]);
                 freeChar (fileInfo[5]);
                 return (viewer->currentButton);
                 break;
   
            case CDK_REFRESH :
                 refreshCDKScreen (viewer->screen);
                 break;
   
            default :
                 break;
         }
      }
      
      /* Do we need to redraw the screen???				*/
      if (REFRESH)
      {
         /* Clear the window.						*/
         werase (viewer->win);

         /* Redraw the title.						*/
         writeChtype (viewer->win, viewer->titlepos, 1, viewer->title, HORIZONTAL, 0, viewer->titlelen);

         /* Redraw the list						*/
         for (x=0; x < viewer->viewSize; x++)
         {
            if ( viewer->currentTop + x <= viewer->infoSize)
            {
               writeChtype (viewer->win, viewer->infopos[viewer->currentTop+x]+1, x+2, viewer->info[x+viewer->currentTop], HORIZONTAL, viewer->leftChar, viewer->infolen[x+viewer->currentTop]);
            }
         }
     
         /* Redraw the buttons.						*/
         for (x=0; x < viewer->buttonCount; x++)
         {
            writeChtype (viewer->win, viewer->buttonpos[x], viewer->boxHeight-2, viewer->button[x], HORIZONTAL, 0, viewer->buttonlen[x]);
         }

         /* Draw in the current line at the top. 			*/
         sprintf (temp, "%d/%d %2.0f%%", (viewer->currentTop+1), viewer->infoSize, ((float)(viewer->currentTop+1)/(float)viewer->infoSize) * 100);
         writeChar (viewer->win, 1, 1, temp, HORIZONTAL, 0, strlen(temp));
     
         /* Box it if we have to.					*/
         if (viewer->box)
         {
            box (viewer->win, ACS_VLINE, ACS_HLINE);
         }

         /* Draw the seperation line.					*/
         for (x=1; x <= viewer->boxWidth+1; x++)
         {
            mvwaddch (viewer->win, viewer->boxHeight-3, x, ACS_HLINE);
         }
         mvwaddch (viewer->win, viewer->boxHeight-3, 0, ACS_LTEE);
         mvwaddch (viewer->win, viewer->boxHeight-3, viewer->win->_maxx, ACS_RTEE);

         /* Refresh the screen.						*/
         wrefresh (viewer->win);
      }
   }
}

/*
 * This searches the document looking for the given word.
 */
void getAndStorePattern (CDKSCREEN *screen, int width)
{
   /* Declare local variables.						*/
   CDKENTRY	*getPattern;
   char		*temp;

   /* Check the direction.						*/
   if (SearchDirection == UP)
   {
      temp	= "</5>Search Up  : <!5>";
   }
   else
   {
      temp	= "</5>Search Down: <!5>";
   }

   /* Pop up the entry field.						*/
#ifdef COLOR
   getPattern	= newCDKEntry (screen, CENTER, CENTER, LEFT, temp, COLOR_PAIR(5)|A_BOLD, '.'|COLOR_PAIR(5)|A_BOLD, vMIXED, 10, 0, 256, &CDKEntryCallBack, TRUE, FALSE);
#else
   getPattern	= newCDKEntry (screen, CENTER, CENTER, LEFT, temp, A_BOLD, '.'|A_BOLD, vMIXED, 10, 0, 256, &CDKEntryCallBack, TRUE, FALSE);
#endif


   /* Is there an old serach pattern?					*/
   if (SearchPattern != (char *)NULL) 
   {
      setCDKEntry (getPattern, SearchPattern, getPattern->min, getPattern->max, getPattern->box);
   }
   freeChar (SearchPattern);

   /* Activate this baby.						*/
   activateCDKEntry (getPattern);

   /* Save the info.							*/
   SearchPattern	= copyChar (getPattern->info);

   /* Clean up.								*/
   destroyCDKEntry (getPattern);
}

/*
 * This searches for the word and realigns the value on the screen.
 */
int searchForWord (CDKVIEWER *viewer, char *pattern, int direction)
{
   /* Declare local variables.						*/
   int x, y, pos, len, plen;

   /* If the pattern is NULL then return.				*/
   if (pattern == (char *)NULL)
   {
      return(0);
   }
   plen	= strlen(pattern);

   /* Given the direction, start looking....				*/
   if (direction == DOWN)
   {
      /* Start looking from 'here' down.				*/
      for (x = viewer->currentTop + 1; x < viewer->infoSize; x++)
      {
         /* Start looking. If we find it, then set the value of		*/
         /* viewer->currentTop and possibly even leftChar...		*/
         len	= chlen (viewer->info[x]);
         pos	= 0;
         for (y=0; y < len; y++)
         {
            /* We have to tear the attributes from the chtype.		*/
            char plainChar	= viewer->info[x][y] & A_CHARTEXT;

            /* We have found the word at this point.			*/
            if (pos == plen)
            {
               viewer->currentTop	= (x < viewer->maxTopLine  ? x : viewer->maxTopLine);
               viewer->leftChar	= (y < viewer->boxWidth ? 0 : viewer->maxLeftChar);
               return (1);
            }

            /* Keep trudging along.					*/
            if (pattern[pos++] != plainChar)
            {
               pos	= 0;
            }
         }
      }
   }
   else
   {
      /* Start looking from 'here' up.					*/
      for (x = viewer->currentTop - 1; x >= 0; x--)
      {
         /* Start looking. If we find it, then set the value of		*/
         /* viewer->currentTop and possibly even leftChar...		*/
         len	= chlen (viewer->info[x]);
         pos	= 0;
         for (y=0; y < len; y++)
         {
            /* We have to tear the attributes from the chtype.		*/
            char plainChar	= viewer->info[x][y] & A_CHARTEXT;

            /* We have found the word at this point.			*/
            if (pos == plen)
            {
               viewer->currentTop	= x;
               viewer->leftChar	= (y < viewer->boxWidth ? 0 : viewer->maxLeftChar);
               return (1);
            }

            /* Keep trudging along.					*/
            if (pattern[pos++] != plainChar)
            {
               pos	= 0;
            }
         }
      }
   }
   return(0);
}

/*
 * This allows us to 'jump' to a given line in the file.
 */
int jumpToLine (CDKVIEWER *viewer)
{
   /* Declare local variables.						*/
   int line		= 0;
   CDKSCALE * newline	= newCDKScale (viewer->screen, CENTER, CENTER, LEFT,
				"</5>Go To Line :", A_BOLD, 5,
				0, 0, viewer->maxTopLine+1, 1, 10, TRUE, TRUE);
   line			= activateCDKScale (newline);
   destroyCDKScale (newline);
   return ( (line-1) );
}

/*
 * This pops a little message up on the screen.
 */
void popUpLabel (CDKVIEWER *viewer, char *mesg)
{
   /* Declare local variables.						*/
   char *info[3];
   CDKLABEL *label;

   /* Set up variables.							*/
   info[0]	= mesg;
   label	= newCDKLabel (viewer->screen, CENTER, CENTER, info, 1, TRUE, FALSE);

   /* Draw the label and wait.						*/
   drawCDKLabel (label, BOX);
   wgetch (label->win);

   /* Clean up.								*/
   destroyCDKLabel (label);
}

/*
 * This function draws the viewer widget.
 */
void drawCDKViewer (CDKVIEWER *viewer, boolean Box)
{
   /* Declare local variables.						*/
   int x	= 0;
   char temp[100];

   /* Erase the old object...						*/
   eraseCDKViewer (viewer);

   /* Do we need to draw in the shadow???				*/
   if (viewer->shadow)
   {
      drawShadow (viewer->shadowwin);
   }
   
   /* Draw in the title							*/
   writeChtype (viewer->win, viewer->titlepos, 1, viewer->title, HORIZONTAL, 0, viewer->titlelen);

   /* Draw in the message contents.					*/
   for (x=0; x < viewer->viewSize; x++)
   {
      if (viewer->info[viewer->currentTop+x] != (chtype *)NULL)
      {
         writeChtype (viewer->win, viewer->infopos[viewer->currentTop+x]+1, x+2, viewer->info[x+viewer->currentTop], HORIZONTAL, viewer->leftChar, viewer->infolen[x+viewer->currentTop]);
      }
   }

   /* Box it if it was asked for.					*/
   if (viewer->box)
   {
      box (viewer->win, ACS_VLINE, ACS_HLINE);
   }

   /* Draw the seperation line.						*/
   for (x=1; x <= viewer->boxWidth+1; x++)
   {
      mvwaddch (viewer->win, viewer->boxHeight-3, x, ACS_HLINE);
   }
   mvwaddch (viewer->win, viewer->boxHeight-3, 0, ACS_LTEE);
   mvwaddch (viewer->win, viewer->boxHeight-3, viewer->win->_maxx, ACS_RTEE);

   /* Draw in the buttons						*/
   for (x=0; x < viewer->buttonCount; x++)
   {
      writeChtype (viewer->win, viewer->buttonpos[x], viewer->boxHeight-2, viewer->button[x], HORIZONTAL, 0, viewer->buttonlen[x]);
   }
   for (x=0; x < viewer->buttonlen[viewer->currentButton]; x++)
   {
      char character	= viewer->button[viewer->currentButton][x] & A_CHARTEXT;
      mvwaddch (viewer->win, viewer->boxHeight-2, viewer->buttonpos[viewer->currentButton]+x, character|viewer->buttonHighlight);
   }
   
   /* Draw in the current line at the top. 				*/
   sprintf (temp, "%d/%d %2.0f%%", (viewer->currentTop+1), viewer->infoSize, ((float)(viewer->currentTop+1)/(float)viewer->infoSize) * 100);
   writeChar (viewer->win, 1, 1, temp, HORIZONTAL, 0, strlen(temp));

   /* Refresh the window.						*/
   wrefresh (viewer->win);
}

/*
 * This function destroys the viewer widget.
 */
void destroyCDKViewer (CDKVIEWER *viewer)
{
   /* Declare local variables.						*/
   int x	= 0;

   /* Erase the object.							*/
   eraseCDKViewer (viewer);

   /* Clear up the char pointers.					*/
   freeChtype (viewer->title);
   for (x=0; x <= viewer->infoSize; x++)
   {
      freeChtype (viewer->info[x]);
   }
   for (x=0; x < viewer->buttonCount; x++)
   {
      freeChtype (viewer->button[x]);
   }

   /* Clean up the windows.						*/
   delwin (viewer->win);
   if (viewer->shadow)
   {
      delwin (viewer->shadowwin);
   }

   /* Unregister this object.						*/
   unregisterCDKObject (vVIEWER, viewer);

   /* Finish cleaning up.						*/
   free (viewer);
}

/*
 * This function erases the viewer widget from the screen.
 */
void eraseCDKViewer (CDKVIEWER *viewer)
{
   werase (viewer->win);
   wclear (viewer->win);
   wrefresh (viewer->win);
   if (viewer->shadow)
   {
      werase (viewer->shadowwin);
     wrefresh (viewer->shadowwin);
   }
}

/*
 * This sets certain attributes of the viewer...
 */
void setCDKViewer (CDKVIEWER *viewer, chtype buttonHighlight, boolean box)
{
   /* Keep some semi-perinant info					*/
   viewer->box			= box;
   viewer->buttonHighlight	= buttonHighlight;
}
